HedgewarsScriptLoad("/Scripts/SimpleMission.lua")
HedgewarsScriptLoad("/Scripts/Locale.lua")
SimpleMission({
	missionTitle = loc("The Great Escape"),
	goalText = loc("Eliminate your captor.") .."|".. loc("Unlimited Attacks: Attacks don't end your turn"),
	missionIcon = -amGrenade,
	wind = 100,
	initVars = {
		TurnTime = 45000,
		Map = "Castle",
		Theme = "Nature",
		MinesTime = 0,
		Seed = 0,
		GameFlags = gfInfAttack + gfDisableWind,
	},
	ammoConfig = {
		[amGrenade] = { count = 1 },
		[amParachute] = { count = 1 },
		[amFirePunch] = { ammoInCrate = 3 },
	},
	teams = {
		{ isMissionTeam = true,
		hogs = {{ health = 1, x = 1050, y = 534 }},
		},
		{ name = loc("Bad Team"),
		flag = "cm_dragonrb",
		grave = "dragonball",
		hogs = {{ name = loc("Bad Guy"), health = 1, x = 1512, y = 158, botLevel = 1 }}
		},
	},
	girders = {
		{ x = 1042, y = 564, frameIdx = 0 },
		{ x = 1028, y = 474, frameIdx = 6 },
		{ x = 1074, y = 474, frameIdx = 6 },
		{ x = 1050, y = 385, frameIdx = 0 },
		{ x = 1175, y = 731, frameIdx = 7 },
		{ x = 1452, y = 905, frameIdx = 2 },
		{ x = 1522, y = 855, frameIdx = 4 },
		{ x = 1496, y = 900, frameIdx = 3 },
		{ x = 1682, y = 855, frameIdx = 4 },
		{ x = 1773, y = 887, frameIdx = 2 },
		{ x = 1647, y = 901, frameIdx = 1 },
		{ x = 1871, y = 883, frameIdx = 6 },
		{ x = 1871, y = 723, frameIdx = 6 },
		{ x = 1774, y = 768, frameIdx = 6 },
		{ x = 1773, y = 767, frameIdx = 6 },
		{ x = 1821, y = 904, frameIdx = 1 },
		{ x = 1822, y = 802, frameIdx = 3 },
		{ x = 1820, y = 723, frameIdx = 1 },
		{ x = 1782, y = 678, frameIdx = 4 },
		{ x = 1822, y = 661, frameIdx = 0 },
		{ x = 1822, y = 644, frameIdx = 0 },
		{ x = 1742, y = 644, frameIdx = 0 },
		{ x = 1742, y = 661, frameIdx = 0 },
		{ x = 1694, y = 676, frameIdx = 2 },
		{ x = 1903, y = 635, frameIdx = 0 },
	},
	gears = {
		{ type = gtMine, x = 1010, y = 680 },
		{ type = gtMine, x = 1031, y = 720 },
		{ type = gtMine, x = 1039, y = 748 },
		{ type = gtMine, x = 1051, y = 777 },
		{ type = gtMine, x = 1065, y = 796 },

		{ type = gtCase, crateType = "supply", x = 1632, y = 943, ammoType = amShotgun },
		{ type = gtCase, crateType = "supply", x = 1723, y = 888, ammoType = amFirePunch },
		{ type = gtCase, crateType = "supply", x = 1915, y = 599, ammoType = amGrenade },
		{ type = gtCase, crateType = "supply", x = 1416, y = 913, ammoType = amBlowTorch },
		{ type = gtCase, crateType = "supply", x = 1227, y = 640, ammoType = amPickHammer },
		{ type = gtCase, crateType = "supply", x = 1519, y = 945, ammoType = amParachute},

		{ type = gtCase, crateType = "health", x = 1476, y = 169 },
		{ type = gtCase, crateType = "health", x = 1551, y = 177 },
		{ type = gtCase, crateType = "health", x = 1586, y = 200 },
		{ type = gtCase, crateType = "health", x = 1439, y = 189 },
		{ type = gtCase, crateType = "health", x = 1401, y = 211 },
		{ type = gtCase, crateType = "health", x = 1633, y = 210 },
	},
})
