locale = {
    ["!!!"] = "!!!",
    ["..."] = "...",
    ["011101000"] = "011101000", -- A_Classic_Fairytale:dragon
    ["011101001"] = "011101001", -- A_Classic_Fairytale:backstab, A_Classic_Fairytale:dragon, A_Classic_Fairytale:enemy, A_Classic_Fairytale:family, A_Classic_Fairytale:journey, A_Classic_Fairytale:queen, A_Classic_Fairytale:shadow, A_Classic_Fairytale:united
    ["10 weapon schemes"] = "10 schematów broni", -- Continental_supplies
    ["15+%d damage, %d invulnerable left"] = "15+%d obrażeń, pozostało %d niezniszczalnych", -- Continental_supplies
    ["1-5, Precise + 1-4: Choose structure type"] = "1-5, Precyzja + 1-4: Wybierz typ struktury", -- Construction_Mode
    ["+1 barrel!"] = "+1 beczka!", -- Tumbler
    ["%.1fs"] = "%.1fs", -- Racer, TechRacer
    ["%.1f seconds were remaining."] = "%.1f sekund pozostało.", -- Basic_Training_-_Bazooka
    ["+1 Grenade"] = "+1 granat", -- Basic_Training_-_Flying_Saucer
    ["+1 mine!"] = "+1 mina!", -- Tumbler
    ["+1 point"] = "+1 punkt", -- Mutant
    ["-1 point"] = "-1 punkt", -- Mutant
    ["-1 to anyone for a suicide"] = "-1 dla każdego za samobójstwo", -- Mutant
    ["+1 to the Bottom Feeder for killing anyone"] = "+1 dla Pasożyta za zabicie kogokolwiek", -- Mutant
    ["+1 to the Mutant for killing anyone"] = "+1 dla Mutanta za zabicie kogokolwiek", -- Mutant
    ["+2 for becoming the Mutant"] = "+2 za zostanie Mutantem", -- Mutant
    ["30 minutes later..."] = "30 minut później...", -- A_Classic_Fairytale:shadow
    ["%.3fs"] = "%.3fs", -- A_Space_Adventure:ice02
    ["5 additional enemies will be spawned during the game."] = "5 dodatkowych wrogów pojawi się podczas gry.", -- A_Space_Adventure:fruit01
    ["5 Deadly Hogs"] = "5 Zabójczych Jeży", -- A_Space_Adventure:death02
    ["6 more seconds added to the clock"] = "6 sekund dodano do zegara", -- A_Space_Adventure:ice02
    ["About a month ago, a cyborg came and told us that you're the cannibals!"] = "Jakiś rok temu przybył do nas cyborg i powiedział nam, że jesteście kanibalami!", -- A_Classic_Fairytale:enemy
    ["Above-average pilot"] = "Nadprzeciętny pilot", -- User_Mission_-_RCPlane_Challenge
    ["Accuracy Bonus! +15 points"] = "Bonus za celność! +15 punktów", -- Space_Invasion
    ["Accuracy bonus: +%d points"] = "Bonus za celność! +%d punktów", -- Basic_Training_-_Sniper_Rifle
    ["Achievement gotten: %s"] = "Osiągnięcie zdobyte: %s", -- User_Mission_-_RCPlane_Challenge, User_Mission_-_That_Sinking_Feeling, User_Mission_-_Bamboo_Thicket, User_Mission_-_Dangerous_Ducklings, Basic_Training_-_Rope, Tumbler
    ["A Classic Fairytale"] = "Klasyczna bajka", -- A_Classic_Fairytale:first_blood
    ["A crate critical to this mission has been destroyed."] = "Skrzynia krytyczna dla tej misji została zniszczona.", -- SimpleMission
    ["Actually, you aren't worthy of life! Take this..."] = "Właściwie, nie zasługujesz by żyć! Przyjmij to...", -- A_Classic_Fairytale:shadow
    ["A cy-what?"] = "Cy-co?", -- A_Classic_Fairytale:enemy
    ["Add %d"] = "Dodaj %d", -- HedgeEditor
    ["Admit what?"] = "Przyznać co?", -- A_Classic_Fairytale:queen
    ["Adventurous"] = "Awanturniczy", -- A_Classic_Fairytale:journey
    ["A frenetic Hedgewars mini-game"] = "Szalona mini-gra Hedgewars", -- Frenzy
    ["Africa"] = "Afryka", -- Continental_supplies
    ["A frozen adventure"] = "Zamrożona przygoda", -- A_Space_Adventure:ice01
    ["After Leaks A Lot betrayed his tribe, he joined the cannibals..."] = "Po tym, jak Spory Przeciek zdradził swoje plemię, dołączył do kanibali...", -- A_Classic_Fairytale:first_blood
    ["After that incident he went underground and started working on his plan to steal the device."] = "Po tym incydencie, zeszedł pod ziemię i zaczął pracować nad swoim planem, by ukraść urządzenie.", -- A_Space_Adventure:moon02
    ["After the shock caused by the enemy spy, Leaks A Lot and Dense Cloud went hunting to relax."] = "Po szoku spowodowanym wrogim szpiegiem, Spory Przeciek i Gęsta Chmura poszli łowić, by się zrelaksować.", -- A_Classic_Fairytale:shadow
    ["After you killed an enemy, you'll lose the weapon that he is named after."] = "Jak zabijesz wroga, stracisz broń, po której jest nazwany.", -- A_Space_Adventure:death02
    ["After you left the moon, my other loyal minions came and resurrected me so I could complete my master plan."] = "Po tym jak opuściłeś księżyc, moi lojalni poddani przyszli i ożywili mnie, żebym mógł ukończyć mój plan.", -- A_Space_Adventure:death01
    ["Again with the 'cannibals' thing!"] = "Znowu z tymi \"kanibalami\"!", -- A_Classic_Fairytale:enemy
    ["A Hedgewars minigame"] = "Minigra Hedgewars", -- Capture_the_Flag
    ["A Hedgewars mini-game"] = "Minigra Hedgewars", -- Racer, Space_Invasion, TechRacer, Tumbler
    ["A Hedgewars tag game"] = "Berek w Hedgewars", -- Mutant
    ["Ahhh, home, sweet home. Made it in %d seconds."] = "Ahhh, nie ma to jak w domu. Udało mi się w %d sekund.", -- ClimbHome
    ["Aim at the ceiling and hold [Attack] pressed until the rope attaches."] = "Wyceluj w sufit i przytrzymaj [Atak], aż lina się zaczepi.", -- Basic_Training_-_Rope
    ["Aiming Practice"] = "Ćwiczenie celności",
    ["Aiming practice"] = "Ćwiczenie celności", -- TargetPractice
    ["Aim: [Up]/[Down]"] = "Celuj: [Góra]/[Dół]", -- Basic_Training_-_Bazooka, Basic_Training_-_Grenade, Basic_Training_-_Rope
    ["Air Attack"] = "Nalot", -- Construction_Mode
    ["Air General"] = "Powietrzny Generał", -- Battalion
    ["Air Mine Placement Mode"] = "Tryb Ustawiania Min Powietrznych", -- HedgeEditor
    ["AIR MINE PLACEMENT MODE"] = "TRYB USTAWIANIA MIN POWIETRZNYCH", -- HedgeEditor
    ["A leap in a leap"] = "Skok w skoku", -- A_Classic_Fairytale:first_blood
    ["Alex"] = "Aleks",
    ["Alien! I wish to be moved!"] = "Obcy! Życzę sobie być przeniesiona!", -- A_Classic_Fairytale:queen
    ["A little gift from the cyborgs"] = "Mały prezent od cyborgów", -- A_Classic_Fairytale:shadow
    ["Al.Kaholic"] = "Al.Kaholik",
    ["All But Last"] = "Wszyscy Oprócz Ostatniego", -- WxW
    ["All But Last: You must not solely attack the team with the least health"] = "Wszyscy Oprócz Ostatniego: Nie możesz wyłącznie atakować drużyny z najmiejszą ilością zdrowia", -- WxW
    ["All gone...everything!"] = "Wszystko przepadło... wszystko!", -- A_Classic_Fairytale:enemy
    ["Allies"] = "Sprzymierzeńcy", -- A_Space_Adventure:ice01, A_Space_Adventure:ice02
    ["All right, I'll admit it!"] = "No dobrze, przyznam to!", -- A_Classic_Fairytale:queen
    ["All right, we just need to get to the other side of the island!"] = "No dobrze, musimy tylko dostać się na drugą stronę wyspy!", -- A_Classic_Fairytale:journey
    ["All right, you got me!"] = "No dobrze, masz mnie!", -- A_Classic_Fairytale:queen
    ["All the other places are protected by our flight-inhibiting weapons."] = "Wszystkie inne miejsca są chronine przez nasze bronie przeciwlotnicze.", -- A_Space_Adventure:fruit01
    ["All the saucer pilots dream to come here one day in order to compete with the best!"] = "Wszyscy piloci spodków marzą, by przyjść tu pewnego dnia i zmierzyć się z najlepszymi!", -- A_Space_Adventure:ice02
    ["All they do is sit around and judge us!"] = "Wszystko co robią to siedzą sobie i nas osądzają!", -- A_Classic_Fairytale:queen
    ["All this to please our beloved “elders” … hick …"] = "Wszysyko to, by zadowolić naszych ukochanych \"starszych\" ... hyp ...", -- A_Classic_Fairytale:queen
    ["All walls touched!"] = "Wszystkie ściany dotknięte!", -- WxW
    ["All you do is take long walks when everyone else works."] = "Wszystko co robisz to bierzesz długie spacery, kiedy każdy inny pracuje.", -- A_Classic_Fairytale:queen
    ["All your hedgehogs must be above the marked height!"] = "Wszystkie twoje jeże muszą być powyżej zaznaczonej wysokości!", -- A_Classic_Fairytale:family
    ["Also, you should know that the only place where you can fly is the left-most part of this area."] = "Powinieneś również wiedzieć, że jedynym miejscem, w którym możesz latać, jest część tego obszaru najbardziej po lewej.", -- A_Space_Adventure:fruit01
    ["Always being considered weak and fragile."] = "Zawsze uzażany za słabego i kruchego.", -- A_Classic_Fairytale:queen
    ["Amazing! I was never beaten in a race before!"] = "Niesamowite! Nigdy wcześniej nie zostałem pokonany w wyścigu!", -- A_Space_Adventure:moon02
    ["Ammo: %d"] = "Amunicja: %d", -- Tumbler
    ["Ammo depleted!"] = "Amunicja wyczerpana!", -- Space_Invasion
    ["Ammo is reset at the end of your turn."] = "Amunicja jest resetowana przy końcu tury.",
    ["Ammo Limit: Hogs can’t have more than 1 ammo per type"] = "Limit amunicji: jeże nie mogą", -- Highlander
    ["Ammo Maniac! +5 points!"] = "Maniak amunicji! +5 punktów!", -- Space_Invasion
    ["A mysterious Box"] = "Tajemnicza skrzynia", -- Basic_Training_-_Movement
    ["And how am I alive?!"] = "Jak to ja żyję?!", -- A_Classic_Fairytale:enemy
    ["And I just forgot the checkpoint of my main mission. Great, just great!"] = "A ja właśnie zapomniałem punktu kontrolnego mojej misji. Świetnie, po prosti świetnie!", -- A_Space_Adventure:cosmos
    ["… and I think they are up to something. Something bad!"] = "... a ja myślę, że coś knują. Coś złego!", -- A_Classic_Fairytale:epil
    ["Andrey"] = "Andrzej",
    ["And so happened that Leaks A Lot failed to complete the challenge! He landed, pressured by shame ..."] = "I zdarzyło się, że Sporemu Przeciekowi nie udało się ukończyć zadania! Wylądował, pod presją wstydu...", -- A_Classic_Fairytale:first_blood
    ["And so it began..."] = "A więc się zaczeło...", -- A_Classic_Fairytale:first_blood
    ["And so the cyborgs took over the island."] = "I tak cyborgi przejęły wyspę.", -- A_Classic_Fairytale:queen
    ["...and so the cyborgs took over the world..."] = "...i tak cyborgi przejęły władzę nad światem...", -- A_Classic_Fairytale:shadow
    ["And so they discovered that cyborgs weren't invulnerable..."] = "I tak odkryli, że cyborgi nie były niezniszczalne...", -- A_Classic_Fairytale:journey
    ["… and then I took a stroll …"] = "...i wtedy udałem się na przechadzkę...", -- A_Classic_Fairytale:epil
    ["And what do they do in the meantime? Nothing!"] = "I co robią w międzyczasie? Nic!", -- A_Classic_Fairytale:queen
    ["And where's all the weed?"] = "I gdzie jest całe zioło?", -- A_Classic_Fairytale:dragon
    ["And you believed me? Oh, god, that's cute!"] = "A ty mi wierzyłeś? O boże, to słodkie!", -- A_Classic_Fairytale:journey
    ["And you need to move to the top!"] = "I musisz dostać się na szczyt!", -- Basic_Training_-_Movement
    ["An experimental editing tool for missions and more"] = "Eksperymentalne narzędzie do edycji i więcej.", -- HedgeEditor
    ["Anno 1032"] = "Rok 1032", -- Continental_supplies
    ["Anno 1032: [The explosion will make a strong push ~ Wide range, wont affect hogs close to the target]"] = "Anno 1032: [Eksplozja będzie miała silny odrzut ~ Szeroki zasięg, nie wpłynie na jeże blisko celu]", -- Continental_supplies
    ["An object has been destroyed before it took enough damage."] = "Obiekt został zniszczony zanim otrzymał wystarczająco dużo obrażeń.", -- SimpleMission
    ["Antarctica"] = "Antarktyda", -- Continental_supplies
    ["Antarctic summer: Every 4th turn you get 1 girder, 1 mudball, 2 sine guns and 1 portable portal device."] = "Antarktyczne lato: Co 4 tury dostajesz 1 belkę, 1 kulę błotną, 2 sinusoidalne giwery i 1 przenośne urządzenie portalowe.", -- Continental_supplies
    ["Antarctic summer: - Will give you one girder/mudball and two sineguns/portals every fourth turn."] = "Antarktyczne lato: Zapewni ci jedną belkę/kulę błotną i dwie sinusoidalne giwery/portale co każde cztery tury.", -- Continental_supplies
    ["Anti-Gravity Device Part (+1)"] = "Część Urządzenia Antygrawitacyjnego (+1)", -- A_Space_Adventure:desert01, A_Space_Adventure:fruit02, A_Space_Adventure:ice01
    ["Anton"] = "Antoni",
    ["An unexpected event!"] = "Nieoczekiwane wydarzenie!", -- A_Space_Adventure:cosmos
    ["Anyway, the aliens accept me for who I am."] = "Tak czy inaczej, obcy akceptuję mnie takiego, jaki jestem.", -- A_Classic_Fairytale:queen
    ["A random hedgehog will inherit the weapons of his deceased team-mates."] = "Losowy jeż odziedziczy bronie jego poległych członków zespołu.", -- A_Space_Adventure:death02
    ["Arashi"] = "Araszi",
    ["Area"] = "Obszar", -- Continental_supplies
    ["Areas surrounded by a green dashed outline are portal-proof and repel portals."] = "Obszary otoczone zielonym kreskowanym obrysem są portaloodporne i odrzucają portale.", -- A_Space_Adventure:final
    ["Areas surrounded by a security border are indestructible."] = "Obszary otoczone granicą bezpieczeństwa są niezniszczalne.", -- A_Space_Adventure:final
    ["Areas with a green dashed outline are portal-proof."] = "Obszary z zielonym kreskowanym obrysem są portaloodporne.", -- A_Space_Adventure:final
    ["Areas with a security outline are indestructible."] = "Obszary z obrysem bezpieczeństwa są niezniszczalne.", -- A_Space_Adventure:final
    ["Are we there yet?"] = "Już dotarliśmy?", -- A_Classic_Fairytale:shadow
    ["Are you accusing me of something?"] = "Czy ty o coś mnie oskarżasz?", -- A_Classic_Fairytale:backstab
    ["Are you helping the aliens?"] = "Czy ty pomagasz obcym?", -- A_Classic_Fairytale:queen
    ["Are you saying that many of us have died for your entertainment?"] = "Mówisz, że wielu z nas umarło dla twojej rozrywki?", -- A_Classic_Fairytale:enemy
    ["Argh, the boredom!"] = "Argh, ta nuda!", -- A_Classic_Fairytale:queen
    ["Artur Detour"] = "Artur Objazd", -- A_Classic_Fairytale:queen
    ["As a reward for your performance, here's some new technology!"] = "Jako nagrodę za twoje osiągnięcie, masz tu trochę nowej technologii.", -- A_Classic_Fairytale:dragon
    ["Ash"] = "Asz",
    ["A Shoppa minigame"] = "Minigra Shoppa", -- WxW
    ["Asia"] = "Azja", -- Continental_supplies
    ["As long you don't touch the ground, you can|re-use the same rope as often as you like."] = "Tak długo jak nie dotykasz ziemi, możesz|użyć tej samej liny ponownie ile tylko chcesz.", -- Basic_Training_-_Rope
    ["A smuggler! Prepare for battle"] = "Przemytnik! Przygotuj się na bitwę", -- A_Space_Adventure:desert01
    ["A Space Adventure"] = "Kosmiczna przygoda", -- A_Space_Adventure:desert01, A_Space_Adventure:moon01
    ["Assault Team"] = "Drużyna Szturmowa", -- A_Classic_Fairytale:backstab
    ["Asteroid"] = "Asteroida", -- Big_Armory
    ["As the ammo is sparse, you might want to reuse ropes while mid-air."] = "Jako iż amunicja jest rzadka, możesz chcieć używać liny wielokrotnie w powietrzu.", -- A_Classic_Fairytale:dragon
    ["As the challenge was completed, Leaks A Lot set foot on the ground..."] = "Gdy wyzwanie było ukończone, Spory Przeciek postawił stopę na ziemi...", -- A_Classic_Fairytale:first_blood
    ["As you are more experienced, I want you to lead them to battle."] = "Ponieważ jesteś bardziej doświadczony, chcę żebyś poprowadził ich na bitwę.", -- A_Space_Adventure:fruit01
    ["As you can see I have survived our last encounter and I had time to plot my master plan!"] = "Jak widzisz, przeżyłem nasze ostatnie spotkanie i miałem czas, by uknuć mój genialny plan!", -- A_Space_Adventure:death01
    ["As you can see, there is no way to get on the other side!"] = "Jak widzisz, nie ma sposobu, by dostać się na drugą stronę!", -- A_Classic_Fairytale:dragon
    ["As you probably noticed, these rubber bands|are VERY elastic. Hedgehogs and many other|things will bounce off without taking any damage."] = "Jak już pewnie zauważyłeś, te gumy są BARDZO elastyczne.|Jeże i wiele innych rzeczy się od nich odbije,|nie otrzymując żadnych obrażeń.", -- Basic_Training_-_Movement
    ["As you've seen, the dropped grenade roughly fell into your flying direction."] = "Jak widziałeś, upuszczony granat z grubsza spadł w kierunku twojego lotu.", -- Basic_Training_-_Flying_Saucer
    ["Athlete"] = "Atleta", -- Battalion
    ["Attack: Activate"] = "Atak: Aktywuj", -- Racer
    ["Attack Captain Lime before he attacks back."] = "Zaatakuj Kapitana Limonkę, zanim odpowie ogniem.", -- A_Space_Adventure:fruit02
    ["Attack From Rope: %s"] = "Atak z liny: %s", -- WxW
    ["Attack From Rope: You may only attack from a rope."] = "Atak z liny: Możesz atakować tylko z liny", -- WxW
    ["Attack rule: %s"] = "Zasada ataku: %s", -- WxW
    ["Attack: Select this continent"] = "Atak: Wybierz ten kontynent", -- Continental_supplies
    ["Attack: [Space]"] = "Atak: [Spacja]", -- Basic_Training_-_Bazooka, Basic_Training_-_Grenade, Basic_Training_-_Movement, Basic_Training_-_Rope
    ["Attack: Tap the [Bomb]"] = "Atak: Wciśnij [Bomba]", -- Basic_Training_-_Bazooka, Basic_Training_-_Grenade, Basic_Training_-_Movement, Basic_Training_-_Rope, A_Classic_Fairytale:first_blood, A_Classic_Fairytale:shadow
    ["Attack the assassins before they attack back."] = "Zaatakuj zabójców, zanim oni to zrobią.", -- A_Space_Adventure:fruit02
    ["Attack: Throw ball"] = "Atak: Rzuć kulę", -- Knockball
    ["At the end of the game your health was %d."] = "Na końcu gry, twoje zdrowie wynasiło %d.", -- A_Space_Adventure:ice01
    ["At the start of the game each enemy hog has only the weapon that he is named after."] = "Na początku gry, każdy wrogi jeż ma tylko tę broń, po której jest nazwany.", -- A_Space_Adventure:death02
    ["Australia"] = "Australia", -- Continental_supplies
    ["Available weapon specials:"] = "Dostępne speciały broni:", -- Continental_supplies
    ["Average pilot"] = "Przeciętny pilot", -- User_Mission_-_RCPlane_Challenge
    ["Avoid bazookas, red and blue invaders."] = "Unikaj bazook, czerwonych i niebieskich najeźdźców.", -- Space_Invasion
    ["Axes"] = "Osie", -- Bazooka_Battlefield
    ["Aye! Fellow! Let me exit this chamber of doom!"] = "Tak! Kolega! Daj mi wyjść z komnaty zagłady!", -- A_Classic_Fairytale:epil
    ["Back Breaker"] = "Łamacz Pleców", -- A_Classic_Fairytale:backstab
    ["Back in the village, after telling the villagers about the threat..."] = "Tymczasem w wiosce, po powiadomieniu mieszkańców o zagrożeniu...", -- A_Classic_Fairytale:united
    ["Back in the village, the two tribes finally started to live in harmony."] = "Tymczasem w wiosce, dwa plemienia w końcu zaczęły żyć w harmonii.", -- A_Classic_Fairytale:epil
    ["Back Jump: [Backspace] ×2"] = "Skok w tył: [Backspace] x2", -- Basic_Training_-_Movement
    ["Back Jump: Double-tap the [Curvy Arrow]"] = "Tylny skok: Wciśnij dwukrotnie [Zakręconą strzałkę]", -- Basic_Training_-_Movement
    ["Back Jumping (1/2)"] = "Skok w tył (1/2)", -- Basic_Training_-_Movement
    ["Back Jumping (2/2)"] = "Skok w tył (2/2)", -- Basic_Training_-_Movement
    ["Backstab"] = "Zdrada", -- A_Classic_Fairytale:backstab
    ["Backwards jump: Press [Backspace] twice"] = "Skok w tył: Wciśnij dwukrotnie [Backspace]", -- A_Classic_Fairytale:first_blood
    ["Backwards jump: Tap the [Curvy Arrow] twice"] = "Skok w tył: Wciśnij dwukrotnie [Zakręconą Strzałkę]", -- A_Classic_Fairytale:first_blood
    ["Bacon"] = "Bekon",
    ["Bad Guy"] = "Zły Koleś", -- User_Mission_-_The_Great_Escape
    ["Badmad"] = "Zływściekły", -- portal
    ["Bad Team"] = "Zła Drużyna", -- User_Mission_-_The_Great_Escape
    ["Bad timing"] = "Złe wyczucie czasu", -- A_Space_Adventure:fruit01
    ["Baggy"] = "Woruś",
    ["Balrog"] = "Balrog",
    ["Bamboo Thicket"] = "Bambusowy Gąszcz",
    ["Barrel Launcher"] = "Wyrzutnia Beczek",
    ["Barrel Placement Mode"] = "Tryb Stawiania Beczek", -- Construction_Mode
    ["BARREL PLACEMENT MODE"] = "TRYB STAWIANIA BECZEK", -- HedgeEditor
    ["Barrier unlocked!"] = "Beczka odblokowana!", -- Basic_Training_-_Rope
    ["Baseballbat"] = "Kij bejsbolwy", -- Continental_supplies
    ["Baseball bat specials cannot be used close to other hogs."] = "Speciały kija bejsbolowego nie mogą być użyte w pobliżu innych jeży.", -- Continental_supplies
    ["Baseball Bat with Ball"] = "Kij bejsbolowy z piłką", -- Knockball
    ["Base damage has been modified to 12 per shot."] = "Podstawowe obrażenia zostały zmodyfikowane do 12 na strzał.", -- Battalion
    ["Based on what you've learned, destroy the target on the girder and as always, land safely!"] = "Bazując na tym, co się nauczyłeś, zniszcz cel na belce i jak zawsze, wyląduj bezpiecznie!", -- Basic_Training_-_Flying_Saucer
    ["Basically this is a combination of diving and launching."] = "W zasadzie jest to połączenie nurkowania i wystrzeliwania.", -- Basic_Training_-_Flying_Saucer
    ["Basic Bazooka Training"] = "Podstawy korzystania z bazooki", -- Basic_Training_-_Bazooka
    ["Basic Grenade Training"] = "Podstawy korzystania z granatów", -- Basic_Training_-_Grenade
    ["Basic Movement Training"] = "Podstawy poruszania się", -- Basic_Training_-_Movement
    ["Basic Rope Training"] = "Podstawy korzystania z liny", -- Basic_Training_-_Rope
    ["Basic Training"] = "Trening podstawowy", -- Basic_Training_-_Bazooka, Basic_Training_-_Grenade, Basic_Training_-_Movement, Basic_Training_-_Rope
    ["Basketball"] = "Koszykówka", -- Basketball
    ["Bat balls at your enemies and|push them into the sea!"] = "Uderzaj piłkami w swoich przeciwników|i strącaj ich do wody!",
    ["Battalion"] = "Batalion", -- Battalion
    ["Battle Starts Now!"] = "Bitwa zaczyna się teraz!", -- A_Space_Adventure:fruit01
    ["Batty"] = "Kijek",
    ["Bat your opponents through the|baskets and out of the map!"] = "Uderzaj swoich przeciwników|wyrzucając przez kosz, poza mapę!",
    ["Bazooka Battlefield"] = "Bazookowe pole bitwy", -- Bazooka_Battlefield
    ["Bazooka Master"] = "Mistrz bazooki", -- Basic_Training_-_Bazooka
    ["Bazookas are influenced by wind."] = "Wiatr ma wpływ na bazookę.", -- Basic_Training_-_Bazooka
    ["Bazooka Training"] = "Trening bazooki",
    ["Bearded Beast"] = "Brodata Bestia",
    ["Be careful, the future of Hogera is in your hands!"] = "Bądź ostrożny, przyszłość Jeżery leży w twoich rękach!", -- A_Space_Adventure:cosmos
    ["Be careful, your fuel is limited from now on!"] = "Uważaj, od teraz twoje paliwo jest ograniczone!", -- Basic_Training_-_Flying_Saucer
    ["Be careful, your gadgets won't work in the bandit area. You should get an ice gun."] = "Uważaj, twoje gadżety nie zadziałają w strefie bandytów. Powinieneś zdobyć lodową spluwę.", -- A_Space_Adventure:ice01
    ["Beep Loopers"] = "Zapętlacze Brzęczków", -- A_Classic_Fairytale:queen
    ["Beginner"] = "Początkujący", -- User_Mission_-_RCPlane_Challenge
    ["Behind these trees on the east side there is Secret Base 17."] = "Za drzewami na wschodniej stronie, leży Sekretna Baza 17.", -- A_Space_Adventure:cosmos
    ["Below-average pilot"] = "Niedoświadczony pilot", -- User_Mission_-_RCPlane_Challenge
    ["Besides, why would I choose certain death?"] = "Poza tym, czemu miałbym wybrać pewną śmierć?", -- A_Classic_Fairytale:queen
    ["Best laps per team: "] = "Najszybsze okrążenie drużyny: ",
    ["Best team times: "] = "Najlepsze czasy drużyny: ", -- Racer, TechRacer
    ["Better get yourself another health crate to heal your wounds."] = "Lepiej zdobądź kolejną skrzynię ze zdrowiem, żeby uleczyć swoje rany.", -- Basic_Training_-_Movement
    ["Better luck next time!"] = "Więcej szczęścia następnym razem!", -- ClimbHome
    ["Better Safe Than Sorry"] = "Lepiej dmuchać na zimne", -- A_Space_Adventure:desert02
    ["Beware, any damage taken will stay until you complete the moon's main mission"] = "Uważaj, jakiekolwiek otrzymane obrażenia zostaną, aż ukończysz księżycową misję", -- A_Space_Adventure:cosmos
    ["Beware of mines: They explode after 3 seconds."] = "Uważaj na miny: Wybuchają po 3 sekundach.", -- A_Classic_Fairytale:journey
    ["Beware of mines: They explode after 5 seconds."] = "Uważaj na miny: Wybuchają po 5 sekundach.", -- A_Classic_Fairytale:journey
    ["Beware, though! If you are slow, you die!"] = "Uważaj jednak! Jeśli będziesz wolny, umrzesz!", -- A_Classic_Fairytale:dragon
    ["Beware, though! Many smugglers come often to explore these tunnels and scavenge whatever valuable items they can find."] = "Uważaj jednak! Wielu przemytników często przychodzi przeszukiwać te tunele i przegrzebywać jakiekolwiek cenne przedmioty uda im się znaleźć.", -- A_Space_Adventure:desert01
    ["Beware, though, you will only be able to move slowly through the water."] = "Uważaj jednak, w wodzie będziesz mógł się poruszać tylko powoli.", -- Basic_Training_-_Flying_Saucer
    ["Big Armory"] = "Wielka Zbrojownia", -- Big_Armory
    ["Billy Frost"] = "Billy Mróz", -- A_Space_Adventure:ice01
    ["Bingo"] = "Bingo",
    ["Bio-Filter: Aggressively removes enemies."] = "Bio-filtr: Agresywnie usuwa wrogów.", -- Construction_Mode
    ["Bio-Filter"] = "Bio-filtr", -- Construction_Mode
    ["Biomechanic Team"] = "Biomechaniczna Drużyna", -- A_Classic_Fairytale:family
    ["Bitter"] = "Gorzki",
    ["Blanka"] = "Blanka",
    ["Blender"] = "Blender", -- A_Classic_Fairytale:family
    ["Bloodpie"] = "Krwiociasto", -- A_Classic_Fairytale:backstab
    ["Bloodrocutor"] = "Krwiolokator", -- A_Classic_Fairytale:shadow
    ["Bloodsucker"] = "Krwiowysysacz", -- A_Classic_Fairytale:shadow
    ["Blue"] = "Niebieski",
    ["Blue Team"] = "Niebieska Drużyna", -- User_Mission_-_Dangerous_Ducklings
    ["Bob"] = "Bob", -- A_Space_Adventure:cosmos
    ["Bobo"] = "Bobo", -- User_Mission_-_Nobody_Laugh
    ["Bone Jackson"] = "Bone Jackson", -- A_Classic_Fairytale:backstab
    ["Bonely"] = "Kościany", -- A_Classic_Fairytale:shadow
    ["Bones"] = "Kości",
    ["BOOM! BOOM! BOOM! %s are the masters of destruction with %d destroyed invaders."] = "BUM! BUM! BUM! %s są mistrzami zniszczenia z %d zniszczonymi najeźdźcami.", -- Space_Invasion
    ["Boom!"] = "BUM!",
    ["Boom! %s has destroyed %d invaders."] = "Bum! %s zniszczyli %d najeźdźców.", -- Space_Invasion
    ["BOOM! %s really didn't like the invaders, so they decided to destroy as much as %d of them."] = "BUM! %s naprawdę nie lubili najeźdźców, więc zdecydowali się zniszczyć aż %d z nich.", -- Space_Invasion
    ["Boris"] = "Borys", -- A_Space_Adventure:moon01
    ["Boss defeated! +30 points!"] = "Szef pokonany! +30 punktów!", -- Space_Invasion
    ["Boss Slayer! +25 points!"] = "Pogromca Szefów! +25 punktów!", -- Space_Invasion
    ["Both Barrels"] = "Obie Beczki",
    ["Both your hedgehogs must survive."] = "Oba twoje jeże muszą przeżyć.", -- User_Mission_-_Teamwork_2, User_Mission_-_Teamwork
    ["Bottom Feeder"] = "Pasożyt", -- Mutant
    ["Bounciness"] = "Odbijalność", -- Basic_Training_-_Grenade
    ["Bouncing Bomb"] = "Odbijająca się bomba", -- Basic_Training_-_Bazooka
    ["Bouncy Boomerang"] = "Odbijający się bumerang", -- Continental_supplies
    ["Bouncy Girder: [4]"] = "Odbijająca Belka: [4]", -- HedgeEditor
    ["Bouncy Land: [4]"] = "Odbijający Teren: [4]", -- HedgeEditor
    ["Bouncy Land"] = "Odbijający Teren", -- HedgeEditor
    ["Bounty: Get 6 weapons for each kill (even on own hogs)."] = "Premia: Dostań 6 broni za każde zabójstwo (nawet na własnych jeżach).", -- Continental_supplies
    ["Bozo"] = "Bozo",
    ["Brain Blower"] = "Mózgorozwalacz", -- A_Classic_Fairytale:journey
    ["Brainiac"] = "Możdżak", -- A_Classic_Fairytale:epil, A_Classic_Fairytale:first_blood, A_Classic_Fairytale:shadow
    ["Brainila"] = "Możdżila", -- A_Classic_Fairytale:united
    ["Brain Stu"] = "Mózgozupa", -- A_Classic_Fairytale:united
    ["Brain Teaser"] = "Łamigłowa", -- A_Classic_Fairytale:backstab
    ["Brigadier Briggs"] = "Brygadier Briggs",
    ["Bruce"] = "Bruce", -- A_Space_Adventure:moon01
    ["Brutal Lily"] = "Brutalna Lilia", -- A_Classic_Fairytale:enemy, A_Classic_Fairytale:epil
    ["Brutus"] = "Brutus", -- A_Classic_Fairytale:backstab
    ["Build a fortress and destroy your enemy."] = "Zbuduj fortecę i zniszcz swojego wroga.", -- Construction_Mode
    ["Build an awesome race track by placing|waypoints which the hedgehogs have to|touch in any order to finish a round."] = "Zbuduj niesamowitą trasę wyścigową,|stawiając punkty kontrolne, które jeże|muszą dotknąć, by zakończyć rundę.", -- Racer
    ["Build a track and race."] = "Zbuduj trasę i ścigaj się.",
    ["Builder"] = "Budowniczy", -- Battalion
    ["Build one of multiple different structures|to aid you in victory, at the cost of energy."] = "Zbuduj jedną z wielu różnych struktur, by|pomóc sobie w zwycięstwie, kosztem energii.", -- Construction_Mode
    ["Bullseye"] = "Oczko", -- A_Classic_Fairytale:dragon
    ["Bunny"] = "Zając",
    ["burp"] = "burp",
    ["Bushes"] = "Krzaki", -- Bazooka_Battlefield
    ["Bushi"] = "Buszi",
    ["Buster"] = "Drań",
    ["But it proved to be no easy task!"] = "Ale okazało się to nie być łatwym zadaniem!", -- A_Classic_Fairytale:dragon
    ["But I want my sandals!"] = "Ale ja chcę moje sandały!", -- A_Classic_Fairytale:queen
    ["But one thing's for sure:"] = "Ale jedna rzecz jest pewna:", -- A_Space_Adventure:final
    ["But that's impossible!"] = "Ale to niemożliwe!", -- A_Classic_Fairytale:backstab
    ["But the ones alive are stronger in their heart!"] = "Ale ci co żyją, są silniejsi w swoich sercach!", -- A_Classic_Fairytale:enemy
    ["But … they kidnapped you!"] = "Ale... porwali cię!", -- A_Classic_Fairytale:queen
    ["But...we died!"] = "Ale... umarliśmy!", -- A_Classic_Fairytale:backstab
    ["But where can we go?"] = "Ale gdzie możemy pójść?", -- A_Classic_Fairytale:united
    ["But why did you betray us?!"] = "Ale dlaczego nas zdradziłeś?!", -- A_Classic_Fairytale:queen
    ["But why would they help us?"] = "Ale czemu by nam pomagali?", -- A_Classic_Fairytale:backstab
    ["But you're cannibals. It's what you do."] = "Ale jesteście kanibalami. Tym się zajmujecie.", -- A_Classic_Fairytale:enemy
    ["But you said you'd let her go!"] = "Ale powiedziałeś, że ją wypuścisz!", -- A_Classic_Fairytale:journey
    ["But you saved me!"] = "Ale uratowałeś mnie!", -- A_Classic_Fairytale:queen
    ["By the way, not only bazookas will bounce on water, but also greandes and many other things."] = "Przy okazji, nie tylko bazooka odbija się od wody, ale rownież granaty i inne rzeczy.", -- Basic_Training_-_Bazooka
    ["By the way, not only bazookas will bounce on water, but also grenades and many other things."] = "Przy okazji, nie tylko bazooki odbijają się od wody, ale również granaty i wiele innych rzeczy.", -- Basic_Training_-_Bazooka
    ["By the way, you can turn around without walking|by holding down Precise when you hit a walk control."] = "Przy okazji, możesz obracać się bez chodzenia|trzymając Precyzję, kiedy wciskasz przycisk chodzenia.", -- Basic_Training_-_Movement
    ["C-1"] = "C-1", -- portal
    ["C-2"] = "C-2", -- portal
    ["Callahan"] = "Callahan",
    ["Call me Beep! Well, 'cause I'm such a nice...person!"] = "Zwij mnie Brzęczek! Bo jestem taką miłą... osobą!", -- A_Classic_Fairytale:family
    ["Cannibal Sentry"] = "Wartownik Kanibali", -- A_Classic_Fairytale:journey
    ["Cannibals"] = "Kanibale", -- A_Classic_Fairytale:enemy, A_Classic_Fairytale:epil, A_Classic_Fairytale:first_blood
    ["Cannibals?! You're the cannibals!"] = "Kanibale?! To wy jesteście kanibalami!", -- A_Classic_Fairytale:enemy
    ["Can you do it?"] = "Możesz to zrobić?", -- A_Space_Adventure:ice02
    ["Cappy"] = "Czapeczka", -- Basic_Training_-_Movement
    ["Captain Lime"] = "Kapitan Limonka", -- A_Space_Adventure:fruit01, A_Space_Adventure:fruit02
    ["Captain Lime offered his help if you assist him in battle."] = "Kapitan Limonka zaoferował swoją pomoc, jeśli wesprzesz go w walce.", -- A_Space_Adventure:fruit01
    ["Capture The Flag"] = "Przejmij Flagę", -- Capture_the_Flag, CTF_Blizzard
    ["Careful, hedgehogs can't swim!"] = "Ostrożnie, jeże nie potrafią pływać!", -- Basic_Training_-_Movement
    ["Careless"] = "Nieostrożny",
    ["Carol"] = "Karolina", -- A_Classic_Fairytale:family
    ["Challenge completed!"] = "Wyzwanie ukończone!", -- User_Mission_-_Rope_Knock_Challenge, User_Mission_-_That_Sinking_Feeling, SpeedShoppa
    ["CHALLENGE COMPLETE"] = "WYZWANIE UKOŃCZONE", -- User_Mission_-_RCPlane_Challenge
    ["Challenge failed!"] = "Wyzwanie nieudane!", -- SpeedShoppa
    ["Challenge objectives"] = "Cele wyzwania", -- A_Space_Adventure:death02, A_Space_Adventure:desert03, A_Space_Adventure:final, A_Space_Adventure:fruit03, A_Space_Adventure:moon02
    ["Challenge over!"] = "Wyzwanie skończone!", -- User_Mission_-_Rope_Knock_Challenge
    ["Challenge"] = "Wyzwanie", -- User_Mission_-_RCPlane_Challenge, User_Mission_-_Rope_Knock_Challenge, User_Mission_-_That_Sinking_Feeling, SpeedShoppa, ClimbHome
    ["Change bounciness: Tap [B]"] = "Zmień sprężystość: Wciśnij [B]", -- Basic_Training_-_Grenade
    ["Change Content: [Left], [Right]"] = "Zmień zawartość: [Lewo], [Prawo]", -- HedgeEditor
    ["Change detonation timer: Tap the [Clock]"] = "Zmień czas detonatora: Wciśnij [Zegar]", -- Basic_Training_-_Grenade, A_Classic_Fairytale:shadow
    ["Change direction: [Left]/[Right]"] = "Zmień kierunek: [Lewo]/[Prawo]", -- Basic_Training_-_Grenade
    ["Change Health Boost: [Left], [Right]"] = "Zmień wzrost zdrowia: [Lewo], [Prawo]", -- HedgeEditor
    ["Change Health: [Left], [Right]"] = "Zmień zdrowie: [Lewo], [Prawo]", -- HedgeEditor
    ["Change modification mode: [Left], [Right]"] = "Zmień tryb modyfikacji: [Lewo], [Prawo]", -- HedgeEditor
    ["Change Placement Mode: [Up], [Down]"] = "Zmień tryb ustawiania: [Lewo], [Prawo]", -- HedgeEditor
    ["Change Rotation: [Left], [Right]"] = "Zmień obrót: [Lewo], [Prawo]", -- HedgeEditor
    ["Change Sprite Frame: [Precise]+[Left], [Precise]+[Right]"] = "Zmień klatkę Grafiki: [Precyzja]+[Lewo], [Precyzja]+[Prawo]", -- HedgeEditor
    ["Change Sprite: [Left], [Right]"] = "Zmień Grafikę: [Lewo], [Prawo]", -- HedgeEditor
    ["Change Timer: [Left], [Right]"] = "Zmień czas: [Lewo], [Prawo]", -- HedgeEditor
    ["Change weapon: [Long jump] or [Slot 1]-[Slot 3]"] = "Zmień broń: [Długi skok] lub [Slot 1]-[Slot 3]", -- Tumbler
    ["Charmander"] = "Charmander",
    ["Chasing the blue hog"] = "Ścigając niebieskego jeża", -- A_Space_Adventure:moon02
    ["Cheater"] = "Oszust", -- User_Mission_-_RCPlane_Challenge
    ["Checkpoint reached!"] = "Punkt kontrolny osiągnęty!", -- A_Space_Adventure:cosmos, A_Space_Adventure:ice01, A_Space_Adventure:moon01
    ["Chef"] = "Kucharz", -- Battalion, HedgeEditor
    ["Chester"] = "Czester",
    ["Chicken"] = "Kurczak",
    ["Chief Sandologist"] = "Naczelny Piaskolog", -- A_Space_Adventure:desert01
    ["Chikorita"] = "Chikorita",
    ["Choose location: Left click"] = "Wybierz lokację: Lewy klik", -- A_Classic_Fairytale:shadow
    ["Choose location: Tap the [Target] button, then tap on the spot you want to choose"] = "Wybierz lokację: Wciśnij przycisk [Cel], a potem na punkt, który chcesz wybrać", -- A_Classic_Fairytale:shadow
    ["Choose Selection/Placement/Deletion: [Left], [Right]"] = "Wybierz Zaznaczanie/Ustawianie/Usuwanie: [Lewo], [Prawo]", -- HedgeEditor
    ["Choose your continent wisely, as your decision will be permanent."] = "Wybierz swój kontynent mądrze, bo twoja decyzja będzie trwała.", -- Continental_supplies
    ["Choose your side! If you want to join the strange man, walk up to him.|Otherwise, walk away from him. If you decide to att...nevermind..."] = "Wybierz swoją stronę! Jeśli chcesz dołączyć do dziwnego mężczyzny, podejdź do niego.|W innym przypadku, odejdź od niego. Jeśli zdecydujesz się zaata... nieważne...", -- A_Classic_Fairytale:shadow
    ["Chunli"] = "Czunli",
    ["Clark Kent"] = "Clark Kent",
    ["Cleaver Placement Mode"] = "Tryb Stawiania Tasaków", -- Construction_Mode
    ["CLEAVER PLACEMENT MODE"] = "TRYB STAWIANIA TASAKÓW", -- HedgeEditor
    ["Cleaver"] = "Tasak", -- Construction_Mode
    ["Climb Home"] = "Wspinaczka do domu", -- ClimbHome
    ["Closing in"] = "Nadejście", -- A_Classic_Fairytale:queen
    ["Clown"] = "Klaun", -- HedgeEditor
    ["Clowns"] = "Klauny", -- User_Mission_-_Nobody_Laugh
    ["Cluck-cluck time: [Fire an egg ~ Sabotages and cures poison ~ Cannot be fired close to another hog]"] = "Czas gdakania: [Wystrzel jajo ~ Sabotuje i leczy truciznę ~ Nie może być wystrzelone blisko innego jeża]", -- Continental_supplies
    ["Clumsy"] = "Fajtłapa",
    ["Cluster Bomb Training"] = "Trening granatami odłamkowymi", -- Basic_Training_-_Cluster_Bomb
    ["Collateral Damage"] = "Dodatkowe Obrażenia", -- A_Classic_Fairytale:journey
    ["Collateral Damage II"] = "Dodatkowe Obrażenia II", -- A_Classic_Fairytale:journey
    ["- Collect all the blue crates"] = "- Zbierz wszystkie niebieskie skrzynie", -- HedgeEditor
    ["Collect all the crates, but remember, our time in this life is limited!"] = "Zbierz wszystkie skrzynie, ale pamiętaj - nasz czas w tym życiu jest ograniczony!", -- A_Classic_Fairytale:first_blood
    ["Collect or destroy all the health crates."] = "Zbierz lub zniszcz wszystkie skrzynki ze zdrowiem.", -- User_Mission_-_RCPlane_Challenge
    ["Collect or destroy the final crate to finish the training."] = "Zbierz lub zniszcz ostatnią skrzynkę, by zakończyć trening.", -- Basic_Training_-_Flying_Saucer
    ["- Collect the blue crate"] = "- Zbierz niebieską skrzynię", -- HedgeEditor
    ["Collect the crate and attack!"] = "Zbierz skrzynię i atakuj!", -- WxW
    ["Collect the crate on the right."] = "Zbierz skrzynię po prawej.", -- A_Classic_Fairytale:first_blood
    ["Collect the crates within the time limit!|If you fail, you'll have to try again."] = "Zbierz skrzynie na czas!|Jeśli i się nie uda, będziesz musiał spróbować ponownie.", -- A_Classic_Fairytale:first_blood
    ["Collect the first crate to begin!"] = "Zbierz pierwszą skrzynię, by zacząć!", -- Basic_Training_-_Flying_Saucer
    ["Collect the freezer and get the device part from Thanta."] = "Zbierz zamrażarkę i zabierz część urządzenia Tancie.", -- A_Space_Adventure:ice01
    ["Collect the green and purple invaders."] = "Zbierz zielonych i fioletowych najeźdźców.", -- Space_Invasion
    ["Collect the remaining crates to complete the training."] = "Zbierz pozostałe skrzynie, by ukończyć ten trening.", -- Basic_Training_-_Movement
    ["Collect the weapon crate and drop|a grenade from rope to destroy the barrels."] = "Zbierz skrzynię z bronią|i upuść granat z liny, by zniszczyć beczki.", -- Basic_Training_-_Rope
    ["Collect the weapon crate at the left coast!"] = "Zbierz skrzynię z bronią na lewym wybrzeżu.", -- A_Classic_Fairytale:journey
    ["Color Squad"] = "Kolorowy Oddział",
    ["Come closer and die! … burp …"] = "Podejdź bliżej i umrzyj! ...bek...", -- A_Classic_Fairytale:queen
    ["Come closer, so that your training may continue!"] = "Podejdź bliżej, żeby kontynuować swój trening!", -- A_Classic_Fairytale:first_blood
    ["Comet"] = "Kometa", -- Big_Armory
    ["Commander"] = "Dowódca", -- HedgeEditor
    ["Compete to use as few planes as possible!"] = "Rywalizuj w użyciu najmniejszej jak się da ilości samolotów!", -- User_Mission_-_RCPlane_Challenge
    ["Complete all main and side missions to complete the spacetrip mission."] = "Ukończ wszystkie główne i poboczne misje, by ukończyć kosmiczną wycieczkę.", -- A_Space_Adventure:cosmos
    ["Complete the obstacle course."] = "Ukończ tor przeszkód.", -- Basic_Training_-_Movement
    ["Complete the remaining side missions to complete this mission."] = "Ukończ pozostałe misje poboczne, by ukończyć tę misję.", -- A_Space_Adventure:cosmos
    ["Complete the track as fast as you can!"] = "Ukończ trasę tak szybko jak tylko potrafisz!",
    ["Completion time: %.2fs"] = "Czas ukończenia: %.2fs", -- User_Mission_-_Rope_Knock_Challenge
    ["Comrades! Sail me away!"] = "Kamraci! Zabierzcie mnie stąd!", -- A_Classic_Fairytale:queen
    ["Configuration accepted."] = "Konfiguracja przyjęta.", -- WxW
    ["Configuration phase"] = "Faza konfiguracjy", -- WxW
    ["Congrats! You won!"] = "Gratulacje! Wygrałeś!", -- A_Space_Adventure:moon01
    ["Congratulations!"] = "Gratulacje!",
    ["Congratulations"] = "Gratulacje", -- Basic_Training_-_Rope
    ["Congratulations, you acquired the device part!"] = "Gratulacje, zdobyłeś część urządzenia!", -- A_Space_Adventure:ice01
    ["Congratulations, you are the best!"] = "Gratulacje, jesteś najlepszy!", -- A_Space_Adventure:desert03
    ["Congratulations, you are the fastest!"] = "Gratulacje, jesteś najszybszy!", -- A_Space_Adventure:moon02
    ["Congratulations, you collected the device part!"] = "Gratulacje, zebrałeś część urządzenia!", -- A_Space_Adventure:ice01
    ["Congratulations! You have completed the obstacle course!"] = "Gratulacje! Ukończyłeś tor przeszkód!", -- Basic_Training_-_Movement
    ["Congratulations! You have destroyed all targets within the time."] = "Gratulacje! Zniszczyłeś wszystkie cele w czasie.", -- TargetPractice
    ["Congratulations, you have saved Hogera!"] = "Gratulacje, uratowałeś Jeżerrę!", -- A_Space_Adventure:final
    ["Congratulations! You have truly mastered this challenge! Don't forget to save the demo."] = "Gratulacje! Prawdziwie opanowałeś to wyzwanie! Nie zapomnij zapisać dema.", -- User_Mission_-_RCPlane_Challenge
    ["Congratulations! You've completed the Basic Rope Training!"] = "Gratulacje! Ukończyłeś podstawowy trening linowy!", -- Basic_Training_-_Rope
    ["Congratulations! You've eliminated all targets|within the allowed time frame."] = "Gratulacje! Zniszczyłeś wszystkie cele|przed upłynięciem czasu.",
    ["Congratulations! You win."] = "Gratulacje! Wygrywasz.", -- Big_Armory
    ["Congratulations, you won!"] = "Gratulacje, wygrałeś!", -- A_Space_Adventure:death01, A_Space_Adventure:death02, A_Space_Adventure:desert01, A_Space_Adventure:desert02, A_Space_Adventure:fruit02, A_Space_Adventure:fruit03, A_Space_Adventure:ice02
    ["Conquering the galaxy"] = "Podbijanie galaktyki", -- A_Space_Adventure:cosmos
    ["Construction Mode tool"] = "Narzędzie Trybu Konstrukcji", -- Construction_Mode
    ["CONSTRUCTION MODE"] = "TRYB KONSTRUKCJI", -- Construction_Mode
    ["Construction Station: Allows placement of|    girders, rubber, mines, sticky mines|    and barrels."] = "Stacja Konstrukcji: Pozwala na stawianie|    belek, gumy, min, min samoprzylepnych|    i beczek.", -- Construction_Mode
    ["Construction Station"] = "Stacja Konstrukcji", -- Construction_Mode
    ["Continental supplies"] = "Kontynentalne zaopatrzenie", -- Continental_supplies
    ["Continent selection"] = "Wybór kontynentu", -- Continental_supplies
    ["Continents: Select a continent at the beginning."] = "Kontynenty: Wybierz kontynent na początku", -- Continental_supplies
    ["Control"] = "Kontrola", -- Control
    ["Control pillars to score points."] = "Kontroluj kolumny, by zdobyć punkty.",
    ["Controls: Hold the Attack key (space by default) to|fire the rope, then, once attached use:|Left and Right to swing the rope;|Up and Down to contract and expand."] = "Sterowanie: Przytrzymaj przycisk Ataku (domyślnie Spacja), by|wystrzelić linę, potem po zaczepieniu, użyj:|Lewo i Prawo, by rozhuśtać linę;|Góra i Dół, by skurczyć i rozszerzyć.", -- Basic_Training_-_Rope
    ["Copper"] = "Miedź", -- User_Mission_-_Nobody_Laugh
    ["Corn"] = "Kukurydza", -- A_Space_Adventure:fruit01
    ["Corporal Calvin"] = "Kapral Kalwin",
    ["Corporationals"] = "Korponacjonaliści", -- A_Classic_Fairytale:queen
    ["Corpsemonger"] = "Ciałohandlarz", -- A_Classic_Fairytale:shadow
    ["Corpse Thrower"] = "Truchło-rzucacz", -- A_Classic_Fairytale:epil
    ["Cost: %d"] = "Koszt: %d", -- Construction_Mode
    ["Cost"] = "Koszt", -- Construction_Mode
    ["Cotton Needer"] = "Bawełniany Potrzebujący", -- Mutant
    ["Count Hogula"] = "Hrabia Jeżula",
    ["Coward"] = "Tchórz", -- A_Classic_Fairytale:queen
    ["Crate Before Attack: %s"] = "Skrzynia przed atakiem: %s", -- WxW
    ["Crate Before Attack: You must collect a crate before you can attack."] = "Skrzynia przed atakiem: Musisz zebrać skrzynię, zanim będziesz móc atakować.", -- WxW
    ["Crate Placer"] = "Stawiacz skrzyń", -- Construction_Mode
    ["Crates: Crates drop more often with a higher chance of bonus ammo"] = "Skrzynie: Skrzynie spadają częściej, z większą szansą na dodatkową amunicję", -- Battalion
    ["Crates: Crates drop randomly and may be empty"] = "Skrzynie: Skrzynie spadają losowo i mogą być puste", -- Battalion
    ["Crates: Crates drop randomly with chance of being empty"] = "Skrzynie: Skrzynie spadają losowo z szansą na bycie pustymi", -- Battalion
    ["Crates left: %d"] = "Pozostałe skrzynie: %d", -- User_Mission_-_RCPlane_Challenge
    ["Crates Left:"] = "Pozostałe skrzynie:", -- User_Mission_-_RCPlane_Challenge
    ["Crates per turn: %d"] = "Skrzynie na turę: %d", -- WxW
    ["Crazy Gravity: Gravity randomly changes within a range from %i%% to %i%% with a period of %s"] = "Szalona grawitacja: Grawitacja losowo zmienia się w zakresie od %i%% do %i%%, z okresem %s", -- Gravity
    ["Crazy Runner"] = "Szalony Biegacz", -- A_Space_Adventure:moon02
    ["Cricket Time"] = "Czas na krykieta", -- Continental_supplies
    ["Cricket time: [Fire away a 1 sec mine! ~ Cannot be fired close to another hog]"] = "Czas krykieta: [Wystrzel 1-sekundową minę! ~ Nie może być wystrzelone blisko innego jeża]", -- Continental_supplies
    ["CTF_Blizzard"] = "CTF Burza Śnieżna", -- CTF_Blizzard
    ["Cursor: Build structure"] = "Kursor: Zbuduj strukturę", -- Construction_Mode
    ["Cursor: Mode action"] = "Kursor: Akcja trybu", -- HedgeEditor
    ["|Cursor: Place crate"] = "|Kursor: Postaw skrzynię", -- Construction_Mode
    ["Cursor: Place waypoint"] = "Kursor: Postaw punkt kontrolny", -- Racer
    ["Cutlass Cain"] = "Kordelas Kain",
    ["Cybernetic Empire"] = "Cybernetyczne Imperium",
    ["Cyborg. It's what the aliens call themselves."] = "Cyborgi. Tak nazywają siebie ci kosmici.", -- A_Classic_Fairytale:enemy
    ["Dahmer"] = "Damer", -- A_Classic_Fairytale:backstab
    ["Daisy"] = "Daisy",
    ["DAMMIT, ROOKIE! GET OFF MY HEAD!"] = "Młody!!! Złaź z mojej głowy!!!",
    ["DAMMIT, ROOKIE!"] = "Żółtodziobie!",
    ["+%d ammo"] = "+%d amunicji", -- Battalion
    ["+%d Ammo"] = "+%d amunicji", -- Space_Invasion
    ["Dangerous Ducklings"] = "Niebezpieczne Kaczory",
    ["Dark Strawberry"] = "Mroczna Truskawa", -- A_Space_Adventure:fruit02
    ["%d crate(s) remaining"] = "%d skrzyń pozostało", -- SpeedShoppa
    ["%d damage was dealt in this game."] = "%d obrażeń zostało zadanych w tej grze.", -- Mutant
    ["+%d"] = "+%d", -- Battalion
    ["%d / %d"] = "%d / %d", -- Battalion
    ["%d | %d"] = "%d | %d", -- Mutant
    ["%d/%d"] = "%d/%d", -- SpeedShoppa
    ["Deadly Grape"] = "Zabójczy Winogron", -- A_Space_Adventure:fruit02
    ["Deadweight"] = "Balast",
    ["Deal 15 damage + 10% of your hog’s health to all hogs around you and get 2/3 back."] = "Zadaj 15 obrażeń + 10% zdrowia twojego jeża wszystkim jeżom dookoła ciebie i dostań 2/3 z powrotem.", -- Continental_supplies
    ["Deals 15 damage to all enemies in the circle."] = "Zadaje 15 obrażeń wszystkim wrogom w kole.", -- Continental_supplies
    ["Deer"] = "Jeleń",
    ["Defeat all enemies!"] = "Pokonaj wszystkich wrogów!", -- portal
    ["Defeat!"] = "Porażka!", -- HedgeEditor
    ["Defeat Professor Hogevil!"] = "Pokonaj Profesora Jeżozło!", -- A_Space_Adventure:death01
    ["Defeat the cannibals!"] = "Pokonaj kanibali!", -- A_Classic_Fairytale:shadow
    ["Defeat the cannibals!|Grenade hint: set the timer with [1-5], aim with [Up]/[Down] and hold [Space] to set power"] = "Pokonaj kanibali!|Porada do granatów: ustaw zapalnik używając [1-5], celuj [Góra]/[Dół] i przytrzymaj [Spację], by ustawić moc", -- A_Classic_Fairytale:shadow
    ["Defeat the cyborgs!"] = "Pokonaj cyborgów!", -- A_Classic_Fairytale:enemy
    ["Defeat the enemy!"] = "Pokonaj wroga!", -- A_Classic_Fairytale:queen
    ["Delete Waypoint"] = "Usuń punkt kontrolny", -- HedgeEditor
    ["Deletion Mode: [5]"] = "Tryb usuwania: [5]", -- HedgeEditor
    ["Deletion Mode"] = "Tryb Usuwania", -- HedgeEditor
    ["Deletition Mode"] = "Tryb Usuwania", -- HedgeEditor
    ["Demo"] = "Demo", -- The_Specialists
    ["Demolition is fun!"] = "Rozwałka jest fajna!",
    ["Dense Cloud"] = "Gęsta Chmura", -- A_Classic_Fairytale:backstab, A_Classic_Fairytale:dragon, A_Classic_Fairytale:enemy, A_Classic_Fairytale:epil, A_Classic_Fairytale:family, A_Classic_Fairytale:journey, A_Classic_Fairytale:queen, A_Classic_Fairytale:shadow, A_Classic_Fairytale:united
    ["Dense Cloud must have already told them everything..."] = "Gęsta Chmura pewnie im już wszystko powiedział...", -- A_Classic_Fairytale:shadow
    ["Dense Cloud?! What are you doing?!"] = "Gęsta Chmura?! Co ty robisz?!", -- A_Classic_Fairytale:queen
    ["Depleted Kamikaze! +5 points!"] = "Wyczerpany Kamikaze! +5 punktów", -- Space_Invasion
    ["Derp"] = "Derp", -- User_Mission_-_Nobody_Laugh
    ["Desert Storm"] = "Pustynna Burza",
    ["Destroy all targets with no more than 10 bazookas."] = "Zniszcz wszystkie cele używając nie więcej niż 10 bazook.", -- Basic_Training_-_Bazooka
    ["Destroy all targets with no more than 5 bazookas."] = "Zniszcz wszystkie cele używając nie więcej niż 5 bazook.", -- Basic_Training_-_Bazooka
    ["Destroy all the targets!"] = "Zniszcz wszystkie cele!", -- Basic_Training_-_Bazooka, Basic_Training_-_Grenade
    ["Destroyer of planes"] = "Niszczyciel samolotów", -- User_Mission_-_RCPlane_Challenge
    ["Destroy him, Leaks A Lot! He is responsible for the deaths of many of us!"] = "Zniszcz go, Spory Przecieku! On jest odpowiedzalny za śmierć wielu z nas!", -- A_Classic_Fairytale:first_blood
    ["Destroy invaders and collect bonuses to score points."] = "Niszcz najeźdźców i zbieraj bonusy, by zaliczać punkty.", -- Space_Invasion
    ["- Destroy the enemy"] = "- Zniszcz wroga", -- HedgeEditor
    ["- Destroy the red target"] = "- Zniszcz czerwony cel", -- HedgeEditor
    ["- Destroy the red targets"] = "- Zniszcz czerwone cele", -- HedgeEditor
    ["Destroy the targets!"] = "Zniszcz cele!", -- Basic_Training_-_Bazooka, Basic_Training_-_Grenade
    ["+%d flamer fuel!"] = "+%d paliwa miotacza ognia!", -- Tumbler
    ["+%d health"] = "+%d zdrowia", -- Mutant
    ["%d-Hit Combo! +%d points!"] = "%d-krotne kombo! +%d punktów!", -- Space_Invasion
    ["Did anyone follow you?"] = "Czy ktoś cię śledził?", -- A_Classic_Fairytale:united
    ["Did I miss something?"] = "Czy ja coś przegapiłem?", -- Space_Invasion
    ["Did not finish"] = "Nieukończony", -- Racer, TechRacer
    ["Did you really think I've changed?"] = "Naprawdę myślałeś, że się zmieniłem?", -- A_Classic_Fairytale:queen
    ["Did you really think that I've changed?"] = "Naprawdę myślałeś, że się zmieniłem?", -- A_Classic_Fairytale:queen
    ["Did you really think that we needed the help of one of you?"] = "Naprawdę myślałeś, że potrzebujemy pomocy jednego z was?", -- A_Classic_Fairytale:queen
    ["Did you see him coming?"] = "Widziałeś jak nadchodzi?", -- A_Classic_Fairytale:shadow
    ["Did you warn the village?"] = "Ostrzegłeś wioskę?", -- A_Classic_Fairytale:shadow
    ["Die, die, die!"] = "Giń, giń, giń!", -- A_Classic_Fairytale:dragon
    ["Difficulty: Easy"] = "Poziom trudności: Łatwy", -- A_Classic_Fairytale:first_blood
    ["Difficulty: Hard"] = "Poziom trudności: Trudny", -- A_Classic_Fairytale:first_blood
    ["Difficulty: "] = "Poziom trudności:", -- Continental_supplies
    ["Dimitry"] = "Dymitr",
    ["%d invaders have been destroyed in this game."] = "%d najeźdźców zostało zniszczonych w tej grze.", -- Space_Invasion
    ["Disabled"] = "Wyłączone", -- WxW
    ["Disguise as a Rockhopper Penguin"] = "Przebież się za Pingwina Rakietoskoczka", -- Continental_supplies
    ["Disguise as a Rockhopper Penguin: [Swap place with a random enemy hog in the circle]"] = "Przebranie za pingwina skalnegi: [Zamień miejsce z losowym wrogim jeżem w kole]", -- Continental_supplies
    ["Displacer"] = "Wypieracz",
    ["Diver"] = "Nurek", -- User_Mission_-_Diver
    ["%d ms"] = "%d ms", -- HedgeEditor
    ["Doing stuff a monkey could do."] = "Robić rzeczy, które mogłaby zrobić małpa.", -- A_Classic_Fairytale:queen
    ["Domination game"] = "Gra o dominację", -- Control
    ["Donald"] = "Donad",
    ["Do not destroy the crates!"] = "Nie niszcz skrzyń!", -- A_Space_Adventure:fruit02
    ["Do not laugh, inexperienced one, for he speaks the truth!"] = "Nie śmiej się, niedoświadczony, on mówi prawdę!", -- A_Classic_Fairytale:backstab
    ["Do not let his words fool you, young one! He will stab you in the back as soon as you turn away!"] = "Nie pozwól jego słowom cię oszukać, młodzieńcze! On wbije ci nóż w plecy zaraz po tym, jak się odwrócisz!", -- A_Classic_Fairytale:first_blood
    ["Don't be foolish, son, there will be more."] = "Nie bądź głupi synu, będzie ich więcej!", -- A_Space_Adventure:fruit01
    ["Don't blow up the crate."] = "Nie wysadź skrzyni.", -- A_Classic_Fairytale:journey
    ["Don't destroy the device crate!"] = "Nie zniszcz skrzyni urządzenia!", -- A_Space_Adventure:desert01
    ["Don't eliminate Captain Lime before collecting the last crate!"] = "Nie eliminuj Kapitana Limonki przed zebraniem ostatniej skrzyni!", -- A_Space_Adventure:fruit02
    ["Don't hit me, you fools!"] = "Nie uderzcie mnie, wy głupcy!", -- A_Space_Adventure:moon01
    ["Don't hit yourself!"] = "Nie uderz się!", -- Basic_Training_-_Bazooka
    ["Don't touch the flames!"] = "Nie dotknij płomieni!", -- ClimbHome
    ["Don't you dare harming our tribe!"] = "Nie waż się skrzywdzić naszego plemienia!", -- A_Classic_Fairytale:queen
    ["Double Kill!"] = "Podwójne zabójstwo!",
    ["Double kill!"] = "Podwójne zabójstwo!", -- Mutant
    ["Do you have any idea how bad an exploding arrow hurts?"] = "Czy ty masz pojęcie jak bardzo boli eksplodująca strzała?", -- A_Classic_Fairytale:queen
    ["Do you have any idea how valuable grass is?"] = "Czy ty masz pojęcie jak cenna jest trawa?", -- A_Classic_Fairytale:enemy
    ["Do you have any idea what it's like in the village for a woman?"] = "Czy ty masz pojęcie jak żyje się kobietom w wiosce?", -- A_Classic_Fairytale:queen
    ["Do you know where they are?"] = "Czy wiesz, gdzie oni są?", -- A_Classic_Fairytale:queen
    ["Do you think you're some kind of god?"] = "Czy ty myślisz, że jesteś jakimś bogiem?", -- A_Classic_Fairytale:enemy
    ["Dragon's Lair"] = "Gniazdo smoka", -- A_Classic_Fairytale:dragon
    ["Dr. Banting"] = "Dr Benting",
    ["Dr. Barnard"] = "Dr Bernard",
    ["Dr. Blackwell"] = "Dr Blakłel",
    ["Dr. Cornelius"] = "Dr Korneliusz", -- A_Space_Adventure:cosmos, A_Space_Adventure:death01
    ["Dr. Crushing"] = "Dr Kraszing",
    ["Dr. Drew"] = "Dr Andrzej",
    ["Dr. Harvey"] = "Dr Harwej",
    ["Dr. Hollows"] = "Dr Holołs",
    ["Dr. Horace"] = "Dr Horacy",
    ["Drill Strike"] = "Wiertniczy nalot", -- Construction_Mode
    ["Drills"] = "Wiertła", -- A_Classic_Fairytale:backstab
    ["Dr. Jenner"] = "Dr Dżener",
    ["Dr. Jung"] = "Dr Dżang",
    ["Drone Hunter! +10 points!"] = "Łowca dronów! +10 punktów!", -- Space_Invasion
    ["Drop a ball of dirt which turns into a|cluster on impact. Doesn’t end turn."] = "Upuść kulę ziemi, która przy uderzeniu|zamieni się w odłamek. Nie kończy tury.", -- Continental_supplies
    ["Drop a bomb: [Drop some heroic wind that will turn into a bomb on impact]"] = "Upuść bombę: [Upuść trochę heroicznego wiatru, który przy uderzeniu zamieni się w bombę]", -- Continental_supplies
    ["- Dropped flags may be returned or recaptured"] = "- Upuszczone flagi mogą być zwrócone lub ponownie przejęte", -- Capture_the_Flag
    ["Dropping a weapon while in water would just drown it, but launching one would work."] = "Upuszczenie broni pod wodą by ją utopiłą, ale wystrzelenie zadziała.", -- Basic_Training_-_Flying_Saucer
    ["Drop weapon (while on rope): [Long Jump]"] = "Upuść broń (będąc na linie): [Długi Skok]", -- Basic_Training_-_Rope
    ["Drowner"] = "Topielec",
    ["Dr. Parkinson"] = "Dr Parkinson",
    ["Drunk greenhorn"] = "Pijany żółtodziób", -- User_Mission_-_RCPlane_Challenge
    ["Drunk with power, perhaps!"] = "Upił się mocą, być może!", -- A_Classic_Fairytale:queen
    ["%d sec"] = "%d sekund", -- Construction_Mode
    ["+%d seconds!"] = "%d sekund!", -- Tumbler
    ["Dubloon Devil"] = "Dublonowy Diabeł",
    ["Dude, all the plants are gone!"] = "Koleś, wszystkie rośliny zniknęły!", -- A_Classic_Fairytale:family
    ["Dude, can you see Ramon and Spiky?"] = "Koleś, widzisz Ramona i Kolczastego?", -- A_Classic_Fairytale:journey
    ["Dude, it's unbearable!"] = "Koleś, to jest nie do zniesienia!", -- A_Classic_Fairytale:queen
    ["Dude, let me out!"] = "Koleś, wypuść mnie!", -- A_Classic_Fairytale:epil
    ["Dude, that outfit is so cool!"] = "Koleś, ten kostium jest taki odjazdowy!", -- A_Classic_Fairytale:epil
    ["Dude, that's so cool!"] = "Koleś, to takie odlotowe!", -- A_Classic_Fairytale:backstab
    ["Dude, this is boring!"] = "Koleś, to jest nudne!", -- A_Classic_Fairytale:queen
    ["Dude, we really need a new shaman..."] = "Koleś, naprawdę potrzebujemy nowego szamana...", -- A_Classic_Fairytale:shadow
    ["Dude, what's this place?!"] = "Koleś, co to za miejsce?", -- A_Classic_Fairytale:dragon
    ["Dude, where are we?"] = "Koleś, gdzie my jesteśmy?", -- A_Classic_Fairytale:backstab
    ["Dude, wow! I just had the weirdest high!"] = "Koleś, wow! Miałem właśnie najdziwniejszy odlot!", -- A_Classic_Fairytale:backstab
    ["Dude, wow, you're so cute!"] = "Koleś, wow, jesteś taki słodki!", -- A_Classic_Fairytale:queen
    ["Dud Mine Placement Mode"] = "Tryb Ustawiania Niewybuchów", -- HedgeEditor
    ["DUD MINE PLACEMENT MODE"] = "TRYB USTAWIANIA NIEWYBUCHÓW", -- HedgeEditor
    ["Duration"] = "Czas trwania", -- Continental_supplies
    ["During the final testing of the device an accident happened."] = "Podczas finałowych testów urządzenia zdarzył się wypadek.", -- A_Space_Adventure:moon02
    ["During the game you can get new RC planes by collecting the weapon crates."] = "Podczas gry możesz zdobyć nowe samolociki, zbierając skrzynie z broniami.", -- A_Space_Adventure:desert03
    ["Dust Storm"] = "Burza pyłowa", -- Continental_supplies
    ["Dust storm: [Deals 15 damage to all enemies in the circle]"] = "Burza pyłowa: [Zadaje 15 obrażeń każdemu wrogowi w kole]", -- Continental_supplies
    ["Each time you destroy all the targets on your current level you'll get teleported to the next level."] = "Za każdym razem, gdy zniszczysz wszystkie cele na twoim aktualnym poziomie, zostaniesz przeteloportowany do następnego poziomu.", -- A_Space_Adventure:desert03
    ["Each time you play this missions enemy hogs will play in a random order."] = "Za każdym razem, gdy grasz w tę misję, wrogie jeże będą grały w losowym porządku.", -- A_Space_Adventure:death02
    ["Each turn is only ONE SECOND!"] = "Każda tura to tylko JEDNA SEKUNDA!", -- Frenzy
    ["Each turn you get 1-3 random weapons"] = "Z każdą turą dostaniesz 1-3 bronie",
    ["Each turn you get one random weapon"] = "Z każdą turą dostaniesz losową broń",
    ["Each turn you'll have only one rope to use."] = "W każdej turze będziesz mieć tylko jedną linę do użycia.", -- A_Space_Adventure:moon02
    ["Eagle Eye: [Blink to the impact ~ One shot]"] = "Sokole Oko: [Mrugnij do uderzenia ~ Jeden strzał]", -- Continental_supplies
    ["Eagle Eye"] = "Sokole Oko", -- A_Classic_Fairytale:backstab
    ["Ear Sniffer"] = "Wąchacz Uszu", -- A_Classic_Fairytale:backstab, A_Classic_Fairytale:epil
    ["EASY"] = "ŁATWY", -- Continental_supplies
    ["Eckles"] = "Ekles", -- User_Mission_-_Nobody_Laugh
    ["Eclipse"] = "Zaćmienie", -- Big_Armory
    ["Editing Commands: (Use while no weapon is selected)"] = "Komendy edytowania: (Użyj ,gdy nie wybrano broni)", -- HedgeEditor
    ["Ehm, okay ..."] = "Em, dobrze...", -- A_Space_Adventure:moon01
    ["Elderbot"] = "Starszyrobot", -- A_Classic_Fairytale:family
    ["Elimate your captor."] = "Wyeliminuj swojego porywacza.", -- User_Mission_-_The_Great_Escape
    ["Eliminate all targets before your time runs out.|You have unlimited ammo for this mission."] = "Wyeliminuj wszystkie cele, zanim upłynie czas.|W tej misji masz nieskończoną ilość amunicji.",
    ["Eliminate the enemy before the time runs out."] = "Wyeliminuj wroga, zanim upłynie czas.", -- User_Mission_-_Diver, User_Mission_-_Spooky_Tree
    ["Eliminate the enemy hogs to win."] = "Wybij jeże przeciwnika by wygrać.",
    ["Eliminate the enemy."] = "Wyeliminuj wroga.", -- User_Mission_-_Bamboo_Thicket, User_Mission_-_Newton_and_the_Hammock, User_Mission_-_Nobody_Laugh
    ["Eliminate Unit 3378."] = "Wyeliminuj Jednostkę 3378.", -- User_Mission_-_Teamwork
    ["Eliminate WatchBot 4000."] = "Wyeliminuj PaczBota 4000.", -- User_Mission_-_Teamwork_2
    ["Eliminate your captor."] = "Wyeliminuj porywacza.", -- User_Mission_-_The_Great_Escape
    ["Elite pilot"] = "Elitarny pilot", -- User_Mission_-_RCPlane_Challenge
    ["Elmo"] = "Elmo", -- A_Classic_Fairytale:dragon, A_Classic_Fairytale:family, A_Classic_Fairytale:queen
    ["Enabled"] = "Włączone", -- WxW
    ["Enemy kills: Collect victim's weapons and +%d%% of its base health"] = "Zabójstwa wroga: Zabierz bronie ofiary i +%d%% jej bazowego zdrowia", -- Battalion
    ["Energetic Engineer"] = "Inżynier Energetyczny",
    ["Engineer"] = "Inżynier", -- HedgeEditor, The_Specialists
    ["Enjoy the swim..."] = "Popływaj trochę...",
    ["Entered boredom phase! Discrepancies detected …"] = "Wchodzę w fazę nudy! Wykryto rozbieżności...", -- A_Classic_Fairytale:queen
    ["Epilogue"] = "Epilog", -- A_Classic_Fairytale:epil
    ["ERROR [getHogInfo]: Hog is nil!"] = "BŁĄD [getHogInfo]: Jeż jest nil!", -- Battalion
    ["Eugene"] = "Eugeniusz",
    ["Europe"] = "Europa", -- Continental_supplies
    ["Everyone knows this."] = "Każdy to wie.", -- A_Classic_Fairytale:enemy
    ["Every single time!"] = "Za każdym razem!", -- A_Classic_Fairytale:dragon
    ["Everything looks OK..."] = "Wszystko wygląda w porządku...", -- A_Classic_Fairytale:enemy
    ["Every time you kill an enemy hog your ammo will get reset next turn."] = "Za każdym razem, gdy zabijesz wrogiego jeża, twoja amunicja zresetuje się w następnej turze.", -- A_Space_Adventure:death02
    ["Everywhere I look, I see hogs walking around …"] = "Wszędzie, gdzie spojrzę, widzę chodzące jeże...", -- A_Classic_Fairytale:epil
    ["Exactly, man! That was my dream."] = "Dokładnie ziom! To był mój sen.", -- A_Classic_Fairytale:backstab
    ["Except me, of course! I just saved a whole planet!"] = "Oprócz mnie, oczywiście! Właśnie uratowałem całą planetę!", -- A_Space_Adventure:final
    ["Experienced beginner"] = "Doświadczony początkujący", -- User_Mission_-_RCPlane_Challenge
    ["Explore the tunnel with the other hedgehogs and search for the device."] = "Przeszukaj tunel z innymi jeżami i poszukaj urządzenia.", -- A_Space_Adventure:fruit02
    ["Exploring the tunnel"] = "Eksploracja tunelu", -- A_Space_Adventure:fruit02
    ["Eye Chewer"] = "Przeżuwacz Oczu", -- A_Classic_Fairytale:journey
    ["Fair Wind"] = "Sprawiedliwy Wiatr",
    ["Fall Damage"] = "Obrażenia od upadku", -- Basic_Training_-_Movement
    ["Fallen Angel"] = "Upadły anioł", -- Tentacle_Terror
    ["Family Reunion"] = "Spotkanie rodzinne", -- A_Classic_Fairytale:family
    ["Fastest escape: %d turns"] = "Najszybsza ucieczka: %d tur", -- A_Space_Adventure:desert02
    ["Fastest lap: %.3fs by %s"] = "Najszybsze okrążenie: %.3fs (%s)", -- TrophyRace
    ["Fastest lap: "] = "Najszybsze okrążenie: ",
    ["Feeble Resistance"] = "Ruch Oporu",
    ["Fell From Grace"] = "Upadły z Łaski", -- A_Classic_Fairytale:backstab, A_Classic_Fairytale:dragon, A_Classic_Fairytale:enemy, A_Classic_Fairytale:epil, A_Classic_Fairytale:family, A_Classic_Fairytale:queen
    ["Fell From Heaven is the best! Fell From Heaven is the greatest!"] = "Upadły z Niebios jest najlepszy! Upadły z Niebios jest najświetniejszy!", -- A_Classic_Fairytale:family
    ["Fell From Heaven"] = "Upadły z Niebios", -- A_Classic_Fairytale:enemy, A_Classic_Fairytale:epil, A_Classic_Fairytale:family, A_Classic_Fairytale:first_blood, A_Classic_Fairytale:journey, A_Classic_Fairytale:queen
    ["Femur Lover"] = "Wielbiciel Ud", -- A_Classic_Fairytale:shadow
    ["Fierce Competition! +8 points!"] = "Zacięta rywalizacja! +8 punktów!", -- Space_Invasion
    ["Fiery Water?! Are you drunk again?"] = "Ognista Woda? Znowu jesteś pijany?", -- A_Classic_Fairytale:queen
    ["Fiery Water"] = "Ognista Woda", -- A_Classic_Fairytale:backstab, A_Classic_Fairytale:dragon, A_Classic_Fairytale:enemy, A_Classic_Fairytale:epil, A_Classic_Fairytale:family, A_Classic_Fairytale:queen, A_Classic_Fairytale:united
    ["Fighting instead of cultivating a beautiful friendship."] = "Walcząc, zamiast podtrzymywać piękną przyjaźń.", -- A_Classic_Fairytale:epil
    ["Fight: Press [Attack]"] = "Walcz: Wciśnij [Atak]", -- A_Space_Adventure:fruit01
    ["Filthy Blue"] = "Brudny Niebieski", -- User_Mission_-_Dangerous_Ducklings
    ["Final Challenge:"] = "Ostatnie wyzwanie:", -- Basic_Training_-_Rope
    ["Finally! We're out of this hellhole. Now go save the princess, %s!"] = "Nareszcie! Wydostaliśmy się z tego piekła. Teraz idźmy uratować księżniczkę, %s!", -- A_Classic_Fairytale:family
    ["Finally you are here!"] = "Nareszcie tu jesteś!", -- A_Space_Adventure:desert01, A_Space_Adventure:ice01
    ["Final result"] = "Końcowy rezultat", -- Mutant
    ["Final Targets"] = "Ostatnie cele", -- Basic_Training_-_Bazooka, Basic_Training_-_Grenade
    ["Final team scores:"] = "Końcowe wyniki drużyn:", -- Space_Invasion
    ["Find all the parts of the anti-gravity device."] = "Znajdź wszystkie części urządzenia antygrawitacyjnego.", -- A_Space_Adventure:cosmos
    ["Find a way to detonate all the explosives and stay alive!"] = "Znajdź sposób, by zdetonować wszystkie ładunki wybuchowe i przeżyć!", -- A_Space_Adventure:final
    ["Find your tribe!|Cross the lake!"] = "Znajdź swoje plemie!|Przekrocz jezioro!", -- A_Classic_Fairytale:dragon
    ["Finish this challenge as fast as possible to earn bonus points."] = "Ukończ to wyzwanie najszybciej, jak to możliwe, by zdobyć dodaktowe punkty.", -- User_Mission_-_Rope_Knock_Challenge
    ["Finish waypoint placement"] = "Zakończ stawianie punktów kontrolnych", -- Racer
    ["Finish your training."] = "Ukończ swój trening.", -- A_Classic_Fairytale:first_blood
    ["Finite Ropes"] = "Skończone liny", -- Basic_Training_-_Rope
    ["Fire a rocket with napalm."] = "Strzel rakietą z napalmem.", -- Continental_supplies
    ["Fire: [Precise]"] = "Strzel: [Precyzja]", -- Space_Invasion, Tumbler
    ["Fire some exploding medicine that will heal 15 health to all hogs in its effect radius."] = "Wystrzel eksplodujące lekarstwo, które wyleczy 15 zdrowia wszystkim jeżom w promieniu efektu.", -- Continental_supplies
    ["Fire your hedgehog like a sticky mine."] = "Wystrzel swojego jeża jak minę samoprzylepną.", -- Continental_supplies
    ["First aid kits?!"] = "Apteczki?", -- A_Classic_Fairytale:united
    ["First Blood"] = "Pierwsza krew", -- A_Classic_Fairytale:first_blood
    ["- First clan to capture the flag wins"] = "- Pierwszy klan, który przejmie flagę, wygrywa", -- Capture_the_Flag
    ["- First clan to score %d captures wins"] = "Pierwszy klan, który przejmie %d flag, wygrywa", -- Capture_the_Flag
    ["First killer will mutate"] = "Pierwszy zabójca zmutuje", -- Mutant
    ["First Steps"] = "Pierwsze kroki", -- A_Classic_Fairytale:first_blood
    ["- First team to capture the flag wins"] = "Pierwsa drużyna, która przejmie flagę, wygrywa", -- Capture_the_Flag
    ["- First team to score %d captures wins"] = "Pierwsa drużyna, która przejmie flagę %d razy, wygrywa", -- Capture_the_Flag
    ["Fishy"] = "Podejrzany",
    ["Flag captured!"] = "Flaga przechwycona!",
    ["Flag respawned!"] = "Flaga przywrócona!",
    ["Flag returned!"] = "Flaga odzyskana!",
    ["Flamer"] = "Miotacz ognia",
    ["Flaming Worm"] = "Płomienisty Robal", -- A_Classic_Fairytale:backstab
    ["Flare"] = "Flara", -- Continental_supplies
    ["Flawless victory!"] = "Bezbłędne zwycięstwo!", -- User_Mission_-_RCPlane_Challenge
    ["Flee: Press [Jump]"] = "Ucieknij: Wciśnij [Skok]", -- A_Space_Adventure:fruit01
    ["Flesh for Brainz"] = "Mięso dla Mózgów", -- A_Classic_Fairytale:journey
    ["Fly around and hurl explosives to your enemies."] = "Lataj dookoła i ciskaj ładunki wybuchowe na swoich wrogów.", -- Tumbler
    ["Flying Saucer Training"] = "Trening latającego talerza", -- Basic_Training_-_Flying_Saucer
    ["Fly into space to fight off the invaders with barrels!"] = "Leć w kosmos, by odeprzeć najeźdźców beczkami!", -- Space_Invasion
    ["Fly to the meteorite and detonate the explosives"] = "Poleć do meteorytu i zdetonuj ładunki", -- A_Space_Adventure:cosmos
    ["Follow the path and destroy the next target."] = "Podążaj ściezką i zniszcz następny cel.", -- Basic_Training_-_Rope
    ["Forgetfulness: You will lose all your weapons each turn."] = "Zapominalstwo: W każdej turze stracisz wszystkie swoje bronie.", -- Continental_supplies
    ["For the next crate, you have to do back jumps."] = "Do następnej skrzyni musisz zrobić skoki w tył.", -- Basic_Training_-_Movement
    ["Four Eyes"] = "Czterooki",
    ["Frank"] = "Franek", -- User_Mission_-_Nobody_Laugh
    ["Frankie"] = "Franka",
    ["Free Dense Cloud and continue the mission!"] = "Uwolnij Gęstą Chmurę i kontynuuj misję!", -- A_Classic_Fairytale:journey
    ["FRENZY"] = "SZAŁ", -- Frenzy
    ["Friendly Fire!"] = "Przyjacielski Ogień!",
    ["Friendly kills: Clear killer's pool and -%d%% of its base health"] = "Przyjacielskie zabójstwa: Wyczyść pulę zabójcy i -%d%% jego bazowego zdrowia", -- Battalion
    ["From the second turn and beyond the water rises."] = "Od drugiej tury woda się podwyższa.", -- A_Space_Adventure:desert02
    ["Frozen Bandits"] = "Zamrożeni Bandyci", -- A_Space_Adventure:ice01
    ["Fruit Assassins"] = "Owocowi Zabójcy", -- A_Space_Adventure:fruit02
    ["Fruit"] = "Owoc",
    ["Fruity"] = "Owocny",
    ["Fuel: %d"] = "Paliwo: %d", -- Tumbler
    ["Fuzzy Beard"] = "Puszystobrody",
    ["“g=150”, where 150 is 150% of normal gravity."] = "\"g=150\", gdzie 150 to 150% normalnej grawitacji.", -- Gravity
    ["“g=50, g2=150, period=4000” for gravity changing|from 50 to 150 and back with period of 4000 ms."] = "\"g=50, g2=150, period=4000\" dla grawitacji zmieniającej|się od 50 do 150 i z powrotem, z okresem 4000 ms.", -- Gravity
    ["Galaxy Guardians"] = "Galaktyczni Strażnicy", -- Big_Armory
    ["Game over!"] = "Koniec gry", -- Space_Invasion
    ["Game Started!"] = "Gra rozpoczęta!",
    ["Game? Was this a game to you?!"] = "Gra? To była dla ciebie gra?", -- A_Classic_Fairytale:enemy
    ["Gangsters"] = "Gangsterzy",
    ["GasBomb"] = "Bomba Gazowa", -- Continental_supplies
    ["Gas Gargler"] = "Gazowy Płukacz", -- A_Classic_Fairytale:queen
    ["Gasp! A smuggler!"] = "*dyszy* Przemytnik!", -- A_Space_Adventure:desert01
    ["Gasp!"] = "*dyszy*", -- A_Space_Adventure:desert01
    ["Gathering fruits all day long."] = "Zbieram owoce przez całe dnie.", -- A_Classic_Fairytale:queen
    ["Gear Placement Tool"] = "Narzędzie Ustawiania Rzeczy", -- HedgeEditor
    ["General information"] = "Ogólne informacje", -- Continental_supplies
    ["General information:"] = "Ogólne informacje:", -- Continental_supplies
    ["General Lemon"] = "Generał Cytryna", -- A_Space_Adventure:fruit01
    ["Generator: Generates energy."] = "Generator: Generuje energię.", -- Construction_Mode
    ["Generator"] = "Generator", -- Construction_Mode
    ["Get Dense Cloud out of the pit!"] = "Wydostań Gęstą Chmurę z dołu!", -- A_Classic_Fairytale:journey
    ["Get him, Spike!"] = "Bierz go, Kolczasy!", -- A_Space_Adventure:desert01
    ["Get on over there and take him out!"] = "Dostań się tam i go wykończ!",
    ["Get on the head of the mole."] = "Wejdź na głowę jeża.", -- A_Classic_Fairytale:first_blood
    ["Get past the flower."] = "Przejdź przez kwiatek.", -- A_Classic_Fairytale:journey
    ["Get ready to fight!"] = "Przygotuj się do walki!", -- A_Space_Adventure:moon01
    ["Get that crate!"] = "Zdobądź tę skrzynię!", -- A_Classic_Fairytale:first_blood
    ["Get the crate on the other side of the island!|"] = "Zdobądź skrzynię na drugim końcu wyspy!|", -- A_Classic_Fairytale:journey
    ["Get the crate on the other side of the island."] = "Zdobądź skrzynię na drugim końcu wyspy.", -- A_Classic_Fairytale:journey
    ["Get the final crate to the right to complete the training."] = "Zdobądź ostatnią skrzynię po prawej, by ukończyć trening.", -- Basic_Training_-_Movement
    ["Get the highest score to win."] = "Zdobądź najwyższy wynik, by wygrać.", -- Space_Invasion
    ["Get the next crate by jumping over the abyss."] = "Zdobądź następną skrzynię, skacząc nad przepaścią.", -- Basic_Training_-_Movement
    ["Getting ready"] = "Przygotowanie", -- A_Space_Adventure:cosmos, A_Space_Adventure:desert01, A_Space_Adventure:desert02, A_Space_Adventure:ice01, A_Space_Adventure:ice02, A_Space_Adventure:moon01
    ["Getting Started"] = "Rozgrzewka", -- Basic_Training_-_Rope
    ["Getting to the device"] = "Dojście do urządzenia", -- A_Space_Adventure:fruit02
    ["Get to the crate using your flying saucer!"] = "Dostań się do skrzyni, używając swojego latającego talerza!", -- Basic_Training_-_Flying_Saucer
    ["Get to the target using your rope!"] = "Dostań się do celu, używając swojej liny!", -- Basic_Training_-_Rope
    ["Get your teammates out of their natural prison and save the princess!|Hint: Drilling holes should solve everything.|Hint: It might be a good idea to place a girder before starting to drill. Just saying.|Hint: All your hedgehogs need to be above the marked height!|Hint: Leaks A Lot needs to get really close to the princess!"] = "Wydostań swoich kolegów z ich naturalnego więzienia i uratuj księżniczkę!|Porada: Wywiercenie dziur powinno wszystko rozwiązać.|Porada: Może być dobrym pomysłem, by ustawić belkę przez rozpoczęciem wiercenia. Tylko mówię.|Porada: Wszystkie twoje jeże powiny być ponad zaznaczoną wysokością!|Porada: Spory Przeciek musi dostać się naprawdę blisko księżniczki!", -- A_Classic_Fairytale:family
    ["Get your teammates out of their natural prison and save the princess!"] = "Wydostań swoich kolegów z ich naturalnego więzienia i uratuj księżniczkę!", -- A_Classic_Fairytale:family
    ["Giggles"] = "Chichot",
    ["Gimme Bones"] = "Dej Kości", -- A_Classic_Fairytale:backstab
    ["Girder"] = "Belka", -- Construction_Mode
    ["Girder Placement Mode"] = "Tryb Stawiania Belek", -- Construction_Mode
    ["GIRDER PLACEMENT MODE"] = "TRYB STAWIANIA BELEK", -- HedgeEditor
    ["Give a hog a preset identity and weapons"] = "Daj jeżowi ustaloną tożsamość i bronie", -- HedgeEditor
    ["Give an entire team themed hats and names"] = "Daj całej drużynie tematyczne czapki i imiona", -- HedgeEditor
    ["Glark"] = "Glark", -- A_Classic_Fairytale:shadow
    ["Glasses"] = "Okulary",
    ["Glassy"] = "Szklisty",
    ["Goal Definition Mode"] = "Tryb Definicji Celu", -- HedgeEditor
    ["GOAL DEFINITION MODE"] = "TRYB DEFINICJI CELU", -- HedgeEditor
    ["Goal: Score %d points or more to win!"] = "Cel: Zdobądź %d lub więcej punktów, by wygrać!", -- Mutant
    ["Go and collect the crate"] = "Idź i zbierz skrzynię", -- A_Space_Adventure:cosmos
    ["Godai"] = "Godai",
    ["Go down and save these PAotH hogs!"] = "Zejdź na dół i uratuj te jeże z PSJ!", -- A_Space_Adventure:moon01
    ["Go, get him again!"] = "Idźcie, złapcie go znowu!", -- A_Space_Adventure:moon02
    ["Goggles"] = "Gogl",
    ["Goggs"] = "Gogs",
    ["GO! GO! GO!"] = "RUCHY! RUCHY! RUCHY!",
    ["Good birdy......"] = "Dobry ptaszek......",
    ["Good bye!"] = "Do widzenia!", -- Basic_Training_-_Flying_Saucer
    ["Good idea, they'll never find us there!"] = "Dobry pomysł, nigdy nas tu nie znajdą!", -- A_Classic_Fairytale:united
    ["Good job! Defeat the rest of the aliens!"] = "Dobra robota! Pokonaj resztę obcych!", -- A_Classic_Fairytale:queen
    ["Good job!"] = "Dobra robota!", -- Basic_Training_-_Flying_Saucer, Basic_Training_-_Rope
    ["Good job! Now destroy the final targets to finish the training."] = "Dobra robota! Teraz zniszcz ostatnie cele, by ukończyć trening.", -- Basic_Training_-_Grenade
    ["Good luck...or else!"] = "Powodzenia... lub nie!", -- A_Classic_Fairytale:journey
    ["Good luck out there!"] = "Powodzenia tam!",
    ["Good luck!"] = "Powodzenia!", -- A_Space_Adventure:desert01, A_Space_Adventure:fruit02
    ["Good so far!"] = "Jak dotąd idzie dobrze!",
    ["Good to go!"] = "Gotowi do akcji!",
    ["Good! You now control Cappy."] = "Dobrze! Teraz kontrolujesz Czapeczkę.", -- Basic_Training_-_Movement
    ["Go on top of the flower."] = "Wejdź na szczyt kwiatka.", -- A_Classic_Fairytale:first_blood
    ["Go, quick!"] = "Idź szybko!", -- A_Classic_Fairytale:backstab
    ["Gorkij"] = "Gorkij", -- A_Classic_Fairytale:journey
    ["Go surf!"] = "Idź serfować!", -- WxW
    ["Got 1 more saucer and 8 more seconds added to the clock"] = "Zdobywasz 1 talerz więcej i 8 więcej sekund dodane do zegara", -- A_Space_Adventure:ice02
    ["Got 1 more saucer"] = "Zdobywasz 1 talerz więcej", -- A_Space_Adventure:ice02
    ["GOTCHA!"] = "MAM CIĘ!",
    ["Go to Thanta and get the device part!"] = "Idź do Tanty i zdobądź część urządzenia!", -- A_Space_Adventure:ice01
    ["Go to the surface!"] = "Idź na powierzchnię!", -- A_Space_Adventure:fruit02
    ["Go to the target."] = "Idź do celu.", -- Basic_Training_-_Rope
    ["Go to the upper platform and get the weapons in the crates!"] = "Idź na górną platformę i zdobądź bronie ze skrzyń!", -- A_Space_Adventure:moon01
    ["Got the saucer!"] = "Dotarto do talerza!", -- A_Space_Adventure:cosmos
    ["Got to go back."] = "Muszę wracać.", -- A_Space_Adventure:cosmos
    ["Got you? You're acting weird."] = "Dopadło cię? Dziwnie się zachowujesz.", -- A_Classic_Fairytale:queen
    ["Grab mines/barrels: [High jump]"] = "Złap miny/beczki: [Wysoki skok]", -- Tumbler
    ["Gravity: 100%"] = "Grawitacja: 100%", -- Gravity
    ["Great choice, Steve! Mind if I call you that?"] = "Świetny wybór, Stefan! Masz coś przeciwko nazywaniu cię tak?", -- A_Classic_Fairytale:shadow
    ["Great! Let’s kill all these enemies, using portals."] = "Świetnie! Zabijmy wszystkich tych wrogów, używając portali.", -- portal
    ["Great!"] = "Świetnie!", -- Basic_Training_-_Rope
    ["Great work! Now hit it with your Baseball Bat! |Tip: You can change weapon with 'Right Click'!"] = "Świetna robota! Teraz uderz go swoim kijem bejsbolowym!|Porada: Możesz zmienić broń 'Prawym kliknięciem'!", -- Basic_Training_-_Rope
    ["Great! You will be contacted soon for assistance."] = "Świetnie! Niedługo skontaktujemy się z tobą po pomoc.", -- A_Classic_Fairytale:shadow
    ["Green Bananas"] = "Zielone Banany", -- A_Space_Adventure:fruit01
    ["Green double rings also give you a new flying saucer."] = "Zielone podwójne pierścienie również dają nowy latający talerz.", -- A_Space_Adventure:ice02
    ["Green Hog Grape"] = "Zielonojeżowy Winogron", -- A_Space_Adventure:fruit01
    ["Green hogs won't intentionally hurt you."] = "Zielone jeże nie zaatakują cię umyślnie.", -- A_Space_Adventure:fruit01
    ["Greenhorn"] = "Żółtodziób", -- User_Mission_-_RCPlane_Challenge
    ["Green Lipstick Bullet"] = "Pocisk zielonej szminki", -- Continental_supplies
    ["Green lipstick bullet: [Poisonous, deals no damage]"] = "Zielony szminkowy pocisk: [Zatruty, nie zadaje obrażeń]", -- Continental_supplies
    ["Green"] = "Zielony",
    ["Greetings, cloudy one!"] = "Witaj, chmurzysty", -- A_Classic_Fairytale:shadow
    ["Greetings from the Navy, %s (%s), for being a distance of %d away from the mainland!"] = "Pozdrowienia od Marynarki, %s (%s), za bycie w odległości %d od lądu!", -- ClimbHome
    ["Greetings, %s!"] = "Pozdrowienia, %s!", -- A_Classic_Fairytale:dragon
    ["Grenades explode after 1 to 5 seconds (you decide)."] = "Granat wybucha po 1 do 5 sekundach (ty wybierasz).", -- Basic_Training_-_Grenade
    ["Grenades with high bounciness bounce a lot and behave chaotic."] = "Granat z dużą odbijalnością odbija się bardzo dużo i zachowuje chaotycznie.", -- Basic_Training_-_Grenade
    ["Grenade Training"] = "Trening Granatowy", -- Basic_Training_-_Grenade
    ["Grenadier"] = "Grenadier", -- Target_Practice_-_Grenade_easy, Target_Practice_-_Grenade_hard, HedgeEditor
    ["Grenadiers"] = "Grenadierzy", -- Basic_Training_-_Grenade
    ["Grey"] = "Szary",
    ["Guards"] = "Strażnicy", -- A_Space_Adventure:cosmos
    ["Guile"] = "Wilhelm",
    ["Guys, do you think there's more of them?"] = "Ziomki, myślicie, że jest ich więcej?", -- A_Classic_Fairytale:backstab
    ["Haha! Come!"] = "Haha! Chodź!", -- A_Classic_Fairytale:queen
    ["HAHA!"] = "HAHA!", -- A_Classic_Fairytale:enemy
    ["Haha!"] = "Haha!", -- A_Classic_Fairytale:united
    ["Hahahaha!"] = "Hahahaha!",
    ["Haha, I love the look on your face!"] = "Haha, uwielbiam to spojrzenie na twojej twarzy!", -- A_Classic_Fairytale:queen
    ["Haha, now THAT would be something!"] = "Haha, to było by COŚ",
    ["Haha, that was just a coincidence!"] = "Haha, to był tylko przypadek!", -- A_Classic_Fairytale:queen
    ["Hammer"] = "Młotek", -- Construction_Mode, Continental_supplies
    ["Hannibal"] = "Hannibal", -- A_Classic_Fairytale:epil
    [" Hapless Hogs left!"] = " Nieszczęsne Jeże pozostały",
    ["Hapless Hogs"] = "Nieszczęsne Jeże",
    ["Happy with your race track?|Then stop building and start racing!"] = "Zadowolony ze swojej trasy?|Więc przestań budować i zacznij wyścig!", -- Racer
    ["Hard flying"] = "Twarde latanie", -- A_Space_Adventure:ice02
    ["HARD"] = "TRUDNY", -- Continental_supplies
    ["Harris"] = "Harys",
    ["Harry"] = "Henryk", -- User_Mission_-_Nobody_Laugh
    ["Harry Potter"] = "Harry Potter",
    ["Hatless Jerry"] = "Bezczapkowy Jerry", -- A_Classic_Fairytale:queen
    ["Have no illusions, your tribe is dead, indifferent of your choice."] = "Nie miej złudzeń, twoje plemię jest martwe, niezależnie od twojego wyboru.", -- A_Classic_Fairytale:shadow
    ["Haven't found it yet ..."] = "Jeszcze tego nie znalazłem...", -- A_Space_Adventure:desert01
    ["Have we ever attacked you first?"] = "Czy kiedykolwiek zaatakowaliśmy pierwsi?", -- A_Classic_Fairytale:enemy
    ["H confirmed that there isn't such a PAotH activity logged."] = "J potwierdził, że nie ma zapisanej takiej aktywności PSJ.", -- A_Space_Adventure:desert01
    ["Healing Station: Heals nearby hogs."] = "Stacja Lecząca: Leczy pobliskie jeże.", -- Construction_Mode
    ["Healing Station"] = "Stacja Lecząca", -- Construction_Mode
    ["Health and Mission Panel"] = "Panel zdrowia i misji", -- Basic_Training_-_Movement
    ["Health Crate Placement Mode"] = "Tryb Stawiania Apteczek", -- Construction_Mode
    ["HEALTH CRATE PLACEMENT MODE"] = "TRYB STAWIANIA APTECZEK", -- HedgeEditor
    ["Health: %d"] = "Zdrowie: %d", -- HedgeEditor
    ["Health: Hogs lose up to 7% base health per turn"] = "Zdrowie: Jeże tracą do 7% bazowego zdrowia co turę", -- Battalion
    ["Health Modification Mode"] = "Tryb Modyfikacji Zdrowia", -- HedgeEditor
    ["HEALTH MODIFICATION MODE"] = "TRYB MODYFIKACJI ZDROWIA", -- HedgeEditor
    ["Health"] = "Zdrowie", -- Basic_Training_-_Movement
    ["Heavenly Defense"] = "Niebiańska obrona", -- Tentacle_Terror
    ["Heavy Cannfantry"] = "Ciężka Kanibaleria", -- A_Classic_Fairytale:united
    ["Heavy"] = "Ciężki",
    ["Heckles"] = "Hekles",
    ["Heck, you even executed one of your own!"] = "Do licha, nawet wykonaliście wyrok na jednym ze swoich!", -- A_Classic_Fairytale:queen
    ["Hedge-cogs"] = "Jeże Zębate", -- A_Classic_Fairytale:enemy
    ["HEDGEEDITOR"] = "JEŻEDYTOR", -- HedgeEditor
    ["HedgeEditor tool"] = "Narzędzie JeżEdytora", -- HedgeEditor
    ["Hedgehog"] = "Jeż",
    ["Hedgehog projectile: [Fire your hog like a Sticky Bomb]"] = "Jeżowy pocisk: [Wystrzel swojego jeża jak minę samoprzylepną]", -- Continental_supplies
    ["Hedgehog Projectile"] = "Pocisk jeżowy", -- Continental_supplies
    ["Hedgehogs can not be deleted."] = "Jeże nie mogą być usunięte", -- HedgeEditor
    ["Hedgehogs left: %d"] = "Pozostałe jeże: %d", -- User_Mission_-_That_Sinking_Feeling
    ["Hedgehogs will be revived after their death."] = "Jeże zostaną ożywione po śmierci.", -- Mutant
    ["Hedgehogs will start in the first waypoint."] = "Jeże zaczną na pierwszym punkcie kontrolnym.", -- Racer
    ["Hedgibal Lecter"] = "Hanijeż Lekter", -- A_Classic_Fairytale:backstab
    ["He doesn't know it but this device is a part of the anti-gravity device."] = "On tego nie wie, ale to urządzenie jest częścią urządzenia antygrawitacyjnego.", -- A_Space_Adventure:ice01
    ["He has captured the rest of the PAotH team and awaits to capture you!"] = "On już porwał resztę zespołu PSJ i teraz oczekuje, aż porwie ciebie!", -- A_Space_Adventure:moon01
    ["Heh, it's not that bad."] = "Heh, nie jest aż tak źle.",
    ["Height over time"] = "Wysokość według czasu", -- ClimbHome
    ["He is a very tough and very determined hedgehog. I would be extremely careful if I were you."] = "On jest bardzo twardym i zdeterminowanym jeżem. Na twoim miejscu byłbym bardzo ostrożny.", -- A_Space_Adventure:moon02
    ["Helena"] = "Helena", -- A_Space_Adventure:moon01
    ["Hell Army"] = "Armia Piekieł", -- portal
    ["Hello again, %s!"] = "Witaj znowu, %s!", -- A_Classic_Fairytale:family
    ["Help Disabled"] = "Pomoc wyłączona", -- HedgeEditor
    ["Help Enabled"] = "Pomoc włączona", -- HedgeEditor
    ["Helpers: Each team starts with %d helper points"] = "Pomocnicy: Każda drużyna zaczyna z %d punktami pomocniczymi", -- Battalion
    ["Helpers: Hogs will get 1 out of 2 helpers randomly each turn"] = "Pomocnicy: Jeże dostaną losowo 1 z 2 pomocników co turę", -- Battalion
    ["Help me, Leaks!"] = "Przeciek, pomóż mi!", -- A_Classic_Fairytale:journey
    ["Help me, please!!!"] = "Pomóż mi, proszę!!!", -- A_Classic_Fairytale:journey
    ["Help me, please!"] = "Pomóż mi, proszę!", -- A_Classic_Fairytale:journey
    ["He moves like an eagle in the sky."] = "Porusza się on jak orzeł po niebie.", -- A_Classic_Fairytale:first_blood
    ["He must be in the village already."] = "On musi już być w wiosce.", -- A_Classic_Fairytale:journey
    ["HeneK"] = "HeneK",
    ["Here, let me help you save her!"] = "Tutaj, daj mi pomóc ci ją uratować!", -- A_Classic_Fairytale:family
    ["Here, let me help you!"] = "Tutaj, daj mi sobie pomóc!", -- A_Classic_Fairytale:backstab
    ["Here...pick your weapon!"] = "Masz... wybierz swoją broń!", -- A_Classic_Fairytale:first_blood
    ["Here! Take it!"] = "Masz! Weź to!", -- A_Space_Adventure:ice01
    ["Here we go!"] = "No to ruszamy!", -- A_Space_Adventure:moon01
    ["Here you will find the current mission instructions."] = "Tutaj znajdziesz instrukcje do aktualnej misji", -- Basic_Training_-_Movement
    ["Here you will learn how to fly the flying saucer|and get so learn some cool tricks."] = "Tutaj poznasz jak lata się latającym talerzem|i będziesz mieć szansę poznać odlotowe sztuczki.", -- Basic_Training_-_Flying_Saucer
    ["Heroic Wind"] = "Heroiczny wiatr", -- Continental_supplies
    ["He's so brave..."] = "Jest taki odważny...", -- A_Classic_Fairytale:first_blood
    ["He was the lab assistant of Dr. Goodhogan, the inventor of the anti-gravity device."] = "Był asystentem laboratoryjnym Dr Dobrojeża, wynalazcy urządzenia antygrawitacyjnego.", -- A_Space_Adventure:moon02
    ["He won't be selling us out anymore!"] = "On już nas nie sprzeda!", -- A_Classic_Fairytale:backstab
    ["Hey, don't forget us! We still need to climb up!"] = "Hej, nie zapomnij o nas! Nadal potrzebujemy się wspiąć!", -- A_Classic_Fairytale:family
    ["Hey, guys!"] = "Hej, koledzy!", -- A_Classic_Fairytale:backstab
    ["Hey guys!"] = "Hej koledzy!", -- A_Classic_Fairytale:united
    ["Hey! I was supposed to collect it!"] = "Hej! Miałem ją zebrać!", -- A_Space_Adventure:fruit02
    ["Hey, %s! Finally you have come!"] = "Hej, %s! W końcu przyszedłeś!", -- A_Space_Adventure:moon01
    ["Hey, %s! Look, someone is stealing the saucer!"] = "Hej, %s! Zobacz, ktoś kradnie nasz talerz!", -- A_Space_Adventure:cosmos
    ["Hey! This is cheating!"] = "Hej! To oszukiwanie!", -- A_Classic_Fairytale:journey
    ["Hidden"] = "Ukryte", -- portal
    ["High Gravity: Gravity is %i%%"] = "Wysoka grawitacja: Grawitacja wynosi %i%%", -- Gravity
    ["High Jump: [Backspace]"] = "Wysoki skok: [Backspace]", -- Basic_Training_-_Movement
    ["High Jump: Tap the [Curvy Arrow] shortly"] = "Wysoki skok: Wciśnij krótko [Zakręconą strzałkę]", -- Basic_Training_-_Movement
    ["Highlander: Eliminate hogs to take their weapons"] = "Highlander: Wyeliminuj jeże, by zabrać ich bronie", -- Highlander
    ["Highland: Hogs get %d random weapons from their pool"] = "Wyżyna: Jeże dostają %d losowych broni ze swoich pul", -- Battalion
    ["--- Highland Mode ---"] = "Tryb wyżyny", -- Battalion
    ["--- Highland ---"] = "--- Wyżyna ---", -- Battalion
    ["High Target"] = "Wysoki cel", -- Basic_Training_-_Bazooka
    ["Hightime"] = "Najwyższa pora", -- A_Classic_Fairytale:first_blood
    ["Hightower"] = "Wysowieża",
    ["Hill Guard"] = "Straż górna", -- Bazooka_Battlefield
    ["Hi! Nice to meet you."] = "Cześć! Miło cię poznać.", -- A_Space_Adventure:ice01
    ["Hint: Cinematics can be skipped with the [Precise] key."] = "Porada: Przerywniki mogą zostać pominięte klawiszem [Precyzja].", -- A_Classic_Fairytale:first_blood, A_Classic_Fairytale:shadow
    ["Hint: Drilling holes should solve everything."] = "Wywiercenie dziur powinno wszystko rozwiązać.", -- A_Classic_Fairytale:family
    ["Hint: Hold down [M] to review the mission texts."] = "Wskazówka: Przytrzymaj [M], by zobaczyć teksty misji.", -- A_Classic_Fairytale:first_blood
    ["Hint: If this mission panel disappears, you can|see it again by hitting the Pause or Quit key."] = "Wskazówka: Jeśli panel misji zniknie, możesz zobaczyć go ponownie, wciskając Pauzę lub Wyjście.", -- Basic_Training_-_Movement
    ["Hint: It might be a good idea to place a girder before starting to drill. Just saying."] = "Wskazówka: To może być dobry pomysł, by ustawić belki przed rozpoczęciem wiercenia. Tylko mówię.", -- A_Classic_Fairytale:family
    ["Hint: It might be easier if you vary the angle only slightly."] = "Wskazówka: Może być łatwiej, jeśli zmienisz kąt tylko trochę.", -- Basic_Training_-_Bazooka
    ["Hint: Just select the parachute, it opens automatically when you fall."] = "Porada: Po prostu wybierz spadochron. Otwiera się automatycznie, gdy spadasz.", -- A_Classic_Fairytale:first_blood
    ["Hint: Kills won't transfer a hog's pool to the killer's pool"] = "Wskazówka: Zabójstwa nie przetransferują puli jeża do puli zabójcy", -- Battalion
    ["Hint: Launch the bazooka horizontally at full power."] = "Wskazówka: Wystrzel bazookę poziomo z pełną mocą.", -- Basic_Training_-_Bazooka
    ["Hint: Pause the game to review the mission texts."] = "Porada: Spauzuj grę, by zobaczyć tekst misji.", -- A_Classic_Fairytale:first_blood
    ["Hint: Select the blow torch, aim and press [Fire]. Press [Fire] again to stop."] = "Porada: Wybierz palnik, wyceluj i wciśnij [Strzał]. Wciśnij ponownie [Strzał], by się zatrzymać.", -- A_Classic_Fairytale:journey
    ["Hint: Select the low gravity and press [Fire]."] = "Porada: Wybierz niską grawitację i wciśnij [Strzał].", -- A_Classic_Fairytale:journey
    ["Hint: Select the rope, [Up] or [Down] to aim, [Attack] to fire, directional keys to move."] = "Porada: Wybierz linę, [Góra] lub [Dół], by celować, [Atak], by strzelić, klawisze kierunkowe, by się poruszać.", -- A_Classic_Fairytale:first_blood
    ["Hint: Select the Shoryuken and hit [Attack].|P.S.: You can use it mid-air."] = "Porada: Wybierz Ognistą pięść i wciśnij [Atak].|PS Możesz użyć jej w powietrzu.", -- A_Classic_Fairytale:first_blood
    ["Hint: %s needs to get really close to the princess!"] = "Wskazówka: %s potrzebuje dostać się naprawdę blisko księżniczki!", -- A_Classic_Fairytale:family
    ["Hint: The rope only bends around objects.|When it doesn't hit anything, it's always straight."] = "Wskazówka: Lina owija się jedynie wokół obiektów.|Jeśli nic nie uderzy, zawsze jest prosta.", -- Basic_Training_-_Rope
    ["Hint: To jump higher, wait a bit before you hit “High Jump” a second time."] = "Wskazówka: By skoczyć wyżej, poczekaj trochę zanim ponownie wciśniesz \"Wysoki skok\".", -- Basic_Training_-_Movement
    ["Hint: To place a girder, select it,|then use [Left] and [Right] to select angle and length,|then choose a location for the girder."] = "Porada: By ustawić belkę wybierz ją,|potem użuj [Lewo] i [Prawo], by wybrać kąt i długość,|a potem wybierz położenie belki.", -- A_Classic_Fairytale:shadow
    ["Hint: Use the quit key to see the team’s continent."] = "Porada: Uzyj klawisza wyjścia, by zobaczyć kontynent drużyny", -- Continental_supplies
    ["Hint: When you shorten the rope, you move faster!|And when you lengthen it, you move slower."] = "Wskazówka: Kiedy skrócisz linę poruszasz się szybciej!|A kiedy ją wydłużysz, poruszasz się wolniej.", -- Basic_Training_-_Rope
    ["--- Hint ---"] = "--- Wskazówka ---", -- Battalion
    ["Hint: you might want to stay out of sight and take all the crates...|"] = "Porada: Możesz chcieć pozostać poza widokiem i zabrać wszystkie skrzynie...|", -- A_Classic_Fairytale:journey
    ["Hint: You might want to stay out of sight and take all the crates ..."] = "Wskazówka: Możesz chcieć pozostać poza widokiem i zabrać wszystkie skrzynie...", -- A_Classic_Fairytale:journey
    ["His arms are so strong!"] = "Jego ręce są tak silne!", -- A_Classic_Fairytale:first_blood
    ["hits"] = "wskazówki", -- Basic_Training_-_Bazooka
    ["Hit the “Switch Hedgehog” key until you have|selected Cappy, the hedgehog with the cap!"] = "Wciśnij klawisz “Zmiana jeża”, aż|wybierzesz Czapeczkę, jeża z czapką!", -- Basic_Training_-_Movement
    ["H"] = "J", -- A_Space_Adventure:cosmos, A_Space_Adventure:death01
    ["Hmm … it's going slower than expected."] = "Hmm... to będzie wolniejsze, niż się spodziewałem.", -- A_Classic_Fairytale:queen
    ["Hmmm...actually...I didn't either."] = "Hmmm... właściwie... ja również nie.", -- A_Classic_Fairytale:enemy
    ["Hmmm..."] = "Hmmm...",
    ["Hmmm, I’ll have to find some way of moving him off this anti-portal surface."] = "Hmmm, będę musiał znaleźć jakiś sposób zepchnięcia go z tej powierzchni antyportalowej.", -- portal
    ["Hmmm...it's a draw. How unfortunate!"] = "Hmmm... to remis. Jak niefortunnie!", -- A_Classic_Fairytale:enemy
    ["Hmmm...perhaps a little more time will help."] = "Hmmm... być może więcej czasu pomoże.", -- A_Classic_Fairytale:first_blood
    ["Hm ... Now I ran out of fuel."] = "Hm... teraz skończyło mi się paliwo.", -- A_Space_Adventure:cosmos
    ["Hog 100"] = "Jeż 100", -- A_Space_Adventure:fruit03
    ["Hog 1"] = "Jeż 1", -- A_Space_Adventure:fruit03
    ["Hog 3x5"] = "Jeż 3x5", -- A_Space_Adventure:fruit03
    ["Hog 7+7"] = "Jeż 7+7", -- A_Space_Adventure:fruit03
    ["Hog decar"] = "Jeż decar", -- A_Space_Adventure:fruit03
    ["Hog dertien"] = "Jeż dertien", -- A_Space_Adventure:fruit03
    ["Hog D"] = "Jeż D", -- A_Space_Adventure:fruit03
    ["Hog %d"] = "Jeż %d", -- SimpleMission
    ["Hog EOF"] = "Jeż EOF", -- A_Space_Adventure:fruit03
    ["Hogera is definitely the last planet I saved!"] = "Jeżera jest definitywnie ostatnią planetą, którą uratowałem!", -- A_Space_Adventure:final
    ["Hogera is safe!"] = "Jeżera jest bezpieczna!", -- A_Space_Adventure:final
    ["Hog exi"] = "Jeż exi", -- A_Space_Adventure:fruit03
    ["Hog Hephaestus"] = "Jeż Hefajstos", -- A_Space_Adventure:fruit03
    ["Hog Identity Mode"] = "Tryb Tożsamości Jeża", -- HedgeEditor
    ["HOG IDENTITY MODE"] = "TRYB TOŻSAMOŚCI JEŻA", -- HedgeEditor
    ["Hog III"] = "Jeż III", -- A_Space_Adventure:fruit03
    ["Hogminator"] = "Jeżminator", -- A_Classic_Fairytale:family
    ["Hog nueve"] = "Jeż nueve", -- A_Space_Adventure:fruit03
    ["Hog octo"] = "Jeż okto", -- A_Space_Adventure:fruit03
    ["Hog onze"] = "Jeż onze", -- A_Space_Adventure:fruit03
    ["Hog Saturn"] = "Jeż Saturn", -- A_Space_Adventure:fruit03
    ["Hogs in sight!"] = "Jeż na widoku!", -- Continental_supplies
    ["Hog Solo and GB"] = "Hog Solo i ZB", -- A_Space_Adventure:fruit02
    ["Hog Solo"] = "Hog Solo", -- A_Space_Adventure:cosmos, A_Space_Adventure:death01, A_Space_Adventure:death02, A_Space_Adventure:desert01, A_Space_Adventure:desert02, A_Space_Adventure:desert03, A_Space_Adventure:final, A_Space_Adventure:fruit01, A_Space_Adventure:fruit02, A_Space_Adventure:fruit03, A_Space_Adventure:ice01, A_Space_Adventure:ice02, A_Space_Adventure:moon01, A_Space_Adventure:moon02
    ["- Hogs will be revived"] = "- Jeże zostaną ożywione", -- Capture_the_Flag
    ["- Hogs will drop the flag when killed"] = "Jeże upuszczą flagę po zginięciu", -- Capture_the_Flag
    ["Hog two"] = "Jeż dwa", -- A_Space_Adventure:fruit03
    ["Hold [Attack] to attach the rope."] = "Wciśnij [Atak], by zaczepić linę", -- Basic_Training_-_Rope
    ["Hold the Attack key pressed for more power."] = "Przytrzymaj klawisz ataku po więcej mocy.", -- Basic_Training_-_Bazooka, Basic_Training_-_Grenade
    ["Holy shit!"] = "Jasna cholera!", -- Mutant
    ["Homing Bee"] = "Pszczoła", -- Construction_Mode
    ["Honda"] = "Honda",
    ["Honest Lee"] = "Szczery Li", -- A_Classic_Fairytale:enemy
    ["Hooks"] = "Haki",
    ["Hooray! I actually did it! Hogera is safe!"] = "Hurra! Jednak mi się udało! Jeżera jest bezpieczna!", -- A_Space_Adventure:final
    ["Hooray! I've found it, now I have to get back to Captain Lime!"] = "Hurra! Znalazłem to. Teraz muszę wrócić do Kapitana Limonki.", -- A_Space_Adventure:fruit02
    ["Hooray! You are a champion!"] = "Hurra! Jesteś mistrzem!", -- A_Space_Adventure:ice02
    ["Hopeless case"] = "Beznadziejny przypadek", -- User_Mission_-_RCPlane_Challenge
    ["Hop on top of the next flower and advance to the left coast."] = "Wskocz na następny kwiatek i posuwaj się w stronę lewego wybrzeża.", -- A_Classic_Fairytale:journey
    ["Horns"] = "Rogi",
    ["Hostage Situation"] = "Zakładnicza sytuacja", -- A_Classic_Fairytale:family
    ["How can I ever repay you for saving my life?"] = "Jak mogę ci się odpłacić za uratowanie mi życia?", -- A_Classic_Fairytale:journey
    ["How come in a village full of warriors, it's up to me to save it?"] = "Jak to jest, że w wiosce pełnej wojowników, to do mnie należy uratowanie jej?", -- A_Classic_Fairytale:dragon
    ["How could you betray us?"] = "Jak mogłeś nas zdradzić?", -- A_Classic_Fairytale:queen
    ["How difficult would you like it to be?"] = "Jak trudne chciałbyś, żeby to było?", -- A_Classic_Fairytale:first_blood
    ["HOW DO THEY KNOW WHERE WE ARE?"] = "SKĄD ONI WIEDZĄ, GDZIE JESTEŚMY?", -- A_Classic_Fairytale:united
    ["However, if you fail to do so, she dies a most violent death, just like your friend! Muahahaha!"] = "Jednakże, jeśli nie uda ci się tego zrobić, ona umrze najbardziej brutalną śmiercią, tak jak twój przyjaciel! Muahahaha!", -- A_Classic_Fairytale:journey
    ["However, if you fail to do so, she dies a most violent death! Muahahaha!"] = "Jednakże, jeśli nie uda ci się tego zrobić, ona umrze najbardziej brutalną śmiercią! Muahahaha!", -- A_Classic_Fairytale:journey
    ["However, my mates don't agree with me on letting you go..."] = "Jednakże, moi kumple nie zgodzą się ze mną, by cię wypuścić...", -- A_Classic_Fairytale:dragon
    ["However, the army of %s is about to attack any moment now."] = "Jednakże, armia %s może zaatakować w każdej chwili.", -- A_Space_Adventure:fruit01
    ["How to Rope"] = "Jak używać liny", -- Basic_Training_-_Rope
    ["How would you like being discriminated against?"] = "Jak podoba ci się bycie dyskryminowanym?", -- A_Classic_Fairytale:queen
    ["Huh?"] = "Huh?", -- A_Classic_Fairytale:backstab, A_Classic_Fairytale:queen
    ["Hunter"] = "Strzelec",
    ["I ain't gonna sit around no more!"] = "Nie mam zamiaru dłużej siedzieć!", -- A_Classic_Fairytale:queen
    ["I already said I'm sorry!"] = "Już powiedziałem, że przepraszam!", -- A_Classic_Fairytale:epil
    ["I always suspected him!"] = "Zawsze go podejrzewałem!", -- A_Classic_Fairytale:epil
    ["I am going to leave the kids play by themselves."] = "Zamierzam zostawić dzieci, by bawiły się same.", -- A_Classic_Fairytale:queen
    ["I am not ready for this planet yet. I should visit it when I have found all the other device parts."] = "Nie jestem jeszcze gotowy na tę planetę. Powinienem odwiedzić ją, kiedy znajdę wszystkie inne części urządzenia.", -- A_Space_Adventure:cosmos
    ["I am sorry but I was looking for a device that may be hidden somewhere around here."] = "Przepraszam, ale szukałem urządzenia, które może być schowane gdzieś tutaj.", -- A_Space_Adventure:fruit02
    ["I believe there's more of them."] = "Nie mogę uwierzyć, że jest ich więcej.", -- A_Classic_Fairytale:backstab
    ["I cannot let you go any further! … burp …"] = "Nie mogę pozwolić ci podejść ani trochę bliżej! ...bek...", -- A_Classic_Fairytale:queen
    ["I can see you have been training diligently."] = "Widzę, że pilnie trenowałeś.", -- A_Classic_Fairytale:first_blood
    ["I can't believe how blind we were."] = "Nie mogę uwierzyć jacy byliśmy ślepi.", -- A_Classic_Fairytale:epil
    ["I can't believe it worked!"] = "Nie mogę uwierzyć, że to zadziałało!", -- A_Classic_Fairytale:shadow
    ["I can't believe this!"] = "Nie mogę w to uwierzyć!", -- A_Classic_Fairytale:enemy
    ["I can't believe what I'm hearing!"] = "Nie mogę uwierzyć w to co słyszę!", -- A_Classic_Fairytale:backstab
    ["I can't let you go further because …"] = "Nie mogę pozwolić ci pójść dalej, bo...", -- A_Classic_Fairytale:queen
    ["I can't wait any more, I have to save myself!"] = "Nie mogę już dłużej czekać, muszę się ratować!", -- A_Classic_Fairytale:shadow
    ["Ice Jake"] = "Lodowy Dżejk", -- A_Space_Adventure:ice01
    ["I could just teleport myself there..."] = "Mógłbym się tam po prostu teleportować.", -- A_Classic_Fairytale:family
    ["Icy Girder: [3]"] = "Lodowa Belka: [3]", -- HedgeEditor
    ["Icy Land: [3]"] = "Lodowy Teren: [3]", -- HedgeEditor
    ["Icy Land"] = "Lodowy Teren", -- HedgeEditor
    ["I'd better get going myself."] = "Lepiej sam już pójdę.", -- A_Classic_Fairytale:journey
    ["Identity Thief"] = "Złodziej tożsamości", -- Mutant
    ["I didn't until about a month ago."] = "Nie robiłem, aż do jakoś zeszłego miesiąca.", -- A_Classic_Fairytale:enemy
    ["I don't care. It's worth a fortune! Good bye, you idiot!"] = "Nie obchodzi mnie to. To jest warte fortunę! Do widzenia, idioto!", -- A_Space_Adventure:fruit02
    ["I don't know how you did that. But good work!|The next one should be easy as cake for you!"] = "Nie wiem jak to zrobiłeś. Ale dobra robota!|Następny powinien być dla ciebie łatwy!", -- Basic_Training_-_Rope
    ["I don't know if I can forget what you've done!"] = "Nie wiem, czy mogę zapomnieć co zrobiłeś!", -- A_Classic_Fairytale:epil
    ["I don't know who I can trust anymore."] = "Nie wiem już komu mogę ufać.", -- A_Classic_Fairytale:epil
    ["I don't like your tone! You're hurting me!"] = "Nie lubię twojego tonu! On mnie rani!", -- A_Classic_Fairytale:queen
    ["I feel something...a place! They will arrive near the circles!"] = "Coś wyczuwam... miejsce! Oni przybędą niedaleko kół!", -- A_Classic_Fairytale:backstab
    ["If only I had a way..."] = "Gdybym tylko miał wyjście...", -- A_Classic_Fairytale:backstab
    ["If only I were given a chance to explain my being here..."] = "Gdybym tylko dostał szansę wytłumaczyć swoją obecność tutaj...", -- A_Classic_Fairytale:first_blood
    ["If only one enemy is left, you'll get bonus ammo."] = "Kiedy zostanie tylko jeden wróg, dostaniesz bonusową amunicję.", -- A_Space_Adventure:death02
    ["I forgot that she's the daughter of the chief, too..."] = "Zapomniałem, że ona jest też córką szefa...", -- A_Classic_Fairytale:backstab
    ["I found it! Hooray!"] = "Znalazłem to! Hurra!", -- A_Space_Adventure:desert01
    ["If some good old explosives were enough to save Hogera …"] = "Jeśli trochę starych dobrych ładunków wybuchowych by wystarczyło, aby uratować Jeżerę...", -- A_Space_Adventure:final
    ["If they try coming here, they can have a taste of my delicious knuckles!"] = "Jeśli spróbują tu przyjść, posmakują moich kastetów!", -- A_Classic_Fairytale:united
    ["If you agree to provide the information we need, you will be spared!"] = "Jeśli zgodzisz się zdradzić informacje, których potrzebujemy, zostaniesz oszczędzony!", -- A_Classic_Fairytale:shadow
    ["If you can get that crate fast enough, your beloved \"princess\" may go free."] = "Jeśli uda ci się zdobyć tę skrzynię wystarczająco szybko, twoja ukochana \"księżniczka\" zostanie uwolniona.", -- A_Classic_Fairytale:journey
    ["If you decide to help us, though, we will no longer need to find a new governor for the island."] = "Jednak jeśli zdecydujesz się nam pomóc, nie będziemy już musieli znaleźć nowego gubernatora dla wyspy.", -- A_Classic_Fairytale:shadow
    ["If you don't want to slip away, you have to keep moving!"] = "Jeśli nie chcesz się ześlizgnąć, musisz wciąż się poruszać!", -- Basic_Training_-_Movement
    ["If you get stuck, use your Desert Eagle or restart the mission!"] = "Jeśli utkniesz, użyj Desert Eagle lub zrestartuj misję!", -- A_Classic_Fairytale:journey
    ["If you get stuck, use your Desert Eagle or restart the mission!|"] = "Jeśli utkniesz, użyj swojego Desert Eagle lub zrestartuj misję!|", -- A_Classic_Fairytale:journey
    ["If you help us you can keep the device if you find it but we'll keep everything else."] = "Jeśli nam pomożesz, możesz zatrzymać urządzenie jeśli je znajdziesz, ale zatrzymay wszytko inne.", -- A_Space_Adventure:fruit02
    ["If you hurt an enemy, you'll get one third of the damage dealt."] = "Jeśli skrzywdzisz wroga, otrzymasz jedną trzecią zadanych obrażeń.", -- A_Space_Adventure:death02
    ["If you injure a hedgehog you'll get 35% of the damage dealt."] = "Jeśli zranisz jeża, otrzymasz 35% zadanych obrażeń.", -- A_Space_Adventure:death02
    ["If you just don’t care …"] = "Jeśli ci nie zależy...", -- Continental_supplies
    ["If you kill a hedgehog with the respective weapon your health points will be set to 100."] = "Jeśli zabijesz jeża odpowiednią bronią, twoje zdrowie zostanie ustawione na 100.", -- A_Space_Adventure:death02
    ["If you kill an enemy, your health will be set to 100."] = "Jeśli zabijesz wroga, twoje zdrowie zostanie ustawione na 100.", -- A_Space_Adventure:death02
    ["If you know what I mean..."] = "Jeśli wiesz co mam na myśli...", -- A_Classic_Fairytale:shadow
    ["If you miss a shot while trying to|re-attach, your rope is gone, too!"] = "Jeśli spudłujesz strzał próbując złapać się|ponownie, twoja lina również przepadnie!", -- Basic_Training_-_Rope
    ["If you say so..."] = "Jeśli tak mówisz...", -- A_Classic_Fairytale:shadow
    ["If you skip a turn then the turn time left will be added to your next turn."] = "Jeśli pominiesz turę, pozostały czas zostanie dodany do twojej następnej tury.", -- A_Space_Adventure:fruit03
    ["If you wish to replay, there are other possible endings, too!"] = "Jeśli zechcesz zagrać ponownie, są również inne możliwe zakończenia!", -- A_Classic_Fairytale:epil
    ["Igmund"] = "Igmund", -- User_Mission_-_Nobody_Laugh
    ["I grew sick of the oppression! I broke free!"] = "Miałam dość opresji! Uwolniłam się!", -- A_Classic_Fairytale:queen
    ["I guess I can't go far without fuel!"] = "Zdaje się, że nie mogę zajść daleko bez paliwa!", -- A_Space_Adventure:cosmos
    ["I guess we lost him!"] = "Zdaje się, że go zgubiliśmy!", -- A_Space_Adventure:cosmos
    ["I guess you'll have to kill them."] = "Zdaje się, że będziesz musiał ich zabić.", -- A_Classic_Fairytale:dragon
    ["I have come to make you an offering..."] = "Przyszedłem, by ci coś zaoferować...", -- A_Classic_Fairytale:shadow
    ["I have heard that the local tribes say that many years ago some PAotH scientists were dumping their waste here."] = "Słyszałem, jak lokalne plemienia mówiły, że wiele lat temu jacyś naukowcy PSJ wyrzucali tu śmieci.", -- A_Space_Adventure:desert01
    ["I have more important things to do!"] = "Mam ważniejsze rzeczy do zrobienia!", -- A_Classic_Fairytale:queen
    ["I have no idea where that mole disappeared...Can you see it?"] = "Nie mam pojęcia, gdzie ten kret zniknął. Ty go widzisz?", -- A_Classic_Fairytale:shadow
    ["I have only 3 hogs available and they are all cadets."] = "Mam dostępne tylko 3 jeże i wszyscy są kadetami.", -- A_Space_Adventure:fruit01
    ["I have to follow that alien."] = "Muszę śledzić tego obcego.", -- A_Classic_Fairytale:backstab
    ["I have to get back to the village!"] = "Muszę dostać się z powrotem do wioski!", -- A_Classic_Fairytale:shadow
    ["I have to reach the surface as quickly as I can."] = "Muszę dotrzeć na powierzchnię tak szybko, jak mogę.", -- A_Space_Adventure:desert02
    ["I hope you are prepared for a small challenge, young one."] = "Mam nadzieję, że jesteś gotowy na małe wyzwanie, młodzieńcze.", -- A_Classic_Fairytale:first_blood
    ["I just don't want to sink to your level."] = "Nie chcę zniżyć się do twojego poziomu.", -- A_Classic_Fairytale:backstab
    ["I just forgot all checkpoints of incomplete missions."] = "Właśnie zapomniałem wszystkie punkty kontrolne niedokończonych misji.", -- A_Space_Adventure:cosmos
    ["I just found out that they have captured your princess!"] = "Właśnie się dowiedziałem, że porwali waszą księżniczkę!", -- A_Classic_Fairytale:family
    ["I just want the strange device you found!"] = "Chcę po prostu to dziwne urządzenie, które znalazłeś!", -- A_Space_Adventure:ice01
    ["I just wonder where Ramon and Spiky disappeared..."] = "Zastanawiam się, gdzie podziali się Ramon i Kolczasty.", -- A_Classic_Fairytale:journey
    ["I know and I'm terribly sorry!"] = "Wiem, i strasznie mi przykro!", -- A_Classic_Fairytale:epil
    ["I know, my hero!"] = "Wiem, mój bohaterze!", -- A_Classic_Fairytale:epil
    ["I know that your resources are low due to the battle but I'll send two of my best hogs to assist you."] = "Wiem, że twoje zasoby są na wyczerpaniu z powodu bitwy, ale wyślę dwóch z moich najlepszych jeży, by cię wsparli.", -- A_Space_Adventure:fruit02
    ["I … like being with you, too."] = "Ja... również lubię z tobą być.", -- A_Classic_Fairytale:epil
    ["I'll get him!"] = "Dopadnę go!", -- A_Space_Adventure:cosmos
    ["I'll hold them off while you return to the village!"] = "Zatrzymam ich, a ty wróć do wioski!", -- A_Classic_Fairytale:shadow
    ["I'll let you know whatever I know about him if you manage to catch me 3 times."] = "Przekażę ci wszystko, co o nim wiem, jeśli uda ci się złapać mnie 3 razy.", -- A_Space_Adventure:moon02
    ["I'll make good use of it."] = "Zdrobię z tego pożytek.", -- A_Space_Adventure:cosmos
    ["I'll protect you!"] = "Będę cię chronił!", -- A_Classic_Fairytale:epil
    ["I love Dense Cloud now!"] = "Teraz kocham Gęstą Chmurę!", -- A_Classic_Fairytale:epil
    ["I love you."] = "Kocham cię.", -- A_Classic_Fairytale:epil
    ["I'm afraid I can't let you proceed!"] = "Obawiam się, że nie mogę pozwolić ci przejść.", -- A_Classic_Fairytale:queen
    ["I'm afraid we cannot afford that."] = "Obawiam się, że nie możemy sobie na to pozwolić.", -- A_Classic_Fairytale:queen
    ["Imagine those targets are the wolves that killed your parents! Take your anger out on them!"] = "Wyobraź sobie, że te cele to wilki, które zabiły twoich rodziców! Wyzwól na nich swój gniew!", -- A_Classic_Fairytale:first_blood
    ["I'm...alive? How? Why?"] = "Ja... żyję? Jak? Dlaczego?", -- A_Classic_Fairytale:backstab
    ["I'm a ninja."] = "Jestem ninja.", -- A_Classic_Fairytale:dragon
    ["I marked the place of their arrival. You're welcome!"] = "Zaznaczyłem miejsce ich przybycia. Nie ma za co!", -- A_Classic_Fairytale:backstab
    ["I may lost this battle, but I haven't lost the war yet!"] = "Może i przegrałem tę bitwę, ale jeszcze nie przegrałem wojny!", -- A_Space_Adventure:moon01
    ["I'm certain that this is a misunderstanding, fellow hedgehogs!"] = "Jestem pewien, że to nieporozumienie, jeżowi koledzy!", -- A_Classic_Fairytale:first_blood
    ["I mean, none of you ceased to live."] = "Znaczy, nikt z was nie przestał żyć.", -- A_Classic_Fairytale:enemy
    ["I'm getting old for this!"] = "Robię się na to za stary.", -- A_Classic_Fairytale:family
    ["I'm getting thirsty..."] = "Chce mi się pić...", -- A_Classic_Fairytale:family
    ["I'm glad this is over!"] = "Cieszę się, że to koniec!", -- A_Classic_Fairytale:epil
    ["I'm here to help you rescue her."] = "Jestem tu pomóc ci ją uratować.", -- A_Classic_Fairytale:family
    ["I'm living a dream!"] = "Przeżywam sen!", -- A_Classic_Fairytale:queen
    ["I'm not sure about that!"] = "Nie jestem tego pewny!", -- A_Classic_Fairytale:united
    ["IMPORTANT: To see the mission panel again, hold the mission panel key."] = "WAŻNE: By ponownie zobaczyć panel misji, przytrzymaj klawisz panelu misji.", -- Basic_Training_-_Movement
    ["IMPORTANT: To see the mission panel again, pause the game."] = "WAŻNE: By zobaczyć ponownie panel misji, zapauzuj grę.", -- Basic_Training_-_Movement
    ["IMPORTANT: To see the mission panel again, use the quit or pause key."] = "WAŻNE: By zobaczyć panel misji ponownie, użyj przycisku wyjścia lub pauzy.", -- Basic_Training_-_Movement
    ["Impressive...you are still dry as the corpse of a hawk after a week in the desert..."] = "Imponujące... nadal jesteś suchy, jak zwłoki jastrzębia po tygodniu na pustyni...", -- A_Classic_Fairytale:first_blood
    ["%i ms"] = "%i ms", -- Gravity
    ["I'm so glad this is finally over!"] = "Tak się cieszę, że to wreszcie koniec!", -- A_Space_Adventure:final
    ["I'm so scared!"] = "Tak się boję!", -- A_Classic_Fairytale:united
    ["I'm still low on hogs. If you are not afraid I could use a set of extra hands."] = "Nadal mam mało jeży. Jeśli się nie boisz, przydałyby się dodatkowe dłonie do pomocy.", -- A_Space_Adventure:fruit02
    ["I'm still with the aliens."] = "Nadal jestem z obcymi.", -- A_Classic_Fairytale:queen
    ["I'm terribly sorry!"] = "Tak mocno przepraszam!", -- A_Classic_Fairytale:queen
    ["I'm the spy! I've been giving you out!"] = "Jestem szpiegiem! Wydawałam was!", -- A_Classic_Fairytale:queen
    ["In am also entrusting you with some rope."] = "Powierzam ci również trochę liny.", -- A_Space_Adventure:cosmos
    ["In case you haven't noticed, I'm a woman, too!"] = "W razie, gdybyś nie zauważył, też jestem kobietą!", -- A_Classic_Fairytale:queen
    ["Increase the dust storm damage by sacrificing|your invulnerable ammo."] = "Zwiększ obrażenia burzy pyłowej poświęcając|swoją niezniszczalną amunicję.", -- Continental_supplies
    ["Incredible..."] = "Niesamowite...", -- A_Classic_Fairytale:shadow
    ["Indestructible Girder: [2]"] = "Niezniszczalna Belka: [2]", -- HedgeEditor
    ["Indestructible Land: [2]"] = "Niezniszczalny Teren: [2]", -- HedgeEditor
    ["Indestructible Land"] = "Niezniszczalny Teren", -- HedgeEditor
    ["In each round, the worst hedgehog of the round is eliminated."] = "W każdej rundzie, najgorszy jeż rundy jest eliminowany.", -- TrophyRace
    ["I need to find the others!"] = "Muszę odnaleźć innych!", -- A_Classic_Fairytale:backstab
    ["I need to get to the other side of this island, fast!"] = "Muszę dostać się na drugą stronę tej wyspy, szybko!", -- A_Classic_Fairytale:journey
    ["I need to move the tribe!"] = "Muszę przemieścić plemię!", -- A_Classic_Fairytale:united
    ["I need to prevent their arrival!"] = "Muszę zapobiec ich przybyciu!", -- A_Classic_Fairytale:backstab
    ["I need to warn the others."] = "Muszę ostrzec innych.", -- A_Classic_Fairytale:backstab
    ["In fact, you are the only one that's been acting strangely."] = "Tak naprawdę, jesteś jedynym, który dziwnie się zachowywał.", -- A_Classic_Fairytale:backstab
    ["Initial health: %d"] = "Początkowe zdrowie: %d", -- Continental_supplies
    ["Initiate escape wish!"] = "Inicjalizuj życzenie ucieczki!", -- A_Classic_Fairytale:queen
    ["In order to get to the other side, you need to get rid of the crates first."] = "Żeby dostać się na drugą stronę, musisz pozbyć się najpierw skrzyń.", -- A_Classic_Fairytale:dragon
    ["Insanity!"] = "Szaleństwo!", -- Mutant
    ["Inside %d"] = "Wewnątrz %d", -- WxW
    ["Inside"] = "Wewnątrz", -- WxW
    ["Instructions"] = "Instrukcje", -- Basic_Training_-_Flying_Saucer
    ["Instructor"] = "Instruktor",
    ["Insufficient Power"] = "Niewystarczająca Moc", -- Construction_Mode
    ["Interesting idea, haha!"] = "Ciekawy pomysł, haha!", -- A_Classic_Fairytale:enemy
    ["Interesting! Last time you said you killed a cannibal!"] = "Ciekawe! Ostanim razem mówiłeś, że zabiłeś kanibala!", -- A_Classic_Fairytale:backstab
    ["In the Ice Planet Flying Saucer Stadium ..."] = "Na Stadionie Latających Talerzy Lodowej Planety...", -- A_Space_Adventure:ice02
    ["In the meantime, take these and return to your \"friend\"!"] = "W międzyczasie, weź je i wróć do swojego \"przyjaciela\"!", -- A_Classic_Fairytale:shadow
    ["In the stadium, where the best pilots compete ..."] = "Na stadionie, gdzie rywalizują najlepsi piloci...", -- A_Space_Adventure:ice02
    ["In this accident, Professor Hogevil lost all his spines on his head!"] = "W wypadku tym, Profesor Jeżozło stracił wszystkie kolce na swojej głowie!", -- A_Space_Adventure:moon02
    ["In this mission you get %d%% fuel."] = "W tej misji dostajesz %d%% paliwa.", -- User_Mission_-_Diver
    ["In this mission you have infinite time."] = "W tej misji masz nieskończony czas.", -- portal
    ["Invalid Placement"] = "Nieprawidłowe Ustawienie", -- Construction_Mode
    ["Invasion"] = "Inwazja", -- A_Classic_Fairytale:united
    ["In your best (and only) flight you took out %d crates with one RC plane!"] = "W swoim najlepszym (i jedynym) locie, usunąłeś %d skrzyń jednym samolotem!", -- User_Mission_-_RCPlane_Challenge
    ["In your best flight you took out %d crates with one RC plane."] = "W swoim najlepszym locie, usunąłeś %d skrzyń jednym samolotem.", -- User_Mission_-_RCPlane_Challenge
    ["I regret to end your little odyssey."] = "Z przykrością kończę twoją małą wędrówkę.", -- A_Classic_Fairytale:queen
    ["I saw it with my own eyes!"] = "Widziałem to na własne oczy!", -- A_Classic_Fairytale:shadow
    ["I see..."] = "Widzę...", -- A_Classic_Fairytale:shadow
    ["I see you already took care of your enemies."] = "Widzę, że już zająłeś się swoimi wrogami.", -- A_Classic_Fairytale:dragon
    ["I see you have already taken the leap of faith."] = "Widzę, że zrobiłeś już skok wiary.", -- A_Classic_Fairytale:first_blood
    ["I see you would like his punishment to be more...personal..."] = "Widzę, że chciałbyś, żeby jego kara była bardziej... osobista...", -- A_Classic_Fairytale:first_blood
    ["I sense another wave of cannibals heading my way!"] = "Wyczuwam kolejną falę kanibali idących w moją stronę!", -- A_Classic_Fairytale:backstab
    ["I sense another wave of cannibals heading our way!"] = "Wyczuwam kolejną falę kanibali idących w naszą stronę!", -- A_Classic_Fairytale:backstab
    ["I should get myself a portal device, maybe this crate has one."] = "Powinienem sobie zdobyć urządzenie portalowe. Może ta skrzynia ma takie.", -- portal
    ["I should go now, goodbye!"] = "Muszę już iść, do widzenia!", -- A_Space_Adventure:moon02
    ["I shouldn't have drunk that last pint."] = "Nie powinienem był wypić tego ostatniego piwa.", -- A_Classic_Fairytale:dragon
    ["%i s"] = "%i s", -- Gravity
    ["Is this place in my head?"] = "Czy to miejsce jest w mojej głowie?", -- A_Classic_Fairytale:dragon
    ["I still can't believe he sold us out like that."] = "Nie mogę uwierzyć, że tak po prostu nas wydał.", -- A_Classic_Fairytale:epil
    ["I still can't believe you forgave her!"] = "Nie mogę uwierzyć, że jej wybaczyłeś!", -- A_Classic_Fairytale:epil
    ["I still have to get rid of the crates."] = "Nadal muszę pozbyć się skrzyń.", -- A_Classic_Fairytale:dragon
    ["Itami"] = "Itami",
    ["It doesn't matter. I won't let that alien hurt my daughter!"] = "To nie ma znaczenia. Nie pozwolę temu obcemu skrzywdzić mojej córki!", -- A_Classic_Fairytale:dragon
    ["I think I love you!"] = "Myślę, że cię kocham!", -- A_Classic_Fairytale:epil
    ["I think we are safe here."] = "Myślę, że jesteśmy tu bezpieczni.", -- A_Classic_Fairytale:backstab
    ["I thought their shaman died when he tried our medicine!"] = "Myślałem, że ich szaman umarł, gdy spróbował naszego lekarstwa!", -- A_Classic_Fairytale:shadow
    ["It is called 'Hogs of Steel'."] = "Nazywa się 'Jeż ze Stali'.", -- A_Classic_Fairytale:enemy
    ["It is time to practice your fighting skills."] = "Już czas, byś poćwiczył swoje zdolności bojowe.", -- A_Classic_Fairytale:first_blood
    ["It must be a childhood trauma..."] = "To musi być trauma z dzieciństwa...", -- A_Classic_Fairytale:family
    ["It must be the aliens' deed."] = "To pewnie robota obcych.", -- A_Classic_Fairytale:backstab
    ["It must be the aliens!"] = "To pewnie obcy!", -- A_Classic_Fairytale:backstab
    ["It must be the cyborgs again!"] = "To pewnie znowu te cyborgi!", -- A_Classic_Fairytale:enemy
    ["It needs some practice, but you have infinite lives."] = "To wymaga trochę praktyki, ale będziesz mieć nieskończone życia.", -- Basic_Training_-_Rope
    ["I told you, I just found them."] = "Mówię ci, właśnie je znalazłem.", -- A_Classic_Fairytale:backstab
    ["It only works in teleportation nodes of your own clan."] = "To działa tylko w węzłach teleportacyjnych twojego klanu.", -- Construction_Mode
    ["It's a good thing SUDDEN DEATH is 99 turns away..."] = "Jak to dobrze, że NAGŁA ŚMIERĆ jest dopiero za 99 tur...",
    ["It's all about the right carrots, you know."] = "Wszystko zależy od właściwej marchewki, no wiesz.", -- A_Classic_Fairytale:epil
    ["It's always up to women to clear up the mess men created!"] = "Zawsze należało do kobiet, by posprzątać bałagan pozostawiony przez mężczyzn!", -- A_Classic_Fairytale:dragon
    ["It's amazing how quickly our lives can change."] = "To niesamowite, jak szybko nasze życia potrafią się zmienić.", -- A_Classic_Fairytale:epil
    ["It's an ancient ritual of theirs."] = "To ich starożytny rytuał.", -- A_Classic_Fairytale:queen
    ["IT'S A SERIOUS MEDICAL CONDITION!"] = "TO POWAŻNE SCHORZENIE!", -- A_Classic_Fairytale:queen
    ["It's a shame, I forgot how to do that!"] = "Ale wstyd, zapomniałem jak to się robi!", -- A_Classic_Fairytale:family
    ["It's a shame, really!"] = "To wstyd, naprawdę!", -- A_Classic_Fairytale:queen
    ["It seems that Professor Hogevil has prepared for your arrival!"] = "Zdaje się, że Profesor Jeżozło przygotował się na twoje przybycie!", -- A_Space_Adventure:moon01
    ["It's empty!"] = "Jest pusta!", -- Battalion
    ["It's impossible to communicate with the spirits without a shaman."] = "Niemożliwa jest komunikacja z duszami, bez szamana.", -- A_Classic_Fairytale:shadow
    ["It's not that easy, so listen carefully:"] = "To nie takie łatwe, więc słuchaj uważnie:", -- Basic_Training_-_Flying_Saucer
    ["It's over..."] = "To koniec!", -- A_Classic_Fairytale:shadow
    ["It's precious to me!"] = "Jest dla mnie cenny!", -- A_Classic_Fairytale:queen
    ["It's time you learned that your actions have consequences!"] = "To czas, byś się nauczył, że twoje akcje mają konsekwencje!", -- A_Classic_Fairytale:journey
    ["It's worth more than wood!"] = "Jest warty więcej niż drewno!", -- A_Classic_Fairytale:enemy
    ["It's your fault you're there!"] = "Jesteś tu ze swojej winy!", -- A_Classic_Fairytale:epil
    ["It wants our brains!"] = "To chce naszych mózgów.", -- A_Classic_Fairytale:shadow
    ["It was all a trick?!"] = "To wszystko było sztuczką?", -- A_Classic_Fairytale:queen
    ["It was all just bad luck!"] = "To wszystko był tylko pech!", -- ClimbHome
    ["It was completely useless!"] = "To było kompletnie bezużyteczne!", -- A_Space_Adventure:final
    ["It was fun to watch, though."] = "Zabawnie się to jednak oglądało.", -- A_Classic_Fairytale:queen
    ["It was fun to watch."] = "Zabawnie się to oglądało.", -- A_Classic_Fairytale:queen
    ["It was not a dream, unwise one!"] = "To nie był sen, nierozsądny!", -- A_Classic_Fairytale:backstab
    ["It wasn't her fault!"] = "To nie była jej wina!", -- A_Classic_Fairytale:epil
    ["It would be wiser to steal the space ship while the PAotH guards are taking a brake!"] = "Byłoby mądrzej ukraść statek kosmiczny, kiedy strażnicy PSJ mają przerwę!", -- A_Space_Adventure:cosmos
    ["Ivan"] = "Iwan",
    ["I've made it! Yeah!"] = "Udało mi się! Tak!", -- A_Space_Adventure:moon01
    ["I've seen this before. They just appear out of thin air."] = "Widziałem to już wcześniej. Oni pojawiają się znikąd.", -- A_Classic_Fairytale:united
    ["I've thought that the best way to get the device is to let you collect most of the parts for me!"] = "Myślałem, że najlepszym sposobem, by zdobyć urządzenie, jest pozwolić ci zebrać większość części dla mnie!", -- A_Space_Adventure:death01
    ["I want to play a game..."] = "Zagrajmy w grę...", -- A_Classic_Fairytale:journey
    ["I want to see how it handles this!"] = "Chcę zobaczyć jak sobie z tym poradzi!", -- A_Classic_Fairytale:backstab
    ["I was heading home, you see!"] = "Szedłem w stronę domu, widzisz!", -- A_Classic_Fairytale:queen
    ["I was so scared."] = "Tak się bałam.", -- A_Classic_Fairytale:epil
    ["I was told that as the leader of the king's guard, no one knows this world better than you!"] = "Powiedziano mi, że jako dowódca królewskiej straży, nikt nie zna tego świata lepiej od ciebie!", -- A_Space_Adventure:fruit01
    ["I will never hand you the parts!"] = "Nigdy nie oddam ci części!", -- A_Space_Adventure:death01
    ["I wish to help you, %s!"] = "Chcę ci pomóc, %s!", -- A_Classic_Fairytale:dragon
    ["I wonder where Dense Cloud is..."] = "Zastanawiam się, gdzie jest Gęsta Chmura...", -- A_Classic_Fairytale:journey, A_Classic_Fairytale:shadow
    ["I wonder why I'm so angry all the time..."] = "Zastanawiam się, dlaczego jestem taki zły cały czas...", -- A_Classic_Fairytale:family
    ["I won't let you kill her!"] = "Nie pozwolę ci jej zabić!", -- A_Classic_Fairytale:journey
    ["I won't let you kill the tribe!"] = "Nie pozwolę ci zabić plemienia!", -- A_Classic_Fairytale:queen
    ["I would gladly help you if we won this battle but under these circumstances I'll only help you if you fight for our side."] = "Z chęcią bym ci pomógł, jeśli wygralibyśmy tę bitwę, ale w tych okolicznościach, pomogę ci tylko jeśli zawalczysz po naszej stronie.", -- A_Space_Adventure:fruit01
    ["Jack"] = "Jacek", -- A_Classic_Fairytale:dragon, A_Classic_Fairytale:family, A_Classic_Fairytale:queen
    ["Jason"] = "Jason",
    ["Jeremiah"] = "Jeremiasz", -- A_Classic_Fairytale:dragon
    ["Jetpack"] = "Plecak odrzutowy", -- Big_Armory
    ["Jigglypuff"] = "Jigglypuff",
    ["Jim Morgan"] = "Jim Morgan",
    ["Jimmy"] = "Dżimi",
    ["Jingo"] = "Dżingo",
    ["Joe"] = "Dżoł", -- A_Space_Adventure:moon01
    ["John"] = "Dżon", -- A_Classic_Fairytale:journey
    ["John Snow"] = "John Snow", -- A_Space_Adventure:ice01
    ["Jolly Roger"] = "Jolly Roger",
    ["Jones"] = "Dżons",
    ["Judas"] = "Judasz", -- A_Classic_Fairytale:backstab
    ["Juicy"] = "Soczysty",
    ["Jumping is disabled"] = "Skakanie jest wyłączone",
    ["Jumping"] = "Skakanie", -- Basic_Training_-_Movement
    ["Just kidding, none of you have died!"] = "Tylko żartuję, nikt z was nie umarł.", -- A_Classic_Fairytale:enemy
    ["Just look at Leaks, may he rest in peace!"] = "Spójrz tylko na Przecieka, niech spoczywa w pokoju.", -- A_Classic_Fairytale:queen
    ["Just on a walk."] = "Tylko na spacer.", -- A_Classic_Fairytale:united
    ["Just wait till I get my hands on that trauma! ARGH!"] = "Tylko czekaj, aż dorwę się do tej traumy! ARGH!", -- A_Classic_Fairytale:family
    ["Kaboom! Hahahaha! Take this, stupid meteorite!"] = "Kabum! Hahahaha! A masz, głupi meteorycie!", -- A_Space_Adventure:final
    ["Kaboom!"] = "Ka-bum!", -- Basic_Training_-_Flying_Saucer
    ["Kamikaze Expert! +15 points!"] = "Ekspert kamikaze! +15 punktów!", -- Space_Invasion
    ["Kamikaze"] = "Kamikaze", -- Construction_Mode
    ["Keep it up!"] = "Tak trzymaj!",
    ["Ken"] = "Ken",
    ["Kenshi"] = "Kenshi",
    ["Kerguelen"] = "Kerguelen", -- Continental_supplies
    ["key."] = "klucz.", -- Continental_supplies
    ["Kill all enemy hedgehogs in a single turn."] = "Zabij wszystkie wrogie jeże w jednej turze.", -- Big_Armory
    ["Kill him or skip your turn."] = "Zabij go lub pomiń swoją turę.", -- A_Classic_Fairytale:backstab
    ["Killing spree!"] = "Szaleństwo zabijania!",
    ["Killing the specialists"] = "Zabijanie specjalistów", -- A_Space_Adventure:death02
    ["KILL IT!"] = "ZABIJ TO!", -- A_Classic_Fairytale:first_blood
    ["Kills: %d"] = "Zabójstwa: %d", -- Space_Invasion
    ["Kill the aliens!"] = "Zabij obcych!", -- A_Classic_Fairytale:dragon
    ["Kill the cannibal!"] = "Zabij kanibala!", -- A_Classic_Fairytale:first_blood
    ["Kill The Leader: You must also hit the team with the most health."] = "Zabij Przywódcę: Musisz również uderzyć drużynę z największą ilością zdrowia.", -- WxW
    ["Kill The Leader"] = "Zabij Przywódcę", -- WxW
    ["Kill the traitor, %s, or spare his life!"] = "Zabij zdrajcę, %s, lub oszczędź mu życie!", -- A_Classic_Fairytale:backstab
    ["--- King ---"] = "--- Król ---", -- Battalion
    ["King"] = "Król", -- Battalion
    ["--- King Mode ---"] = "--- Tryb Króla ---", -- Battalion
    ["Knight"] = "Rycerz", -- Battalion
    ["Knives"] = "Noże",
    ["Knockball"] = "Stukany", -- Knockball
    ["Knockball weapon"] = "Broń stukanego", -- Knockball
    ["Knock off the enemies from the left-most place of the map!"] = "Zrzuć wrogów z lewej części mapy!", -- A_Space_Adventure:fruit01
    ["koda"] = "koda",
    ["Kostya"] = "Kostja",
    ["Lady Mango"] = "Pani Mango", -- A_Space_Adventure:fruit01, A_Space_Adventure:fruit02
    ["LandFlag Modification Mode"] = "Tryb Modyfikacji Flag Terenu", -- HedgeEditor
    ["Land mines explode instantly."] = "Miny wybuchają natychmiast.", -- User_Mission_-_Teamwork_2
    ["Lassard"] = "Draniówa",
    ["Last Resort: Having less than 25% base health gives kamikaze"] = "Ostatnia Deska Ratunku: Posiadanie mniej niż 25% bazowego zdrowia daje kamikaze", -- Battalion
    ["Last Target!"] = "Ostatni cel!",
    ["Launch a bouncy ball which explodes into a crate."] = "Wystrzel odbijającą się piłkę, która eksloduje w skrzynię.", -- Continental_supplies
    ["Launch some bazookas to destroy the targets!"] = "Wystrzel trochę bazook i zniszcz cele!", -- Basic_Training_-_Bazooka
    ["Leaderbot"] = "Dowódcobot", -- A_Classic_Fairytale:queen
    ["Leader"] = "Dowódca", -- A_Classic_Fairytale:enemy
    ["Lead your allies to battle and eliminate all the enemies!"] = "Poprowadź swoich sprzymierzeńców do walki i wyeliminuj wszystkich wrogów!", -- A_Space_Adventure:fruit01
    ["Leaks A Lot, depressed for killing his loved one, failed to save the village..."] = "Spory Przeciek, przygnębiony po zabiciu ukochanej, zawiódł w ratowaniu wioski...", -- A_Classic_Fairytale:journey
    ["Leaks A Lot gave his life for his tribe! He should have survived!"] = "Spory Przeciek oddał swoje życie dla jego plemienia! Powinien był przeżyć!", -- A_Classic_Fairytale:first_blood
    ["Leaks A Lot must survive!"] = "Spory Przeciek musi przeżyć!", -- A_Classic_Fairytale:journey
    ["Leaks A Lot"] = "Spory Przeciek", -- A_Classic_Fairytale:backstab, A_Classic_Fairytale:dragon, A_Classic_Fairytale:enemy, A_Classic_Fairytale:epil, A_Classic_Fairytale:family, A_Classic_Fairytale:first_blood, A_Classic_Fairytale:journey, A_Classic_Fairytale:queen, A_Classic_Fairytale:shadow, A_Classic_Fairytale:united
    ["Leap of Faith"] = "Skok Wiary", -- Basic_Training_-_Movement
    ["Led Heart"] = "Prowadzące Serce", -- A_Classic_Fairytale:queen
    ["Lee"] = "Li", -- A_Classic_Fairytale:dragon, A_Classic_Fairytale:family, A_Classic_Fairytale:queen
    ["Left and right"] = "Lewo i prawo", -- WxW
    ["Left, right and roof"] = "Lewo, prawo i sufit", -- WxW
    ["[Left], [Right]: Change between identities."] = "[Lewo], [Prawo]: Zmień między tożsamościami.", -- HedgeEditor
    ["[Left], [Right]: Change health value."] = "[Lewo], [Prawo]: Zmień ilość zdrowia.", -- HedgeEditor
    ["Left/right: Choose crate contents"] = "Left/right: Wybierz zawartość skrzyni", -- Construction_Mode
    ["Left/right: Choose structure type|Cursor: Build structure"] = "Left/right: Wybierz rodzaj struktury|Kursor: Zbuduj strukturę", -- Construction_Mode
    ["Left/right: Choose structure type"] = "Lewo/Prawo: Wybierz typ struktury", -- Construction_Mode
    ["Legs"] = "Nogi",
    ["Less tools, more fun"] = "Mniej narzędzi, więcej zabawy", -- Battalion
    ["Lestat"] = "Lestat", -- portal
    ["Let a continent provide your weapons!"] = "Daj kontynentowi zapewnić ci twoje bronie!", -- Continental_supplies
    ["Let me test your skills a little, will you?"] = "Pozwól mi sprawdzić trochę twoje zdolności, co?", -- A_Classic_Fairytale:journey
    ["Let's get started!"] = "Zaczynajmy!", -- Basic_Training_-_Bazooka
    ["Let's go home!"] = "Wracajmy do domu!", -- A_Classic_Fairytale:journey
    ["Let's go, %s!"] = "Chodźmy, %s!", -- WxW
    ["Let's go!"] = "Zaczynajmy!", -- A_Space_Adventure:moon02
    ["Let's head back to the village!"] = "Udajmy się z powrotem do wioski!", -- A_Classic_Fairytale:shadow
    ["Let's see what your comrade does now!"] = "Zobaczmy, co twój towarzysz teraz robi!", -- A_Classic_Fairytale:journey
    ["Let's show those cannibals what we're made of!"] = "Pokażmy kanibalom z czego jesteśmy zrobieni!", -- A_Classic_Fairytale:backstab
    ["Let them have a taste of my fury!"] = "Dajmy im posmakować mojej furii!", -- A_Classic_Fairytale:backstab
    ["Let us help, too!"] = "Pozwól nam również pomóc!", -- A_Classic_Fairytale:backstab
    ["Level 1 clear!"] = "Poziom 1 ukończony!", -- A_Space_Adventure:desert03
    ["Level 2 clear!"] = "Poziom 2 ukończony!", -- A_Space_Adventure:desert03
    ["Level Data Saved!"] = "Wczytano dane poziomu!", -- HedgeEditor
    ["Lightbender"] = "Władca Światła",
    ["Light Cannfantry"] = "Lekka Kanibaleria", -- A_Classic_Fairytale:united
    ["Limited Ammo"] = "Ograniczona Amunicja", -- Basic_Training_-_Bazooka
    ["Listen carefully! The bandit leader, Thanta, has recently found a very strange device."] = "Słuchaj uważnie! Przywódca bandytów, Tanta, znalazł ostatnio bardzo dziwne urządzenie.", -- A_Space_Adventure:ice01
    ["Listen up, maggot!"] = "Posłuchaj, robaku!", -- User_Mission_-_Dangerous_Ducklings
    ["Listen up, maggot!!"] = "Słuchaj mnie, gnido!",
    ["Little did they know that this hunt will mark them forever..."] = "Nie zdają sobie sprawy, że to polowanie naznaczy ich na zawsze...", -- A_Classic_Fairytale:shadow
    ["Little Obstacle Course"] = "Mały tor przeszkód", -- Basic_Training_-_Rope
    ["Lively Lifeguard"] = "Żwawy Ratownik",
    ["Lonely Cries: [Rise the water if no hog is in the circle and deal 6 damage to all enemy hogs.]"] = "Samotne łkanie: [Podnosi wodę, jeśli nie ma jeża w kole i zadaje 6 obrażeń każdemu wrogiemu jeżowi.]", -- Continental_supplies
    ["Lonely Cries"] = "Samotne łkanie", -- Continental_supplies
    ["Long Jump: [Enter]"] = "Długi skok: [Enter]", -- Basic_Training_-_Movement
    ["Long Jump: Tap the [Curvy Arrow] button for long"] = "Długi skok: Przytrzymaj dłużej przycisk [Zakręconej strzałki]", -- Basic_Training_-_Movement, A_Classic_Fairytale:first_blood
    ["Long Live The Queen"] = "Niech żyje królowa", -- A_Classic_Fairytale:queen
    ["Look around: [Mouse movement]"] = "Rozejrzyj się: [Ruch myszą]", -- Basic_Training_-_Movement
    ["Look around: [Tap or swipe on the screen]"] = "Rozejrzyj się: [Wciśnij lub przejedź po ekranie]", -- Basic_Training_-_Movement
    ["Look, boss! There is the target!"] = "Zobacz, szefie! Tam jest cel!", -- A_Space_Adventure:moon01
    ["Look, I had no choice!"] = "Słuchaj, nie miałem wyboru!", -- A_Classic_Fairytale:backstab
    ["Look out! There's more of them!"] = "Uważaj! Jest ich więcej!", -- A_Classic_Fairytale:backstab
    ["Look out! We're surrounded by cannibals!"] = "Uważaj! Jesteśmy otoczeni przez kanibali!", -- A_Classic_Fairytale:enemy
    ["Looks like the whole world is falling apart!"] = "Wygląda, jakby cały świat się walił!", -- A_Classic_Fairytale:enemy
    ["Look to the left and do a backwards jump towards the mushroom."] = "Spójrz w lewo i zrób skok w tył w stronę grzyba.", -- A_Classic_Fairytale:first_blood
    ["Loon"] = "Gbur", -- The_Specialists
    ["Loopy"] = "Wariat",
    ["Losing Condition: Destroy"] = "Warunek przegranej: Zniszcz", -- HedgeEditor
    ["Low Gravity: Gravity is %i%%"] = "Niska grawitacja: Grawitacja wynosi %i%%", -- Gravity
    ["Loyal Highlander: Eliminate enemy hogs to take their weapons"] = "Lojalny Highlander: Wyeliminuj jeże wroga, by przejąć ich bronie", -- Highlander
    ["Lt. Luke"] = "Pułkownik Łukasz",
    ["Lucifer"] = "Lucyfer", -- portal
    ["Luck: %d%% (modifier for crates)"] = "Szczęście: %d%% (modyfikator do skrzyń)", -- Battalion
    ["Luckily, I've managed to snatch some of them."] = "Szczęśliwie, udało mi się uszczknąć parę z nich.", -- A_Classic_Fairytale:united
    ["Ludicrous kill!"] = "Niedorzeczne zabójstwo!", -- Mutant
    ["Lugia"] = "Lugia",
    ["Luigi"] = "Luigi",
    ["Made it!"] = "Udało się!", -- ClimbHome
    ["Mahoney"] = "Mahoney",
    ["Make fun of me when I fart …"] = "Śmiać się ze mnie, gdy pierdnę...", -- A_Classic_Fairytale:queen
    ["Manual: https://hedgewars.org/hedgeeditor"] = "Instrukcja: https://hedgewars.org/hedgeeditor", -- HedgeEditor
    ["Many long forgotten things can be found in the same tunnels that we are about to explore!"] = "Wiele dawno zapomnianych rzeczy może być znalezione w tych samych tunelach, które mamy właśnie odkrywać!", -- A_Space_Adventure:fruit02
    ["Many meters below the surface ..."] = "Wiele metrów pod ziemią...", -- A_Space_Adventure:desert02
    ["Mario"] = "Mario",
    ["Mark gears for win/lose conditions"] = "Zaznacz rzeczy jako warunki wygranej/przegranej", -- HedgeEditor
    ["Mark/unmark gear: [Left Click]"] = "Zaznacz/odznacz rzecz: [Lewy klik]", -- HedgeEditor
    ["- Massive weapon bonus on first turn"] = "- Masywny bonus broni w pierwszej turze", -- Continental_supplies
    ["Max Citrus"] = "Maks Cytrus", -- A_Space_Adventure:fruit01
    ["Maybe you should try an easier map next time."] = "Może następnym razem powinieneś spróbować łatwiejszej mapy.", -- Racer
    ["Maybe you should try an easier TechRacer map."] = "Może powinieneś spróbować łatwiejszej mapy TechWyścigu.", -- TechRacer
    ["Maybe you should try easier waypoints next time."] = "Może następnym razem powinieneś spróbować łatwiejszych punktów kontrolnych.", -- Racer
    ["May the spirits aid you in all your quests!"] = "Niech dusze pomogą wam w waszych zadaniach!", -- A_Classic_Fairytale:backstab
    ["Meals"] = "Mils",
    ["Medicine: [Fire some exploding medicine that will heal all hogs effected by the explosion]"] = "Lekarstwo: [Wystrzel trochę eksplodującego leku, który uleczy wszystkie jeże w jej zasięgu]", -- Continental_supplies
    ["Medicine"] = "Lekarstwo", -- Continental_supplies
    ["Medic"] = "Medyk", -- Battalion
    ["MEDIUM"] = "ŚREDNI", -- Continental_supplies
    ["Mega kill!"] = "Mega zabójstwo!", -- Mutant
    ["Meiwes"] = "Meifs", -- A_Classic_Fairytale:backstab
    ["mikade"] = "mikade",
    ["Mindy"] = "Mindi", -- A_Classic_Fairytale:united
    ["Mine Deployer"] = "Rozmieszczacz Min",
    ["Mine Placement Mode"] = "Tryb Ustawiania Min", -- Construction_Mode
    ["MINE PLACEMENT MODE"] = "TRYB USTAWIANIA MIN", -- HedgeEditor
    ["Mines explode after %d s."] = "Miny wybuchają po %d s.", -- Mutant
    ["Mines time: 0s-5s"] = "Czas min: 0s-5s", -- SimpleMission
    ["Mines time: 0 seconds"] = "Czas min: 0 sekund", -- portal, User_Mission_-_Spooky_Tree, User_Mission_-_Teamwork, User_Mission_-_The_Great_Escape, A_Space_Adventure:desert01, A_Space_Adventure:final, A_Space_Adventure:fruit02, A_Space_Adventure:ice01
    ["Mines time: 1.5 seconds"] = "Czas min: 1.5 sekundy", -- A_Space_Adventure:death01
    ["Mines time: %.1fs"] = "Czas min: %.1fs", -- SimpleMission
    ["Mines time: 1 second"] = "Czas min: 1 sekunda", -- User_Mission_-_Diver, User_Mission_-_Newton_and_the_Hammock, A_Space_Adventure:desert02
    ["Mines time: %.2fs"] = "Czas min: %.2fs", -- SimpleMission
    ["Mines time: 3 seconds"] = "Czas min: 3 sekundy", -- A_Classic_Fairytale:journey
    ["Mines time: 5 seconds"] = "Czas min: 5 sekund", -- A_Classic_Fairytale:dragon, A_Classic_Fairytale:family, A_Classic_Fairytale:journey
    ["Mines time: %ds"] = "Czas min: %ds", -- SimpleMission
    ["Mine Strike"] = "Nalot", -- Construction_Mode
    ["Minion"] = "Sługus", -- A_Space_Adventure:moon01
    ["Minions"] = "Sługusy", -- A_Space_Adventure:moon01
    ["Mission failed!"] = "Misja zakończona niepowodzeniem!", -- Big_Armory
    ["Mission failure in %d s"] = "Porażka misji w %d s", -- Big_Armory
    ["Mission"] = "Misja", -- HedgeEditor
    ["Mission lost!"] = "Misja stracona!", -- Basic_Training_-_Grenade
    ["Mission panel: [M]"] = "Panel misji: [M]", -- Basic_Training_-_Movement
    ["Mission Panel"] = "Panel misji", -- Basic_Training_-_Movement
    ["Mission succeeded!"] = "Misja ukończona!", -- portal, User_Mission_-_Bamboo_Thicket, User_Mission_-_Dangerous_Ducklings, User_Mission_-_Diver, User_Mission_-_Spooky_Tree, User_Mission_-_Teamwork_2, User_Mission_-_Teamwork, SimpleMission, HedgeEditor
    ["Mission won!"] = "Misja wygrana!", -- Basic_Training_-_Grenade
    ["Mister Pear"] = "Pan Perła", -- A_Space_Adventure:fruit01, A_Space_Adventure:fruit02
    ["Mixed %d"] = "Mieszane %d", -- WxW
    ["Mixed"] = "Mieszane", -- WxW
    ["Modes: Activate “highland”, “king” or “points” mode by putting mode=<name>|into the script parameter"] = "Tryby: Aktywuj tryb \"highland\", \"king\" lub \"points\" dodając mode=<name>|do parametrów skryptu", -- Battalion
    ["Modifiers: Unlimited ammo, per-hog ammo"] = "Modyfikatory: Nieskończona amunicja, amunicja na jeża", -- Battalion
    ["Modifiers: Unlimited ammo, shared clan ammo"] = "Modyfikatory: Nieskończona amunicja, amunicja klanowa", -- Battalion
    ["Modifiers: Unlimited attacks, per-hog ammo"] = "Modyfikatory: Nieskończony atak, oddzielna amunicja dla jeży", -- Battalion
    ["Modifiers: Unlimited attacks, shared clan ammo"] = "Modyfikatory: Nieskończone ataki, dzielona amunicja klanowa", -- Battalion
    ["Modify Sprite under Cursor: [Left Click]"] = "Modyfikuj Grafikę pod Kursorem: [Lewy klik]", -- HedgeEditor
    ["Molly"] = "Moli",
    ["Molotov"] = "Koktajl Mołotowa", -- Continental_supplies
    ["Monster kill!"] = "Potworne zabójstwo!", -- Mutant
    ["Monsters"] = "Potwory",
    ["Mooney"] = "Muni",
    ["Morris"] = "Moris",
    ["Most mines are not active."] = "Większość min jest nieaktywna", -- A_Space_Adventure:desert02
    ["Most of the destructible terrain in marked with blue color"] = "Większość zniszczalnego terenu jest zaznaczone na niebieski kolor", -- A_Space_Adventure:desert01
    ["Most of the destructible terrain is marked with dashed lines."] = "Większość zniszczalnego terenu jest zaznaczona kreskowanymi liniami.", -- A_Space_Adventure:desert01
    ["Most of the time you'll be able to use the freezer only."] = "Przez większość czasu będziesz mógł użyć tylko zamrażarki.", -- A_Space_Adventure:ice01
    ["Movement: [Up], [Down], [Left], [Right]"] = "Poruszanie się: [Góra], [Dół], [Lewo], [Prawo]",
    ["Mr Mango"] = "Pan Mango", -- A_Space_Adventure:fruit01
    ["Mudkip"] = "Mudkip",
    ["Multi-shot! +15 points!"] = "Mutli-strzał! +15 punktów!", -- Space_Invasion
    ["Multi-Use: You can take and use the same ammo type multiple times in a turn"] = "Wielokrotne użycie: Możesz wziąć i użyć amunicji tego samego typu podczas tury", -- Highlander
    ["Muriel"] = "Miuriel", -- A_Classic_Fairytale:backstab, A_Classic_Fairytale:dragon, A_Classic_Fairytale:family, A_Classic_Fairytale:queen
    ["Muscle Dissolver"] = "Rozpuszczacz Mięśni", -- A_Classic_Fairytale:shadow
    ["Mushroom Kingdom"] = "Grzybowe Królestwo",
    ["Mutant"] = "Mutant", -- Mutant
    ["My First Bazooka"] = "Moja pierwsza bazooka", -- Basic_Training_-_Bazooka
    ["My flying saucer stopped working!"] = "Mój latający talerz przestał działać!", -- A_Space_Adventure:ice01
    ["Nade Boy"] = "Natowy Chłopak", -- Basic_Training_-_Grenade
    ["Nah, probably everyone was just stupid."] = "Nie, prawdopodobnie każdy był po prostu głupi.", -- A_Space_Adventure:final
    ["Name"] = "Imię", -- A_Classic_Fairytale:queen
    ["Nancy Screw"] = "Nansi Gwóźdź", -- A_Classic_Fairytale:enemy, A_Classic_Fairytale:queen
    ["Napalm"] = "Napalm", -- Construction_Mode
    ["Napalm rocket: [Fire a bomb with napalm!]"] = "Napalmowa rakieta: [Wystrzel bombę z napalmem!]", -- Continental_supplies
    ["Napalm Rocket"] = "Rakieta napalmowa", -- Continental_supplies
    ["Naranja Jed"] = "Narandża Dżed", -- A_Space_Adventure:fruit01
    ["Naughty Ninja"] = "Niegrzeczny Ninja", -- User_Mission_-_Dangerous_Ducklings
    ["Near a PAotH base on the moon ..."] = "Niedaleko bazy PSJ na księżycu...", -- A_Space_Adventure:moon01
    ["Near Secret Base 17 of PAotH in the rural Hogland ..."] = "Niedaleko Sekretnej Bazy 17 PSJ w miejskim Jeżlandzie...", -- A_Space_Adventure:cosmos
    ["nemo"] = "nemo",
    ["Neutralize your enemies and be careful!"] = "Zneutralizuj swoich wrogów i bądź ostrożny!", -- A_Space_Adventure:moon01
    ["New barrels per turn: %d"] = "Nowe beczki na turę: %d", -- Tumbler
    ["New clan record: %.1fs"] = "Nowy rekord klanu: %.1fs", -- Racer, TechRacer
    ["NEW fastest lap: "] = "NOWE najszybsze okrążenie: ",
    ["New mines per turn: %d"] = "Nowe miny na turę: %d", -- Tumbler
    ["New race record: %.1fs"] = "Nowy rekord wyścigu: %.1fs", -- Racer, TechRacer
    ["Newton and the Hammock"] = "Niuton i Hamak", -- User_Mission_-_Newton_and_the_Hammock
    ["Next target is ready!"] = "Następny cel jest gotowy!", -- Basic_Training_-_Flying_Saucer
    ["Next time you play \"Searching in the dust\" you'll have an RC plane available."] = "Następnym razem grając w \"Poszukiwanie w kurzu\", będziesz miał dostępny radiowy samolocik.", -- A_Space_Adventure:desert03
    ["Nicely done, meatbags!"] = "Ładnie zrobione, mięsne worki!", -- A_Classic_Fairytale:enemy
    ["Nice!"] = "Nieźle!", -- A_Space_Adventure:cosmos
    ["Nice! Now hurry and get down! You have to rescue my friends!"] = "Nieźle! Teraz pośpiesz się i tu zejdź! Musisz uratować moich przyjaciół!", -- A_Space_Adventure:moon01
    ["Nice, then I should get the part as soon as possible!"] = "Nieźle, więc powinienem zabrać tę część tak szybko, jak to możliwe!", -- A_Space_Adventure:ice01
    ["Nice work!"] = "Ładna robota!", -- A_Classic_Fairytale:enemy
    ["Nice work, meatbags!"] = "Ładna robota, worki na mięso!", -- A_Classic_Fairytale:queen
    ["Nice work, %s!"] = "Ładna robota, %s!", -- A_Classic_Fairytale:dragon
    ["Nilarian"] = "Nilarianin", -- A_Classic_Fairytale:queen
    ["Ninja"] = "Ninja", -- Battalion, HedgeEditor, The_Specialists
    ["Ninpo"] = "Ninpo",
    ["Nobody Laugh"] = "Nikt się nie śmieje", -- User_Mission_-_Nobody_Laugh
    ["Nobody managed to finish the race. What a shame!"] = "Nikomu nie udało się ukończyć wyścigu. Co za wstyd!", -- Racer, TechRacer
    ["Nobody takes walks every day!"] = "Nikt nie chadza na spacery każdego dnia!", -- A_Classic_Fairytale:epil
    ["No continent selected"] = "Nie wybrano kontynentu", -- Continental_supplies
    ["No, I am afraid I had to travel light."] = "Nie, obawiam się, że musiałem podróżować bez bagażu.", -- A_Space_Adventure:moon01
    ["No, I came back to help you out..."] = "Nie, wróciłem, by ci pomóc...", -- A_Classic_Fairytale:shadow
    ["No...I wonder where they disappeared?!"] = "Nie... ciekawe, gdzie oni zniknęli?!", -- A_Classic_Fairytale:journey
    ["Nom-Nom"] = "Nom-Nom", -- A_Classic_Fairytale:journey
    ["NomNom"] = "NomNom", -- A_Classic_Fairytale:united
    ["No Multi-Use: Once you used an ammo, you can’t take it again in this turn"] = "Bez wielokrotnego użycia: Jak już użyjesz amunicji, nie możesz wziąć jej ponownie w tej turze", -- Highlander
    ["Noo, Thanta has to stay alive!"] = "Niee, Tanta musi zostać przy życiu!", -- A_Space_Adventure:ice01
    ["Nope. It was one fast mole, that's for sure."] = "Nie. To był jeden szybki kret, to na pewno.", -- A_Classic_Fairytale:shadow
    ["No! Please, help me!"] = "Nie! Proszę, pomóż mi!", -- A_Classic_Fairytale:journey
    ["No problem, Captain!"] = "Nie ma sprawy, Kapitanie!", -- A_Space_Adventure:fruit01
    ["No problem, I would do anything for H!"] = "Nie ma sprawy, zrobiłbym wszystko dla J!", -- A_Space_Adventure:desert01
    ["No radar pings left!"] = "Brak pozostałych sygnałów radaru!", -- Space_Invasion
    ["Normal Girder: [1]"] = "Normalna Belka: [1]", -- HedgeEditor
    ["Normal Land: [1]"] = "Normalny Teren: [1]", -- HedgeEditor
    ["Normal Land"] = "Normalny Teren", -- HedgeEditor
    ["Normally, the mission panel disappears after a few seconds."] = "Normalnie, panel misji znika po kilku sekundach.", -- Basic_Training_-_Movement
    ["NORMAL"] = "NORMALNY", -- Continental_supplies
    ["Normal Rubber: [1]"] = "Normalna Guma: [1]", -- HedgeEditor
    ["North America"] = "Ameryka Północna", -- Continental_supplies
    ["Not being able to fight or hunt."] = "Bez możliwości walki, czy łowu.", -- A_Classic_Fairytale:queen
    ["Note: Some weapons have a second option (See continent information). Find and use them with the \""] = "Notka: Niektóre bronie mają drugą opcję (zobacz informację o kontynencie). Znajdź je i użyj przez \"", -- Continental_supplies
    ["Note: Some weapons have a second option (See continent information). Find and use them with the \"%s\" key."] = "Notka: Niektóre bronie mają drugą opcję (Patrz informacje kontynentowe). Znajdź je i użyj przyciskiem \"%s\".", -- Continental_supplies
    ["Note: This basic training assumes default controls."] = "Notka: Ten podstawowy trening zakłada domyślne sterowanie.", -- Basic_Training_-_Movement
    ["Note: Walking is disabled in this mission."] = "Uwaga: Chodzenie jest w tej misji wyłączone.", -- Basic_Training_-_Grenade
    ["Note: We only give you grenades if you stay in your flying saucer."] = "Notka: Dajemy ci granaty tylko, gdy jesteśw swoim spodku.", -- Basic_Training_-_Flying_Saucer
    ["Nothing of interest has happened."] = "Nic interesującego się nie wydarzyło.", -- Space_Invasion
    ["Not now, Fiery Water!"] = "Nie teraz, Ognista Wodo!", -- A_Classic_Fairytale:backstab
    ["Not So Friendly Match"] = "Mecz Nie-Do-Końca Towarzyski",
    ["Not you again! My head still hurts from last time!"] = "Tylko nie znowu ty! Moja głowa nadal boli od ostatniego razu!", -- A_Classic_Fairytale:shadow
    ["No waypoint to be removed!"] = "Brak punktu kontrolnego do usunięcia!", -- Racer
    ["Now collect the 2 crates to the far left and right."] = "Teraz zbierz 2 skrzynie na końcu po lewej i po prawej.", -- Basic_Training_-_Flying_Saucer
    ["Now collect the next crate!"] = "Teraz zdobądź następną skrzynię!", -- Basic_Training_-_Flying_Saucer
    ["Now dive just one more time and collect the next crate."] = "Teraz zanurkuj ostatni raz i zbierz następną skrzynię.", -- Basic_Training_-_Flying_Saucer
    ["No, we made sure of that!"] = "Nie, upewniliśmy się co do tego!", -- A_Classic_Fairytale:united
    ["Now find the next target! |Tip: Normally you lose health by falling down, so be careful!"] = "Teraz znajdź następny cel!|Wskazówka: Normalnie tracisz zdrowie po upadku, więc bądź ostrożny!", -- Basic_Training_-_Rope
    ["Now for the supreme discipline of saucer flying, the underwater attack."] = "Teraz najwyższa dyscyplina latania talerzem, podwodny atak.", -- Basic_Training_-_Flying_Saucer
    ["Now go and don't waste more of my time, you coward!"] = "Teraz idź i nie marnuj więcej mojego czasu, ty tchórzu!", -- A_Space_Adventure:fruit01
    ["Now go and play the menu mission to complete the campaign."] = "Teraz idź i zagraj w misję z menu, by ukończyć kampanię.", -- A_Space_Adventure:death01
    ["Now go to the next crate."] = "Teraz idź do następnej skrzyni", -- Basic_Training_-_Movement
    ["No! What have I done?! What have YOU done?!"] = "Nie! Co ja narobiłem?! Co TY narobiłeś?!", -- A_Classic_Fairytale:journey
    ["No. Where did he come from?"] = "Nie. Skąd on przyszedł?", -- A_Classic_Fairytale:shadow
    ["Now how do I get on the other side?!"] = "Teraz, jak mam dostać się na drugą stronę?", -- A_Classic_Fairytale:dragon
    ["Now I have to climb these trees"] = "Teraz muszę wspiąć się na te drzewa", -- A_Space_Adventure:cosmos
    ["No Wind Influcence"] = "Bez wpływu wiatru", -- Basic_Training_-_Grenade
    ["No Wind Influence"] = "Brak wpływu wiatru", -- Basic_Training_-_Grenade
    ["Now let's try to drop weapons while flying!"] = "Teraz spróbujmy zrzucić bronie latając!", -- Basic_Training_-_Flying_Saucer
    ["Now listen carefully! Below us there are tunnels that have been created naturally over the years"] = "Teraz słuchaj uważnie! Pod nami są tunele, które zostały naturalnie stworzone przez lata.", -- A_Space_Adventure:desert01
    ["Now try to get out of this bounce house|and take the next crate."] = "Teraz spróbuj wydostać się z tego odbijającego|domu i weź następną skrzynię.", -- Basic_Training_-_Movement
    ["Now use it and go to the moon PAotH station to get more fuel!"] = "Teraz użyj tego i udaj się na stację księżycową PSJ, by zdobyć więcej paliwa!", -- A_Space_Adventure:cosmos
    ["Now you have the chance to try and claim the place that you deserve among the best."] = "Teraz masz szansę, by spróbować i zdobyć zasłużone miejsce między najlepszymi.", -- A_Space_Adventure:ice02
    ["No. You and the rest of the tribe are safer there!"] = "Nie. Ty i reszta plemienia jesteście tu bezpieczniejsi.", -- A_Classic_Fairytale:backstab
    ["Objective completed! Now land safely."] = "Cel wykonany! Teraz wyląduj bezpiecznie.", -- Basic_Training_-_Flying_Saucer
    ["Objectives"] = "Cele", -- A_Space_Adventure:ice01
    ["Object Placer"] = "Stawiacz Obiektów", -- Construction_Mode
    ["Obliterate them!|Hint: You might want to take cover..."] = "Unicestwić ich!|Porada: Możesz chcieć się schować...", -- A_Classic_Fairytale:shadow
    ["Obstacle course"] = "Tor przeszkód", -- A_Classic_Fairytale:dragon
    ["Obstacle"] = "Przeszkoda", -- Basic_Training_-_Rope
    ["Of course, but you're … special."] = "Oczywiście, ale jesteś... specjalna.", -- A_Classic_Fairytale:epil
    ["Of course I am!"] = "Oczywiście, że jestem!", -- A_Classic_Fairytale:queen
    ["Of course I have to save her. What did I expect?!"] = "Oczywiście, że muszę ją uratować. Czego ja oczekiwałem?!", -- A_Classic_Fairytale:family
    ["Of course! It's all obvious now!"] = "No jasne! Teraz jest wszystko oczywiste!", -- A_Classic_Fairytale:epil
    ["Of course, I will observe the battle and intervene if necessary."] = "Oczywiście, będę obserwował bitwę i interweniował, kiedy trzeba.", -- A_Space_Adventure:fruit01
    ["OH, COME ON!"] = "OCH, NO DAWAJ!", -- A_Classic_Fairytale:journey
    ["Oh man! Learn how to fly!"] = "Człowieku! Naucz się latać!", -- A_Space_Adventure:ice02
    ["Oh, my! I forgot something!"] = "Jejku, zapomniałem o czymś!", -- A_Classic_Fairytale:queen
    ["Oh, my!"] = "Ojej!", -- A_Classic_Fairytale:first_blood
    ["Oh, my! This is even more entertaining than I've expected!"] = "Ojejku! To jest bardziej zabawne, niż myślałem!", -- A_Classic_Fairytale:backstab
    ["Oh no, not %s!"] = "O nie, to %s!", -- A_Classic_Fairytale:backstab, A_Classic_Fairytale:united
    ["Oh no, the companions have betrayed %s and stole the anti-gravity device part!"] = "O nie, kompani zdradzili %s i ukradli część urządzenia antygrawitacyjnego!", -- A_Space_Adventure:fruit02
    ["Oh no! You have died. Try again!"] = "Och nie! Umarłeś. Spróbuj jeszcze raz!", -- Basic_Training_-_Flying_Saucer
    ["Oh! Please spare me. You can take all my treasures!"] = "Och! Proszę, oszczędź mnie. Możesz zabrać wszystkie moje skarby!", -- A_Space_Adventure:ice01
    ["Oh, silly me! I forgot that I'm the shaman."] = "Och, głupi ja! Zapomniałem, że to ja jestem szamanem.", -- A_Classic_Fairytale:backstab
    ["Oh, that. We were just having fun!"] = "Och, to. Tylko się bawiliśmy.", -- A_Classic_Fairytale:queen
    ["Oh yeah! You sure know how to rope!"] = "O tak! Z pewnością wiesz jak używać liny!", -- Basic_Training_-_Rope
    ["Oh yes! I got the device part! Now it belongs to me alone."] = "O tak! Mam teraz część urządzenia! Teraz należy tylko do mnie.", -- A_Space_Adventure:fruit02
    ["Okay, I'll be extra careful!"] = "Dobrze, będę ekstra ostrożny!", -- A_Space_Adventure:desert01
    ["Okay, now destroy the target|using the baseball bat."] = "W porządku, teraz zniszcz cel,|używając kija bejsbolowego.", -- Basic_Training_-_Rope
    ["Okay, then you have to go and take some of the weapons we have hidden in case of an emergency!"] = "Dobrze, więc musisz iść i wziąć trochę broni, które ukryliśmy na nagły wypadek!", -- A_Space_Adventure:moon01
    ["Okay then!"] = "Zatem w porządku!", -- A_Space_Adventure:fruit02
    ["Old One Eye"] = "Stary Jednooki",
    ["Oleg"] = "Oleg",
    ["Olive"] = "Oliwka", -- A_Classic_Fairytale:united
    ["Omnivore"] = "Wszystkożerca", -- A_Classic_Fairytale:first_blood
    ["Once upon a time, on an island with great natural resources, lived two tribes in heated conflict..."] = "Pewnego razu, na wyspie z bogatymi zasobami naturalnymi, żyły dwa plemienia w gorącym konflikcie...", -- A_Classic_Fairytale:first_blood
    ["Once you set off the proximity trigger, Mr. Mine is not your friend"] = "Kiedy uruchomisz zbliżeniowy wyzwalacz, Pani Mina nie jest twoim przyjacielem", -- ClimbHome
    ["One does not simply rope to the moon!"] = "Nikt tak po prostu nie dostaje się liną na księżyc!", -- A_Space_Adventure:cosmos
    ["One flower: Incomplete side missions"] = "Jeden kwiatek: Nieukończone misje poboczne", -- A_Space_Adventure:cosmos
    ["One shall not judge one by one's appearance!"] = "Nie powinno się kogoś sądzić po jego wyglądzie!", -- A_Classic_Fairytale:epil
    ["One tribe was peaceful, spending their time hunting and training, enjoying the small pleasures of life..."] = "Nasze plemię było pokojowe, spędzając swój czas na polowaniu i treningu, ciesząc się z małych przyjemności w życiu...", -- A_Classic_Fairytale:first_blood
    ["Oneye"] = "Jednooki", -- portal
    ["Only one hog per team allowed! Excess hogs will be removed"] = "Dozwolony tylko jeden jeż na drużynę! Nadmiarowe jeże będą usunięte", -- Mutant
    ["Only one hog per team allowed! Excess hogs will be removed."] = "Dozwolony tylko jeden jeż na drużynę! Nadmiarowe jeże będą usunięte.", -- Mutant
    ["Only %s can be trusted with the crate."] = "Tylko jeżowi %s można ufać ze skrzynią.", -- A_Space_Adventure:fruit02
    ["Only the best pilots can master the following stunts."] = "Tylko najlepsi piloci mogą opanować następujące wyczyny.", -- Basic_Training_-_Flying_Saucer
    ["Only two clans allowed! Excess hedgehogs will be removed."] = "Tylko dwa klany dozwolone! Nadmiarowe jeże będą usunięte.", -- CTF_Blizzard
    ["On the Ice Planet, where ice rules ..."] = "Na Lodowej Planecie, gdzie rządzi lód...", -- A_Space_Adventure:ice01
    ["On the other side of the moon ..."] = "Po drugiej stronie księżyca...", -- A_Space_Adventure:moon02
    ["On the Planet of Sand, you have to double check your moves ..."] = "Na Planecie Piasku musisz starannie wykonywać swoje ruchy...", -- A_Space_Adventure:desert01
    ["On this map you get %d%% fuel."] = "Na tej mapie dostajesz %d%% paliwa.", -- TechRacer
    ["On this map you get infinite fuel."] = "Na tej mapie dostajesz nieskończone paliwo.", -- TechRacer
    ["Oops...I dropped them."] = "Ups... upuściłem je.", -- A_Classic_Fairytale:united
    ["Oops, I've been spotted and I have no weapons! I am doomed!"] = "Ups, zostałem zauważony i nie mam broni! Już po mnie!", -- A_Space_Adventure:moon01
    ["Oops! You have selected the wrong hedgehog! Just try again."] = "Ups! Wybrałeś złego jeża! Po prostu spróbuj ponownie.", -- Basic_Training_-_Movement
    ["Open ammo menu: [Right click]"] = "Otwórz menu amunicji: [Prawy klik]", -- Basic_Training_-_Bazooka, Basic_Training_-_Grenade, Basic_Training_-_Movement, Basic_Training_-_Rope
    ["Open ammo menu: Tap the [Suitcase]"] = "Otwórz menu amunicji: Wciśnij [Walizkę]", -- Basic_Training_-_Bazooka, Basic_Training_-_Grenade, Basic_Training_-_Movement, Basic_Training_-_Rope
    ["Open that crate and we will continue!"] = "Otwórz tę skrzynię i będziemy kontynuować!", -- A_Classic_Fairytale:first_blood
    ["Opposing Team: "] = "Przeciwna Drużyna",
    ["Orange"] = "Pomarańcz",
    ["Orlando Boom!"] = "Orlando Boom!", -- A_Classic_Fairytale:queen
    ["Or let the next player place waypoints|if less than 2 waypoints have been placed."] = "Lub pozwól następnemu graczowi postawić punkty kontrolne|jeśli mniej niż 2 zostały postawione.", -- Racer
    ["Or maybe this was all part of an evil plan, so evil that even Prof. Hogevil can't think of it!"] = "Lub może to było wszystko częścią złego planu, tak złego, że nawet Prof. Jeżozło nie mógł go wymyśleć!", -- A_Space_Adventure:final
    ["Oscillating Gravity: Gravity periodically changes within a range from %i%% to %i%% with a period of %s"] = "Oscylująca grawitacja: Grawitacja okresowo zmienia się w zakresie od %i%% do %i%% z okrese %s.", -- Gravity
    ["Other kills don't give you points."] = "Inne zabójstwa nie dają ci punktów.", -- Mutant
    ["Ouch! That must have hurt. %s (%s) hit the ground with %d damage points."] = "Auć! To musiało boleć. %s (%s) uderzył w zimię z %d punktami obrażeń.", -- ClimbHome
    ["Ouch! That must have hurt. You mutilated your poor hedgehog hog with %d damage."] = "Auć! To musiało boleć. Upokorzyłeś swojego biednego jeża %d punktami obrażeń.", -- ClimbHome
    ["Ouch! You just took fall damage."] = "Auć! Właśnie otrzymałeś obrażenia od upadku.", -- Basic_Training_-_Movement
    ["Our tribe, our beautiful island!"] = "Nasze plemię, nasza piękna wyspa!", -- A_Classic_Fairytale:enemy
    ["Out of ammo!"] = "Brak amunicji!", -- A_Space_Adventure:desert03, Tumbler
    ["Out of ammo! Try again!"] = "Koniec amunicji! Spróbuj ponownie!", -- Basic_Training_-_Bazooka
    ["Over the Water"] = "Ponad wodą", -- Basic_Training_-_Rope
    ["PAotH has sent explosives but unfortunately the trigger mechanism seems to be faulty!"] = "PSJ wysłało ładunki wybuchowe, ale niestety, mechanizm zapalnika wydaje się być wadliwy!", -- A_Space_Adventure:cosmos
    ["PAotH"] = "PSJ", -- A_Space_Adventure:cosmos, A_Space_Adventure:death01, A_Space_Adventure:desert01, A_Space_Adventure:moon01
    ["Parachute"] = "Spadochron", -- Continental_supplies
    ["Patches"] = "Łatek",
    ["Paul McHoggy"] = "Paul McJeżu", -- A_Space_Adventure:ice01, A_Space_Adventure:ice02
    ["Pause: [P]"] = "Pauza: [P]", -- Basic_Training_-_Movement
    ["Pause: Tap the [Pause] button"] = "Pauza: Wciśnij przycisk [Pauza]", -- Basic_Training_-_Movement
    ["Penalty: If you violate above rule, you have to skip in the next turn."] = "Kara: Jeśli naruszysz powyższą zasadę, będziesz musiał opuścić następną turę.", -- WxW
    ["Penguin roar: [Deal 15 damage + 10% of your hog’s health to all hogs around you and get 2/3 back]"] = "Ryk pingwina: [Zadaj 15 obrażeń + 10% zdrowia twojego jeża wszystkim jeżom wokół ciebie i otrzymaj 2/3 z powrotem]", -- Continental_supplies
    ["Penguin roar: [Deal 15 damage + 10% of your hogs health to all hogs around you and get 2/3 back]"] = "Ryk pingwina: [Zadaje 15 obrażeń + 10% zdrowia twojego jeża wszystkim jeżom dookoła i dostaje 2/3 z powrotem]", -- Continental_supplies
    ["Penguin Roar"] = "Ryk pingwina", -- Continental_supplies
    ["Perfect! Now try to get the next crate without hurting yourself!"] = "Idealnie! Teraz spróbuj zdobyć następną skrzynię bez zranienia się!", -- A_Classic_Fairytale:first_blood
    ["Per-hog Ammo: Weapons are not shared between hogs"] = "Oddzielna amunicja dla jeży: Bronie nie są dzielone między jeżami", -- User_Mission_-_Nobody_Laugh
    ["Personal best: %.3f seconds"] = "Osobisty rekord: %.3f sekund", -- A_Space_Adventure:ice02
    ["Per team weapons"] = "Drużynowa amunicja", -- Continental_supplies
    ["Pfew! That was close!"] = "Fiu! To było bliskie!", -- A_Classic_Fairytale:shadow
    ["Phosphat"] = "Fosfat", -- portal
    ["Physicist"] = "Fizyk", -- HedgeEditor
    ["Piano Strike"] = "Zrzut pianina", -- Construction_Mode
    ["Pikachu"] = "Pikachu",
    ["Pings left: %d"] = "Pozostałe sygnały: %d", -- Space_Invasion
    ["Pink"] = "Róż",
    ["Pirates"] = "Piraci",
    ["Place 2-%d waypoints using the waypoint placement tool."] = "Postaw 2-%d punktów kontrolnych, używając narzędzia do stawiania punktów kontrolnych.", -- Racer
    ["Place 2 waypoints using the waypoint placement tool."] = "Postaw 2 punkty kontrolne, używając narzędzia do stawiania punktów kontrolnych.", -- Racer
    ["Place air mines"] = "Postaw latające miny", -- HedgeEditor
    ["Place barrels"] = "Postaw beczki", -- HedgeEditor
    ["Place cleavers"] = "Postaw tasaki", -- HedgeEditor
    ["Place/Delete Waypoint: [Left Click]"] = "Postaw/usuń punkt kontrolny: [Lewy klik]", -- HedgeEditor
    ["Place dud mines"] = "Postaw niewybuchy", -- HedgeEditor
    ["Place Gears (and more): Gear Placement Tool"] = "Postaw rzeczy (i więcej): Narzędzie do Stawiania Reczy", -- HedgeEditor
    ["Place Girder: Girder"] = "Postaw belkę: Belka", -- HedgeEditor
    ["Place Girder: [Left Click]"] = "Postaw belkę: [Lewy klik]", -- HedgeEditor
    ["Place girders"] = "[Postaw belki]", -- HedgeEditor
    ["Place health crates"] = "Postaw apteczki", -- HedgeEditor
    ["Place hedgehogs: Place your hedgehogs at the start of the game."] = "Postaw jeże: Postaw jeże na początku gry.", -- WxW
    ["Placement Mode"] = "Tryb Stawiania", -- HedgeEditor
    ["Place mines"] = "Postaw miny", -- HedgeEditor
    ["Place, modify and delete gears (e.g. objects)|and waypoints, edit hedgehog settings, values,|victory conditions, and more."] = "Postaw, modyfikuj i usuwaj rzeczy (np. obiekty)|i punkty kontrolne, edytuj utawienia jeży, wartości|warunki zwycięstwa i więcej.", -- HedgeEditor
    ["Place Object: [Left Click]"] = "Postaw obiekt: [Lewy klik]", -- HedgeEditor
    ["Place or delete waypoints"] = "Postaw lub usuń punkt kontrolny", -- HedgeEditor
    ["Place rubber"] = "Postaw gumę", -- HedgeEditor
    ["Place Rubber: Rubber"] = "Postaw gumę: Guma", -- HedgeEditor
    ["Place Sprite: [Left Click]"] = "Postaw Grafik: [Lewy klik]", -- HedgeEditor
    ["Place sprites to build land"] = "Postaw grafiki, by zbudować teren", -- HedgeEditor
    ["Place sticky mines"] = "Postaw miny samoprzylepne", -- HedgeEditor
    ["Place targets"] = "Postaw cele", -- HedgeEditor
    ["Place utility crates"] = "Postaw skrzynie z narzędziami", -- HedgeEditor
    ["Place Waypoint"] = "Postaw Punkt Kontrolny", -- HedgeEditor
    ["Place waypoint"] = "Postaw punkt kontrolny", -- Racer
    ["Place weapon crates"] = "Postaw skrzynie z brońmi", -- HedgeEditor
    ["- Place your clan flag at the end of your first turn"] = "- Ustaw flagę swojego klanu na końcu swojej pierwszej tury", -- Capture_the_Flag
    ["- Place your team flag at the end of your first turn"] = "- Postaw flagę twojej drużyny na końcu swojej pierwszej tury", -- Capture_the_Flag
    ["Planes used: %d"] = "Użyte samoloty: %d", -- User_Mission_-_RCPlane_Challenge
    ["Planes Used"] = "Użyte Samoloty", -- User_Mission_-_RCPlane_Challenge
    ["Planes Used:"] = "Użyte Samoloty:", -- User_Mission_-_RCPlane_Challenge
    ["Planets with all missions completed will be marked with two flowers."] = "Planety ze wszystkimi misjami ukończonymi będą zaznaczone dwoma kwiatkami.", -- A_Space_Adventure:cosmos
    ["Planets with completed main missions will be marked with a flower."] = "Planety z ukończonymi głównymi misjami będą zaznaczone kwiatkiem.", -- A_Space_Adventure:cosmos
    ["Play with me!"] = "Zagraj ze mną", -- A_Classic_Fairytale:shadow
    ["Please click on a crate."] = "Kliknij na skrzynię.", -- HedgeEditor
    ["Please click on a gear."] = "Kliknij na rzecz.", -- HedgeEditor
    ["Please click on a hedgehog, barrel, health crate or dud mine."] = "Kliknij na jeża, beczkę, apteczkę lub niewybuch.", -- HedgeEditor
    ["Please click on a hedgehog."] = "Kliknij na jeża.", -- HedgeEditor
    ["Please place the waypoint further away from the waterline"] = "Postaw punkt kontrolny dalej od powierzchni wody", -- Racer, TechRacer
    ["Please place the waypoint in the air and within the map boundaries"] = "Postaw punkt kontrolny w powietrzu i wewnątrz granic mapy", -- TechRacer
    ["Please place the waypoint in the air, within the map boundaries"] = "Postaw punkt kontrolny w powietrzu, wewnątrz granic mapy", -- Racer
    ["Please place your hedgehog first!"] = "Postaw najpierw swojego jeża!", -- WxW
    ["Please, stop releasing your \"smoke signals\"!"] = "Proszę, przestań wysyłać swoje \"sygnały dymne\"!", -- A_Classic_Fairytale:shadow
    ["Please wait …"] = "Proszę czekać...", -- WxW
    ["Point Blank Combo! +5 points!"] = "Kombo martwego punktu! +5 punktów!", -- Space_Invasion
    ["--- Points Mode ---"] = "--- Tryb Punktów ---", -- Battalion
    ["--- Points ---"] = "--- Punkty ---", -- Battalion
    ["Poisonous Apple"] = "Zatrute Jabłko", -- A_Space_Adventure:fruit02
    ["Poisonous, deals no damage."] = "Zatrute, nie zadaje obrażeń.", -- Continental_supplies
    ["Poison"] = "Truciciel",
    ["Pokémon"] = "Pokémon",
    ["Poor %s (%s) died %d times."] = "Biedny %s (%s) umarł %d razy.", -- Mutant
    ["Population"] = "Populacja", -- Continental_supplies
    ["Porkey"] = "Jeżozwierzuś",
    ["Portal hint: one goes to the destination, and one is the entrance.|"] = "Porada do portalu: jeden idzie do celu, a drugi jest wejściem.|", -- A_Classic_Fairytale:dragon
    ["Portal hint: One goes to the destination, the other one is the entrance.|"] = "Wskazówka do portali: Jeden idzie do celu, drugi jest wejściem.|", -- A_Classic_Fairytale:dragon
    ["Portal Mind Challenge"] = "Wyzwanie portalowego umysłu", -- portal
    ["Precise Aim: [Left Shift]"] = "Precyzyjne celowanie: [Lewy Shift]", -- Basic_Training_-_Movement
    ["Precise Aim: [Left Shift] + [Up]/[Down]"] = "Precyzyjne celowanie: [Lewy Shift] + [Góra]/[Dół]", -- Basic_Training_-_Bazooka, Basic_Training_-_Grenade
    ["Precise flying"] = "Precyzyhne ltanie", -- A_Space_Adventure:desert03
    ["Precise: Remove previous waypoint"] = "Precyzja: Usuń poprzedni punkt kontrolny", -- Racer
    ["Precise shooting"] = "Precyzyjne strzelanie", -- A_Space_Adventure:fruit03
    ["Predator"] = "Predator", -- portal
    ["Prepare for battle!"] = "Przygotować się do bitwy!", -- A_Space_Adventure:moon01
    ["Prepare to fight"] = "Przygotuj się do walki", -- A_Space_Adventure:moon01
    ["Prepare to flee!"] = "Przygotuj się do ucieczki!", -- A_Space_Adventure:cosmos
    ["Prepare yourself, %s!"] = "Przygotuj się, %s!", -- The_Specialists
    ["Press [Attack] (space bar by default) to start,|repeadedly tap the up, left and right movement keys to accelerate."] = "Wciśnij [Atak] (domyślnie spacja), by zacząć,|wielokrotnie wciskaj góra, lewo i prawo, by przyśpieszyć.", -- Basic_Training_-_Flying_Saucer
    ["Press [Attack] (space bar by default) to start,|repeatedly tap the up, left and right movement keys to accelerate."] = "Wciśnij [Atak] (domyślnie spacja), by zacząć,|wciśnij wielokrotnie przyciski ruchu w górę, lewo i prawo, by przyśpieszyć.", -- Basic_Training_-_Flying_Saucer
    ["Press [Attack] to begin."] = "Wciśnij [Atak], by zacząć.", -- A_Classic_Fairytale:first_blood
    ["Press [Attack] to confirm."] = "Wciśnik [Atak], by zatwierdzić.", -- Continental_supplies
    ["Press [Attack] to select this continent!"] = "Wciśnij [Atak], by wybrać ten kontynent!", -- Continental_supplies
    ["Press [Left] and [Right] to change the difficulty."] = "Wciśnij [Lewo] i [Prawo], by zmienić trudność.", -- A_Classic_Fairytale:first_blood
    ["Press [Left] or [Right] to move around, [Long Jump] to jump forwards."] = "Wciśnij [Lewo] lub [Prawo], by się poruszać, [Długi skok], by skoczyć w przód.", -- A_Classic_Fairytale:first_blood
    ["Press [Long jump] to accept this configuration and begin placing hedgehogs."] = "Wciśnij [Długi skok] by zaakceptować tę konfigurację i zacząć ustawiać jeże.", -- WxW
    ["Press [Long jump] to accept this configuration and start the game."] = "Wciśnij [Długi skok] by zaakceptować tę konfigurację i zacząć grę.", -- WxW
    ["Press [M] to see the mission texts"] = "Wciśnij [M], by zobaczyć tekst misji", -- Basic_Training_-_Movement
    ["Press [Precise] to skip intro"] = "Naciśnij [Precyzja] by pominąć intro",
    ["Press [Up] and [Down] to move between menu items.|Press [Attack], [Left], or [Right] to toggle."] = "Wciśnij [Góra] i [Dół], by poruszać się między pozycjami menu.|Wciśnij [Atak], [Lewo] lub [Prawo], by przełączyć.", -- WxW
    ["Prestigious Pilot"] = "Prestiżowy pilot", -- User_Mission_-_RCPlane_Challenge
    ["Princess"] = "Księżniczka", -- A_Classic_Fairytale:family, A_Classic_Fairytale:journey
    ["Princess Peach"] = "Księżniczka Peach",
    ["Problems, dude? Chillax!"] = "Problem, koleś? Wrzuć na luz!", -- A_Classic_Fairytale:epil
    ["Professional pilot"] = "Profesjonalny pilot", -- User_Mission_-_RCPlane_Challenge
    ["Professional stunt pilot"] = "Profesjonalny pilot kaskaderski", -- User_Mission_-_RCPlane_Challenge
    ["Professor Hogevil, then known as James Hogus, worked for PAotH back in my time."] = "Profesor Jeżozło, wtedy znany jako Jakub Jeżus, za moich czasów pracował dla PSJ.", -- A_Space_Adventure:moon02
    ["Professor"] = "Profesor", -- A_Space_Adventure:death01, A_Space_Adventure:moon01
    ["Professor's Team"] = "Drużyna Profesora", -- A_Space_Adventure:death01
    ["Prof. Hogevil"] = "Prof. Jeżozło", -- A_Space_Adventure:death01, A_Space_Adventure:moon01
    ["Protect the King: When the king dies, so does the team"] = "Chroń Króla: Kiedy król umiera, drużyna ginie razem z nim", -- Battalion
    ["Protect yourselves!|Grenade hint: set the timer with [1-5], aim with [Up]/[Down] and hold [Space] to set power"] = "Broń się!|Porada do granatu: ustaw zapalnik używając [1-5], celuj za pomocą [Góra]/[Dół] i przytrzymaj spację, by ustawić moc", -- A_Classic_Fairytale:shadow
    ["Purple"] = "Fiolet",
    ["Pyromancer"] = "Piromanta", -- Battalion
    ["Pyro"] = "Piro", -- HedgeEditor, The_Specialists
    ["Quit: [Esc]"] = "Wyjście: [Esc]", -- Basic_Training_-_Movement
    ["Race complexity limit reached"] = "Limit złożoności wyścigu osiągnięty", -- Racer, TechRacer
    ["Race failed!"] = "Wyścik nieudany!", -- A_Space_Adventure:moon02
    ["Racer tool"] = "Narzędzie wyścigu", -- Racer
    ["Racer"] = "Wyścig", -- Racer
    ["Race"] = "Wyścig", -- TrophyRace
    ["Rachel"] = "Rachela", -- A_Classic_Fairytale:dragon, A_Classic_Fairytale:family, A_Classic_Fairytale:queen
    ["Radar: Off"] = "Radar: Wyłączony", -- WxW
    ["Radar: On"] = "Radar: Włączony", -- WxW
    ["Radar Ping: [High jump]"] = "Sygnał radaru: [Wysoki skok]", -- Space_Invasion
    ["Radar: Show after crate drop"] = "Radar: Pokaż po zrzucie skrzyni", -- WxW
    ["Raging Buffalo"] = "Wściekły Bawół", -- A_Classic_Fairytale:backstab, A_Classic_Fairytale:dragon, A_Classic_Fairytale:enemy, A_Classic_Fairytale:epil, A_Classic_Fairytale:family, A_Classic_Fairytale:queen, A_Classic_Fairytale:united
    ["Ramesses"] = "Ramzes",
    ["Ramon"] = "Ramon", -- A_Classic_Fairytale:enemy, A_Classic_Fairytale:epil, A_Classic_Fairytale:family, A_Classic_Fairytale:queen, A_Classic_Fairytale:shadow
    ["Random continent"] = "Losowy kontynent", -- Continental_supplies
    ["Rank: %s"] = "Ranga: %s", -- User_Mission_-_RCPlane_Challenge
    ["Razac"] = "Razak", -- portal
    ["RC Plane Challenge"] = "Wyzwanie Samolotowe", -- User_Mission_-_RCPlane_Challenge
    ["RC Plane"] = "Radiowy samolocik", -- Construction_Mode
    ["Reach and destroy the final target to win."] = "Dotrzyj i zniszcz ostatni cel, by wygrać.", -- Basic_Training_-_Rope
    ["Read the challenge objectives from within the mission for more details."] = "Przeczytaj cele wyzwania z misji po więcej szczegółów.", -- A_Space_Adventure:death02, A_Space_Adventure:desert03, A_Space_Adventure:fruit03
    ["Ready for Battle?"] = "Gotowy do bitwy?", -- A_Space_Adventure:fruit01
    ["Really?! You thought you could harm me with your little toys?"] = "Naprawdę?! Myślałeś, że mógłbyś zranić mnie swoimi zabaweczkami?", -- A_Classic_Fairytale:shadow
    ["Red"] = "Czerwony",
    ["Reflector Shield: Reflects enemy projectiles."] = "Tarcza Reflektorowa: Odbija wrogie pociski.", -- Construction_Mode
    ["Reflector Shield"] = "Tarcza Reflektorowa", -- Construction_Mode
    ["Regurgitator"] = "Zwracacz", -- A_Classic_Fairytale:backstab
    ["Reinforcements! +2 of each weapon!"] = "Posiłki! +2 każdej broni!", -- A_Space_Adventure:death02
    ["Reinforcements"] = "Posiłki", -- A_Classic_Fairytale:backstab
    ["Release rope: [Attack]"] = "Wypuść linę: [Atak]", -- Basic_Training_-_Rope
    ["Remember: Hold down [Left Shift] to prevent slipping"] = "Pamiętaj: Przytrzymaj [Lewy Shift], by zapobiec ślizganiu się", -- Basic_Training_-_Movement
    ["Remember! Many will seek the anti-gravity device! Now go, hurry up!"] = "Pamiętaj! Wielu będzie szukać urządzenia antygrawitacyjnego! Teraz idź, pośpiesz się!", -- A_Space_Adventure:cosmos
    ["Remember: The rope only bend around objects, |if it doesn't hit anything it's always stright!"] = "Pamiętaj: Lina zagina się tylko wokół obiektów,|jeśli nic nie uderzy, zawsze jest prosta!", -- Basic_Training_-_Rope
    ["Remember this, pathetic animal: when the day comes, you will regret your blind loyalty!"] = "Zapamiętaj to, żałosny zwierzęciu: kiedy nadejdzie dzień, pożałujesz swojej ślepej lojalności!", -- A_Classic_Fairytale:shadow
    ["Remember this, pathetic animal: When the day comes, you will regret your blind loyalty!"] = "Zapamiętaj to, żałosny zwierzu: Kiedy nadejdzie dzień, pożałujesz swojej ślepej lojalności!", -- A_Classic_Fairytale:shadow
    ["Replenishment: Weapons are restocked on turn start of a new hog"] = "Uzupełnianie: Bronie są uzupełnione na początku tury nowego jeża", -- Highlander
    ["Repositioning Mode"] = "Tryb Przesuwania", -- HedgeEditor
    ["REPOSITIONING MODE"] = "TRYB PRZESUWANIA", -- HedgeEditor
    ["Rescue the imprisoned PAotH team and get the fuel!"] = "Uratuj uwięziony zespół PSJ i zdobądź paliwo!", -- A_Space_Adventure:moon01
    ["Respawner"] = "Odradzacz", -- Construction_Mode
    ["Respawner: Resurrects dead hogs."] = "Odradzacz: Ożywia martwe jeże.", -- Construction_Mode
    ["Resurrector"] = "Wskrzeszacz", -- Construction_Mode
    ["Retract/Extend rope: [Up]/[Down]"] = "Zwiń/Rozwiń linę: [Góra]/[Dół]", -- Basic_Training_-_Rope
    ["- Return the enemy flag to your base to score"] = "- Zanieś wrogą flagę do swojej bazy, by zapunktować", -- Capture_the_Flag
    ["Return to Leaks A Lot!"] = "Wróć do Sporego Przecieka!", -- A_Classic_Fairytale:shadow
    ["Return to the mission menu by pressing the \"Go back\" button."] = "Wróć do menu misji, wciskając przycisk \"Wstecz\".", -- A_Space_Adventure:cosmos
    ["Return to the Surface"] = "Wróć na Powierzchnię", -- A_Space_Adventure:fruit02
    ["Return to the training menu by pressing the “Go back” button."] = "Wróć do menu treningu, wciskając przycisk \"Powrót\".", -- Basic_Training_-_Movement
    ["Rider"] = "Jeźdźec", -- portal
    ["Rifleman"] = "Strzelec", -- Battalion
    ["Righteous Beard"] = "Cnotliwa Broda", -- A_Classic_Fairytale:backstab, A_Classic_Fairytale:dragon, A_Classic_Fairytale:enemy, A_Classic_Fairytale:epil, A_Classic_Fairytale:family, A_Classic_Fairytale:first_blood, A_Classic_Fairytale:queen, A_Classic_Fairytale:united
    ["Ripe"] = "Dojrzały",
    ["Rise the water if nobody else is in the circle and deal 6 damage to all enemy hogs."] = "Podwyższ wodę, jeśli nikt inny nie jest w kole i zadaj 6 obrażeń wszystkim wrogim jeżom.", -- Continental_supplies
    ["Robert Yellow Apple"] = "Robert Żółte Jabłko", -- A_Space_Adventure:fruit01
    ["Rocket"] = "Rakieta", -- Big_Armory
    ["Ronald"] = "Ronald", -- portal
    ["Roof"] = "Dach", -- WxW
    ["Rope-knocking Challenge"] = "Wyzwania Uderzania Liną", -- User_Mission_-_Rope_Knock_Challenge
    ["Rope Master"] = "Mistrz Liny", -- Basic_Training_-_Rope
    ["Ropes and Crates"] = "Liny i Skrzynie", -- Challenge_-_Speed_Shoppa_-_Ropes
    ["Ropes can be fired again in the air!"] = "Liny mogą być wystrzelone ponownie w powietrzu!", -- A_Classic_Fairytale:first_blood
    ["Rope Team"] = "Linowa Drużyna", -- Basic_Training_-_Rope
    ["Rope Training"] = "Trening linowy", -- Basic_Training_-_Rope
    ["Rope Weapons"] = "Linowe bronie", -- Basic_Training_-_Rope
    ["Roshi"] = "Roshi",
    ["Rot Molester"] = "Zepsuty Molester", -- A_Classic_Fairytale:shadow
    ["Rotten"] = "Zepsuty",
    ["Round draw"] = "Remis", -- Racer, TechRacer
    ["Round %d (Sudden Death in round %d)"] = "Runda %d (Nagła Śmierć w rundzie %d)", -- Battalion
    ["Round limit: %d"] = "Limit rund: %d", -- Racer
    ["Round Limit: %d"] = "Limit rund: %d", -- Space_Invasion
    ["Round limit:"] = "Limit rund: ", -- TechRacer
    ["Rounds complete: %d/%d"] = "Ukończone rundy: %d/%d", -- Racer, Space_Invasion, TechRacer
    ["Round score: %d"] = "Wynik rundy: %d", -- Space_Invasion
    ["Round's slowest lap: %.3fs by %s"] = "Najwolniejsze okrążenie rundy: %.3fs (%s)", -- TrophyRace
    ["RS1"] = "RS1", -- A_Space_Adventure:fruit03
    ["RS2"] = "RS2", -- A_Space_Adventure:fruit03
    ["Rubber"] = "Guma", -- Construction_Mode, HedgeEditor
    ["Rubber Placement Mode"] = "Tryb Stawiania Gumy", -- Construction_Mode
    ["RUBBER PLACEMENT MODE"] = "TRYB STAWIANIA GUMY", -- HedgeEditor
    ["Rules:"] = "Zasady:", -- Capture_the_Flag
    ["RULES:"] = "ZASADY:", -- Frenzy
    ["Rules: "] = "Zasady: ", -- Mutant
    ["Run away, you coward!"] = "Uciekaj, ty tchórzu!", -- A_Space_Adventure:desert01
    ["Running displacement algorithm …"] = "Uruchamianie algorytmu wysiedlania...", -- A_Classic_Fairytale:queen
    ["Running for survival"] = "Bieg o przetrwanie", -- A_Space_Adventure:desert02
    ["Rusted Diego"] = "Zardzewiały Diego",
    ["Rusty Joe"] = "Rdzawy Joe", -- A_Classic_Fairytale:queen
    ["Ryu"] = "Ryu",
    ["%s: %.1fs"] = "%s: %.1fs", -- Racer, TechRacer
    ["%s (+1)"] = "%s (+1)", -- A_Space_Adventure:fruit03
    ["Sabotage all hogs in the circle and fire a cluster above you.|Sabotaged hogs lose health and have to deal with a very high gravity during their turn."] = "Sabotuj wszystkie jeże w kole i wystrzel odłamek nad siebie.|Sabotowane jeże tracą zdrowie i muszą sobie poradzić z wysoką grawitacją w swojej turze.", -- Continental_supplies
    ["Sabotage/Flare: [Sabotage all hogs in the circle and deal ~1 dmg OR Fire a cluster up into the air]"] = "Sabotaż/Flara: [Sabotuje wszystkie jeże w kole i zadaje ~1 obrażeń LUB wystrzeliwuje w powietrze odłamek]", -- Continental_supplies
    ["Saint"] = "Święty", -- HedgeEditor, The_Specialists
    ["Salivaslurper"] = "Ślinosiorb", -- A_Classic_Fairytale:united
    ["Salty Dog"] = "Słony Pies",
    ["Salvation"] = "Odkupienie", -- A_Classic_Fairytale:family
    ["Salvation was one step closer now..."] = "Odkupienie było teraz o krok bliżej...", -- A_Classic_Fairytale:dragon
    ["Sam"] = "Sam", -- A_Space_Adventure:cosmos
    ["Sandals?! I thought you left your ring!"] = "Sandały?! Myślałem, że zostawiłeś swój pierścień!", -- A_Classic_Fairytale:queen
    ["%s and GB"] = "%s i GB.", -- A_Space_Adventure:fruit02
    ["%s and %s enter the battlefield"] = "%s i %s wchodzą na pole bitwy", -- A_Space_Adventure:fruit01
    ["Sandstorm"] = "Burza piaskowa", -- A_Space_Adventure:desert01
    ["Sandy"] = "Piaskowy", -- A_Space_Adventure:desert01
    ["%s arrived at the Desert Planet!"] = "%s dotarł na Pustynną Planetę!", -- A_Space_Adventure:cosmos
    ["%s arrived at the Fruit Planet!"] = "%s dotarł na Planetę Owoców!", -- A_Space_Adventure:cosmos
    ["%s arrived at the Ice Planet!"] = "%s dotarł na Lodową Planetę!", -- A_Space_Adventure:cosmos
    ["%s arrived at the meteorite!"] = "%s dotarł na meteoryt!", -- A_Space_Adventure:cosmos
    ["%s arrived at the moon!"] = "%s dotarł na księżyc!", -- A_Space_Adventure:cosmos
    ["%s arrived at the Planet of Death!"] = "%s dotarł na Planetę Śmierci!", -- A_Space_Adventure:cosmos
    ["Save as many hogs as possible!"] = "Uratuj tyle jeży, ile się da!", -- User_Mission_-_That_Sinking_Feeling
    ["Save Fell From Heaven!"] = "Uratuj Upadłego z Niebios!", -- A_Classic_Fairytale:journey
    ["Save Leaks A Lot!|Hint: The switch hedgehog utility might be of help to you."] = "Uratuj Sporego Przecieka!|Porada: narzędzie zmiany jeża może być dla ciebe pomocne.", -- A_Classic_Fairytale:shadow
    ["Save Level: [Precise]+[4]"] = "Zapisz poziom: [Precyzja]+[4]", -- HedgeEditor
    ["Save the princess! All your hogs must survive!|Hint: Kill the cyborgs first! Use the ammo very carefully!|Hint: You might want to spare a girder for cover!"] = "Uratuj księżniczkę! Wszystkie twoje jeże muszą przeżyć!|Porada: zabij najpierw cyborgi! Używaj amunicji bardzo ostrożnie!|Porada: możesz chcieć poświęcić belkę jako osłonę!", -- A_Classic_Fairytale:family
    ["Save the princess by collecting the crate in under 12 turns!"] = "Uratuj księżniczkę, poprzez zebranie skrzyni w mniej niż 12 tur!", -- A_Classic_Fairytale:journey
    ["Saving Hogera"] = "Ratowanie Jeżery", -- A_Space_Adventure:cosmos
    ["%s barely made it past the hogosphere."] = "%s ledwo przekroczył jeżosferę.", -- ClimbHome
    ["%s bravely climbed up to a dizzy height of %d to reach home."] = "%s odważnie wspiął się na zawrotną wysokość %d, by dotrzeć do domu.", -- ClimbHome
    ["Scallywag"] = "Łobuziak",
    ["Scalp Muncher"] = "Przeżuwacz Skalpów", -- A_Classic_Fairytale:backstab
    ["Scenario"] = "Scenariusz", -- Big_Armory, portal, User_Mission_-_Bamboo_Thicket, User_Mission_-_Dangerous_Ducklings, User_Mission_-_Diver, User_Mission_-_Newton_and_the_Hammock, User_Mission_-_Nobody_Laugh, User_Mission_-_Spooky_Tree, User_Mission_-_Teamwork_2, User_Mission_-_Teamwork, User_Mission_-_The_Great_Escape
    ["Scientist"] = "Naukowiec", -- Battalion
    ["%s climbed home in %d seconds!"] = "%s wspiął się do domu w %d sekund!", -- ClimbHome
    ["%s (contd.)"] = "%s (kont.)", -- A_Classic_Fairytale:epil
    ["Score: %d"] = "Wynik: %d", -- Space_Invasion
    ["Score goal: %d"] = "Wymagany wynik: %d", -- Control
    ["Score graph"] = "Graf wyniku", -- Mutant, Space_Invasion
    ["Score points by killing other hedgehogs (see below)."] = "Zdobądź punkty, zabijając inne jeże (zobacz niżej).", -- Mutant
    ["Score points by killing other hedgehogs."] = "Zdobądź punkty zabijając inne jeże.", -- Mutant
    ["Scores: "] = "Wyniki: ", -- Capture_the_Flag
    ["Scores"] = "Wyniki", -- Mutant
    ["Scores:"] = "Wyniki:", -- Mutant
    ["Scoring: "] = "Wynki: ", -- Mutant
    ["%s couldn't escape, try again!"] = "%s nie mógł uciec, spróbuj ponownie!", -- A_Space_Adventure:fruit01
    ["Script parameter examples:"] = "Przykłady parametrów skryptu:", -- Gravity
    ["%s: %d (deaths: %d)"] = "%s: %d (śmierci: %d)", -- Mutant
    ["%s (%d), %d sec"] = "%s (%d), %d sekund", -- Continental_supplies
    ["%s: Did not finish"] = "%s: Nie ukończył", -- Racer, TechRacer
    ["%s did not finish the race."] = "%s nie ukończył wyścigu", -- Racer, TechRacer
    ["%s didn't expect that."] = "%s nie spodziewał się tego", -- User_Mission_-_Rope_Knock_Challenge
    ["%s died … and lives again!"] = "%s umarł... i żyje ponownie!", -- Construction_Mode
    ["%s doesn’t really know how to handle a rope properly."] = "%s nie wie tak naprawdę jak posługiwać się liną.", -- ClimbHome
    ["%s (+%d)"] = "%s (+%d)", -- Battalion
    ["%s: %d"] = "%s: %d", -- Capture_the_Flag, Control
    ["%s (%d)"] = "%s (%d)", -- Continental_supplies
    ["%s, %d sec"] = "%s, %d sekund", -- Continental_supplies
    ["Search for the device with the help of the other hedgehogs."] = "Posukaj urządzenia z pomocą innych jeży.", -- A_Space_Adventure:fruit02
    ["Searching in the dust"] = "Poszukiwanie w kurzu", -- A_Space_Adventure:desert01
    ["Searching the stars!"] = "Szukanie gwiazd!", -- A_Space_Adventure:cosmos
    ["Seduction"] = "Uwodzenie", -- Continental_supplies
    ["Seems like every time you take a \"walk\", the enemy finds us!"] = "Wygląda na to, żeza każdym razem, gdy idziesz na \"spacer\", wróg nas znajduje!", -- A_Classic_Fairytale:backstab
    ["See that crate farther on the right?"] = "Widzisz tę skrzynię po prawej?", -- A_Classic_Fairytale:first_blood
    ["See ya!"] = "Do zobaczenia!",
    ["Segmentation Paul"] = "Naruszenie Pawła", -- A_Classic_Fairytale:dragon
    ["Select a placement mode and read the infos|in the mission panel to learn how to use it."] = "Wybierz tryb ustawiania i przeczytaj informacje|w panelu misji, by dowiedzieć się jak go używać.", -- HedgeEditor
    ["Select continent!"] = "Wybierz kontynent!", -- Continental_supplies
    ["Select continent"] = "Wybierz kontynent", -- Continental_supplies
    ["Selection Mode"] = "Tryb Wyboru", -- HedgeEditor
    ["Select, modify, or delete girders, rubbers and sprites"] = "Wybierz, modyfikuj lub usuń belki, gumy i grafiki", -- HedgeEditor
    ["Select/Place/Delete Gear: [Left Click]"] = "Wybierz/Ustaw/Usuń rzecz: [Lewy klik]", -- HedgeEditor
    ["Select, reposition and delete gears"] = "Wybierz, przestaw i usuń rzecz", -- HedgeEditor
    ["Select Rope"] = "Wybierz linę", -- Basic_Training_-_Rope
    ["Select “Switch Hedgehog” from the ammo menu and|hit the “Attack” key to proceed."] = "Wybierz “Zmień jeża” z menu amunicji i wciśnij “Atak”, by kontynuować", -- Basic_Training_-_Movement
    ["Select “Switch Hedgehog” from the ammo menu and|hit the “Attack” key."] = "Wybierz “Zmień jeża” z menu amunicji i wciśnij “Atak”.", -- Basic_Training_-_Movement
    ["Select the current continent."] = "Wybierz aktualny kontynent.", -- Continental_supplies
    ["Select the rope to begin!"] = "Wybierz linę, by zacząć!", -- Basic_Training_-_Rope
    ["Select this item for a random continent."] = "Wybierz ten przedmiot dla losowego kontynentu.", -- Continental_supplies
    ["Select weapon: [Left click]"] = "Wybierz broń: [Lewy klik]", -- Basic_Training_-_Bazooka, Basic_Training_-_Grenade
    ["Select Weapon"] = "Wybierz broń", -- Basic_Training_-_Bazooka, Basic_Training_-_Grenade
    ["Select win/lose condition: [Left], [Right]"] = "Wybierz warunki wygranej/przegranej: [Lewo], [Prawo]", -- HedgeEditor
    ["Select your continent/weaponset: with the \"Up\" or \"Down\" keys. You can also select one with the weapons menu."] = "Wybierz swój kontynent/zestaw broni używając klawiszy \"Góra\" lub \"Dół\". Możesz także wybrać je z menu broni.", -- Continental_supplies
    ["Select your continent/weaponset: With the \"Up\" or \"Down\" keys. You can also select one with the weapons menu."] = "Wybierz swój kontynent/zestaw broni wciskając klawisze \"Góra\" lub \"Dół\". Możesz także je wybrać używając menu broni.", -- Continental_supplies
    ["Select your continent with [Up]/[Down] or by selecting a representative weapon."] = "Wybierz swój kotnynent używając [Góra]/[Dół] lub wybierając reprezentatywną broń.", -- Continental_supplies
    ["%s enters the battlefield"] = "%s wchodzi na pole bitwy", -- A_Space_Adventure:fruit01
    ["Sergey"] = "Sergiusz",
    ["%s escaped successfully!"] = "%s uciekł pomyślnie!", -- A_Space_Adventure:fruit01
    ["Set bounciness: [Left Shift] + [1]-[5]"] = "Wybierz odbijalność: [Lewy Shift] + [1]-[5]", -- Basic_Training_-_Grenade
    ["Set detonation timer: [1]-[5]"] = "Wybierz czas detonacji: [1]-[5]", -- Basic_Training_-_Grenade
    ["Set Health: [Left Click]"] = "Ustaw zdrowie: [Lewy klik]", -- HedgeEditor
    ["Set Identity: [Left Click]"] = "Ustaw tożsamość: [Lewy klik]", -- HedgeEditor
    ["Set period to negative value for random gravity."] = "Ustaw okres na negatywną wartość dla losowej grawitacji.", -- Gravity
    ["Set the health of hogs, health crates, barrels and duds"] = "Ustaw zdrowie jeży, skrzyń ze zdrowiem, beczek i niewybuchów", -- HedgeEditor
    ["Set to %d"] = "Ustaw na %d", -- HedgeEditor
    ["%s exploded."] = "%s wybuchł.", -- User_Mission_-_Rope_Knock_Challenge
    ["%s fell from a high cliff."] = "%s upadł z wysokiego klifu.", -- User_Mission_-_Rope_Knock_Challenge
    ["%s fell too fast."] = "%s spadł za szybko.", -- User_Mission_-_Rope_Knock_Challenge
    ["%s fell victim to a weapon filter"] = "%s padł ofiarą filtra broni", -- Construction_Mode
    ["%s felt unstable."] = "%s poczuł się niestabilnie.", -- User_Mission_-_Rope_Knock_Challenge
    ["%s felt victim to rope-knocking."] = "%s stał się ofiarą uderzania liną.", -- User_Mission_-_Rope_Knock_Challenge
    ["%s flew like a rock."] = "%s poleciał jak kamień.", -- User_Mission_-_Rope_Knock_Challenge
    ["%s gets an extra life"] = "%s dostaje dodatkowe życie", -- Construction_Mode
    ["%s goes the way of the lemming."] = "%s poszedł drogą lemingów.", -- User_Mission_-_Rope_Knock_Challenge
    ["Sgt. Smith"] = "Sierżant Smith",
    ["%s had it coming."] = "%s czuł, że tak będzie.", -- User_Mission_-_Rope_Knock_Challenge
    ["%s had no chance."] = "%s nie miał szans.", -- User_Mission_-_Rope_Knock_Challenge
    ["... share your beauty with the world every morning, my princess!"] = "...dziel się swoją pięknością ze światem każdego ranka, moja księżniczko!", -- A_Classic_Fairytale:journey
    ["%s has been killed before taking enough damage first."] = "%s został zabity, zanim wcześniej dostał odpowiednią ilość obrażeń.", -- SimpleMission
    ["%s has been knocked out."] = "%s został zrzucony.", -- User_Mission_-_Rope_Knock_Challenge
    ["%s has been rescued from death"] = "%s został uratowany od śmierci", -- Construction_Mode
    ["%s has dropped the flag!"] = "%s upuścił flagę!", -- CTF_Blizzard
    ["%s has fallen victim to gravity."] = "%s stał się ofiarą grawitacji.", -- User_Mission_-_Rope_Knock_Challenge
    ["%s has mutated! +2 points"] = "%s zmutował! +2 punkty", -- Mutant
    ["%s has passed the best height of %s!"] = "%s przekroczył najlepszą wysokość %s!", -- ClimbHome
    ["%s has scored!"] = "%s zapunktował!", -- Capture_the_Flag
    ["%s has to refuel the saucer."] = "%s musi zatankować spodek.", -- A_Space_Adventure:moon01
    ["%s hates Newton."] = "%s nienawidzi Newtona.", -- User_Mission_-_Rope_Knock_Challenge
    ["She endangered the whole tribe!"] = "Ona naraziła całe plemię!", -- A_Classic_Fairytale:epil
    ["sheepluva"] = "sheepluva",
    ["Sheepy"] = "Owieczka",
    ["She's behind that tall thingy."] = "Ona jest za tym wysokim czymś.", -- A_Classic_Fairytale:family
    ["Shield boosted! +%d power"] = "Osłona wzmocniona! +%d mocy", -- Space_Invasion
    ["Shield depleted"] = "Osłona wyczerpana", -- Space_Invasion
    ["Shield is fully recharged!"] = "Osłona całkowicie naładowana!",
    ["Shield Master! +10 points!"] = "Mistrz osłony! +10 punktów", -- Space_Invasion
    ["Shield Miser! +%d points!"] = "Osłonowy skąpiec! +%d punktów!", -- Space_Invasion
    ["Shield OFF: %d power remaining"] = "Osłona wyłączona: pozostało %d mocy", -- Space_Invasion
    ["Shield ON: %d power remaining"] = "Osłona włączona: pozostało %d mocy", -- Space_Invasion
    ["Shield Seeker! +10 points!"] = "Poszukiwacz osłon! +10 punktów!", -- Space_Invasion
    ["Shinobi"] = "Shinobi",
    ["%s hit the ground."] = "%s uderza w ziemię", -- User_Mission_-_Rope_Knock_Challenge
    ["Shoppa Love"] = "Kochamy Shoppa", -- Challenge_-_Speed_Shoppa_-_Hedgelove
    ["Shotgun"] = "Strzelba", -- Continental_supplies
    ["Sigh."] = "Ech.", -- A_Classic_Fairytale:epil
    ["Silly"] = "Głuptas",
    ["Silver"] = "Srebro",
    ["Sine Gun"] = "Sinusoidalna giwera", -- Construction_Mode
    ["Sinky"] = "Śmierdziel",
    ["Sirius Lee"] = "Syriusz Li", -- A_Classic_Fairytale:enemy
    ["%s is dead, who was critical to this mission!"] = "%s nie żyje, a był krytyczny dla tej misji!", -- SimpleMission
    ["%s is eliminated!"] = "%s jest wyeliminowany!", -- User_Mission_-_Rope_Knock_Challenge
    ["%s is now as poor as a church mouse"] = "%s jest teraz tak biedny, jak mysz kościelna", -- Construction_Mode
    ["%s is now a zombie hedgehog"] = "%s jest teraz zombie jeżem", -- Construction_Mode
    ["%s is suddenly low on ammo"] = "%s nagle ma mało amunicji", -- Construction_Mode
    ["Skulls"] = "Czaszki", -- Bazooka_Battlefield
    ["Slimer"] = "Szlamiarz",
    ["Slippery"] = "Śliski", -- A_Classic_Fairytale:journey
    ["%s lost all the weapons"] = "%s stracił wszystkie bronie", -- Construction_Mode
    ["%s lost, try again!"] = "%s straconył, spróbuj ponownie!", -- A_Space_Adventure:death01, A_Space_Adventure:death02, A_Space_Adventure:desert01, A_Space_Adventure:desert02, A_Space_Adventure:desert03, A_Space_Adventure:final, A_Space_Adventure:fruit01, A_Space_Adventure:fruit02, A_Space_Adventure:fruit03, A_Space_Adventure:ice01, A_Space_Adventure:moon01
    ["Slot %d: %s"] = "Slot %d: %s", -- Frenzy
    ["Slot keys save time! (F1-F10 by default)"] = "Klawisze slotów oszczędzają czas! (domyślnie F1-F10)", -- Frenzy
    ["Slowpoke"] = "Slowpoke",
    ["%s made it past the hogosphere."] = "%s przekroczył jeżosferę.", -- ClimbHome
    ["%s managed to pass half of the distance towards home."] = "%s zdołał pokonać połowę dystansu do domu.", -- ClimbHome
    ["%s may choose the rules."] = "%s mogą wybrać zasady.", -- WxW
    ["Smith 0.97"] = "Smith 0.97", -- A_Classic_Fairytale:enemy
    ["Smith 0.98"] = "Smith 0.98", -- A_Classic_Fairytale:enemy
    ["Smith 0.99a"] = "Smith 0.99a", -- A_Classic_Fairytale:enemy
    ["Smith 0.99b"] = "Smith 0.99b", -- A_Classic_Fairytale:enemy
    ["Smith 0.99f"] = "Smith 0.99f", -- A_Classic_Fairytale:enemy
    ["Smith 1.0"] = "Smith 1.0", -- A_Classic_Fairytale:enemy
    ["Smugglers"] = "Przemytnicy", -- A_Space_Adventure:desert01
    ["%s must collect the final crates."] = "%s musi zebrać ostatnie skrzynie.", -- A_Space_Adventure:fruit02
    ["%s must skip this turn for rule violation."] = "%s musi opuścić swoją turę za naruszenie zasad.", -- WxW
    ["Sneaks"] = "Donosiciele", -- Bazooka_Battlefield
    ["%s never got the ninja diploma."] = "%s nigdy nie zdobył dyplomu ninja.", -- ClimbHome
    ["%s never wanted to reach for the sky in the first place."] = "%s wcale nie chciał dosięgnąć nieba.", -- ClimbHome
    ["Sniper! +8 points!"] = "Snajper! +8 punktów!", -- Space_Invasion
    ["Sniper Rifle"] = "Karabin snajperski", -- Continental_supplies
    ["Sniper"] = "Snajper", -- HedgeEditor, The_Specialists
    ["Sniper Training"] = "Trening snajperski",
    ["So, as promised I have brought you where I think that the device you are looking for is hidden."] = "Więc jak obiecałem, przyprowadziłem cię tam, gdzie myślę, że jest schowane urządzenie, którego szukasz.", -- A_Space_Adventure:fruit02
    ["So far, you had infinite ropes, but in the|real world, ropes are usually limited."] = "Dotąd miałeś nieskończone liny, ale w prawdziwym|świecie, liny są zazwyczaj ograniczone.", -- Basic_Training_-_Rope
    ["So humiliating..."] = "Tak upokarzające...", -- A_Classic_Fairytale:first_blood
    ["So, I believe that it's a good place to start."] = "A więc, wierzę, że to jest dobre miejsce, by zacząć.", -- A_Space_Adventure:desert01
    ["So, I kindly ask for your help."] = "Więc, grzecznie proszę o twoją pomoc.", -- A_Space_Adventure:fruit01
    ["So I shook my fist in the air!"] = "Więc potrząsłem moją pięścią w powietrzu!", -- A_Classic_Fairytale:epil
    ["Soldier"] = "Żołnierz", -- HedgeEditor, The_Specialists
    ["So, let me tell you what I know about Professor Hogevil."] = "A więc, pozwól mi powiedzieć, co wiem o Profesorze Jeżozło.", -- A_Space_Adventure:moon02
    ["Some parts of the land are indestructible."] = "Niektóre części lądu są niezniszczalne.", -- A_Space_Adventure:fruit03
    ["Some sick game of yours?!"] = "Jakaś twoja chora gra?!", -- A_Classic_Fairytale:queen
    ["Some weapons can be dropped from the rope."] = "Niektóre bronie mogą być upuszczone z liny.", -- Basic_Training_-_Rope
    ["Somewhere else on the planet of fruits, Captain Lime helps %s"] = "Gdzieś indziej na planecie owoców, Kapitan Limonka pomaga jeżowi %s", -- A_Space_Adventure:fruit02
    ["Somewhere else on the planet of fruits, %s gets closer to the device"] = "Gdzieś indziej na planecie owoców, %s zbliża się do urządzenia", -- A_Space_Adventure:fruit02
    ["Somewhere on the Planet of Fruits a terrible war is about to begin ..."] = "Gdzieś na Planecie Owoców, straszliwa wojna ma właśnie się rozpocząć...", -- A_Space_Adventure:fruit01
    ["Somewhere on the uninhabitable Death Planet ..."] = "Gdzieś na niezamieszkalnej Planecie Śmierci...", -- A_Space_Adventure:death01
    ["So, now I got the last part and I have your friends captured."] = "Więc, mam teraz ostatnią część i przetrzymuję twoich przyjaciół.", -- A_Space_Adventure:death01
    ["So, %s, here we are ..."] = "A więc, %s, dotarliśmy...", -- A_Space_Adventure:cosmos
    ["So the princess was never heard of again ..."] = "Więc o księżniczce nigdy więcej nie słyszano...", -- A_Classic_Fairytale:family
    ["So, uhmm, how did you manage to teleport them so far?"] = "Więc, um, jak ci się udało ich teleportować tak daleko?", -- A_Classic_Fairytale:epil
    ["Sour"] = "Surowy",
    ["South America"] = "Ameryka Południowa", -- Continental_supplies
    ["So? What will it be?"] = "Więc? Co to będzie?", -- A_Classic_Fairytale:shadow
    ["So you are able to launch projectiles into your aiming direction, always at full power."] = "Więc możesz wystrzelić pociski w kierunku celowania, zawsze z pełną mocą.", -- Basic_Training_-_Flying_Saucer
    ["So you are interested in Professor Hogevil, huh?"] = "Więc jesteś zainteresowany Profesorem Jeżozło, co?", -- A_Space_Adventure:moon02
    ["So you basically did the dirty work for us."] = "Więc w zaszadzie zrobiłeś dla nas całą brudną robotę.", -- A_Classic_Fairytale:dragon
    ["Space Invasion"] = "Kosmiczna inwazja", -- Space_Invasion
    ["SPACE INVASION"] = "KOSMICZNA INWAZJA", -- Space_Invasion
    ["Spacetrip"] = "Kosmiczna wycieczka", -- A_Space_Adventure:cosmos
    ["Spawn the crate and attack!"] = "Ustaw skrzynię i atakuj!", -- WxW
    ["Specials: Kings and air generals drop helpers, not weapons"] = "Specjalne: Królowie i powietrzni generałowie upuszczają pomocników, nie bronie", -- Battalion
    ["Special weapons:"] = "Bronie specjalne:", -- Continental_supplies
    ["Special Weapons:"] = "Specjalne bronie:", -- Continental_supplies
    ["Speckles"] = "Spekles",
    ["Specs Appeal"] = "Urok Speców",
    ["Specs"] = "Spece",
    ["Spectator"] = "Widz",
    ["Speed Roping"] = "Szybkie skakanie na linie", -- Basic_Training_-_Rope
    ["Speed Shoppa"] = "Szybka Shoppa", -- SpeedShoppa
    ["Spike"] = "Kolec", -- A_Space_Adventure:desert01
    ["Spikes"] = "Kolce",
    ["Spiky Cheese"] = "Kolczasty Ser", -- A_Classic_Fairytale:enemy, A_Classic_Fairytale:epil, A_Classic_Fairytale:family, A_Classic_Fairytale:queen, A_Classic_Fairytale:shadow
    ["%s, place the first hedgehog!"] = "%s, ustaw pierwszego jeża!", -- WxW
    ["Spleenlover"] = "Wielbiciel Śledzion", -- A_Classic_Fairytale:united
    ["Sponge"] = "Gąbka",
    ["Spooky Tree"] = "Straszne drzewo",
    ["Sprite Erasure Mode"] = "Tryb Usuwania Grafik", -- HedgeEditor
    ["Sprite Modification Mode"] = "Tryb Modyfikacji Grafik", -- HedgeEditor
    ["SPRITE MODIFICATION MODE"] = "TRYB MODYFIKACJI GRAFIK", -- HedgeEditor
    ["Sprite Placement Mode"] = "Tryb Stawiania Grafik", -- Construction_Mode
    ["SPRITE PLACEMENT MODE"] = "TRYB STAWIANIA GRAFIK", -- HedgeEditor
    ["Sprite Testing Mode"] = "Tryb Testowania Grafik", -- Construction_Mode
    ["Squirtle"] = "Squirtle",
    ["Squishy"] = "Grząski",
    ["%s reached home in %.3f seconds. Congratulations!"] = "%s dotarł do domu w %.3f sekund. Gratulacje!", -- ClimbHome
    ["%s (%s) destroyed %d invaders in one round."] = "%s (%s) zniszczył %d najeźdźców w jednej rundzie.", -- Space_Invasion
    ["%s (%s) does not have to feel ashamed for their best height of %d."] = "%s (%s) nie musi się wstydzić swojej najlepszej wysokości %d.", -- ClimbHome
    ["%s, select your continent!"] = "%s, wybierz swój kontynent!", -- Continental_supplies
    ["%s (%s) gave short shrift to the invaders: Longest combo of %d!"] = "%s (%s) dał najeźdźcom krótką pokutę: Najdłuższe kombo %d!", -- Space_Invasion
    ["%s (%s) has been invited to join the Planetary Association of the Hedgehogs, it destroyed a staggering %d invaders in just one round!"] = "%s (%s) został zaproszony do dołączenia do Planetarnego Stowarzyszenia Jeży, zniszczył zdumiewające %d najeźdźców w jednej rundzie!", -- Space_Invasion
    ["%s (%s) has captured the flag %d times."] = "%s (%s) przejął flagę %d razy.", -- Capture_the_Flag
    ["%s (%s) hate life and suicided %d times."] = "%s (%s) nienawidzi życia i zabił się %d razy.", -- Mutant
    ["%s should try the rope training mission first."] = "%s powinien najpierw spróbować treningu linowego.", -- ClimbHome
    ["%s (%s) is addicted to killing: %d invaders destroyed in one round."] = "%s (%s) jest uzależniony od zabijania: %d najeźdźców zniszczonych w jednej rundzie.", -- Space_Invasion
    ["%s (%s) is a hardened hunter: No misses and %d hits in its best round!"] = "%s (%s) jest zahartowanym łowcą: Bez pudła i z %d uderzeniami w swojej najlepszej rundzie!", -- Space_Invasion
    ["%s (%s) is a tumbleweed: %d points in one round."] = "%s (%s) jest biegaczem: %d punktów w jednej rundzie.", -- Space_Invasion
    ["%s (%s) is good at this: %d points in only one round!"] = "%s (%s) jest w tym dobry: %d punktów w tylko jednej rundzie!", -- Space_Invasion
    ["%s (%s) is Rambo in a hedgehog costume! He destroyed %d invaders in one round."] = "%s (%s) jest Rambo w kostiumie jeża! Zniszczył %d najeźdźcóww jednej rundzie.", -- Space_Invasion
    ["%s skipped ninja classes."] = "%s wagarował na zajęciach ninja.", -- ClimbHome
    ["%s spawned at a really bad position."] = "%s pojawił się na naprawdę złej pozycji.", -- User_Mission_-_Rope_Knock_Challenge
    ["%s splatted."] = "%s plasnął.", -- User_Mission_-_Rope_Knock_Challenge
    ["%s (%s) reached a decent peak height of %d."] = "%s (%s) osiągnął przyzwoitą szczytową wysokość %d.", -- ClimbHome
    ["%s (%s) reached a peak height of %d."] = "%s (%s) osiągnął szczytową wysokość %d.", -- ClimbHome
    ["%s (%s) reached for the sky and beyond with a height of %d!"] = "%s (%s) dosięgnął nieba i dalej, z wysokością %d!", -- ClimbHome
    ["%s (%s) reached home in %.3f seconds."] = "%s (%s) dotarł do domu w %.3f sekund.", -- ClimbHome
    ["%s (%s) shot %d invaders and never missed in the best round!"] = "%s (%s) zestrzelił %d najeźdźców i nigdy nie spudłował w najlepszej rundzie!", -- Space_Invasion
    ["%s: %s"] = "%s: %s", -- Continental_supplies
    ["%s (%s) struck like a meteor: %d points in only one round!"] = "%s (%s) uderzył jak meteor: %d punktów w tylko jednej rundzie!", -- Space_Invasion
    ["%s still had a long way to go."] = "%s nadal miał daleką drogę do przejścia.", -- ClimbHome
    ["%s stumbled."] = "%s potknął się.", -- User_Mission_-_Rope_Knock_Challenge
    ["%s (%s) tumbles like no other: %d points in one round."] = "%s (%s) toczy się jak nikt inny: %d punktów w jednej rundzie.", -- Space_Invasion
    ["%s stumpled."] = "%s potknął się.", -- User_Mission_-_Rope_Knock_Challenge
    ["%s (%s) was certainly not afraid of heights: Peak height of %d!"] = "%s (%s) z pewnością nie bał się wysokości: Szczytowa wysokość %d!", -- ClimbHome
    ["%s (%s) was lightning-fast! Longest combo of %d, absolutely insane!"] = "%s (%s) był szybki jak błyskawica! Najdłuższe kombo %d, absolutnie szalone!", -- Space_Invasion
    ["%s (%s) was on fire: Longest combo of %d."] = "%s (%s) palił się: Najdłuższe kombo %d.", -- Space_Invasion
    ["%s (%s) was panicly afraid of the water and decided to get in a safe distance of %d from it."] = "%s (%s) panicznie bał się wody i zdecydował się dostać na bezpieczną odległość %d od niej.", -- ClimbHome
    ["%s (%s) was the best baby tumbler: %d points in one round."] = "%s (%s) był najlepszym dziecięcym akrobatą: %d punktów w jednej rundzie.", -- Space_Invasion
    ["%s (%s) was the greediest hedgehog and collected %d crates."] = "%s (%s) był najchciwszym jeżem i zebrał %d skrzyń.", -- Mutant
    ["%s (%s) was undoubtedly the very best professional tumbler in this game: %d points in one round!"] = "%s (%s) był niewątpliwie najlepszym profesjonalnym akrobatą w tej grze: %d punktów  w jednej rundzie.", -- Space_Invasion
    ["Star"] = "Gwiazda", -- Big_Armory
    ["Status update"] = "Aktualizacja statusu", -- Racer, TechRacer
    ["Status Update"] = "Aktualizacja statusu", -- Space_Invasion
    ["Stay away from our weapons!"] = "Trzymaj się z dala od naszych broni!", -- A_Classic_Fairytale:queen
    ["Stay there, comrades!"] = "Zostańcie tam, towarzysze!", -- A_Classic_Fairytale:queen
    ["Stay there to flee!"] = "Zostań tu, by uciec!", -- A_Space_Adventure:fruit01
    ["Steel Eye"] = "Stalowooki", -- A_Classic_Fairytale:queen
    ["Step 1: Activate your flying saucer but do NOT move yet!"] = "Krok 1: Aktywuj swój latający talerz, ale NIE ruszaj się jeszcze!", -- Basic_Training_-_Flying_Saucer
    ["Step 1: Start roping"] = "Krok 1: Zacznij używać liny", -- Basic_Training_-_Rope
    ["Step 2: Select grenade"] = "Krok 2: Wybierz granat", -- Basic_Training_-_Rope
    ["Step 2: Select your grenade."] = "Krok 2: Wybierz swój granat.", -- Basic_Training_-_Flying_Saucer
    ["Step 3: Drop the grenade"] = "Krok 3: Upuść granat", -- Basic_Training_-_Rope
    ["Step 3: Start flying and get yourself right above the target."] = "Krok 3: Zacznij lecieć i ustaw się tuż nad swoim celem.", -- Basic_Training_-_Flying_Saucer
    ["Step 4: Drop your grenade by pressing the [Long jump] key."] = "Krok 4: Upuść swój granat wciskając przycisk [Długi skok].", -- Basic_Training_-_Flying_Saucer
    ["Step 5: Get away quickly and land safely anywhere."] = "Krok 5: Szybko odleć i bezpiecznie wyląduj gdziekolwiek.", -- Basic_Training_-_Flying_Saucer
    ["Step By Step"] = "Krok po kroku", -- A_Classic_Fairytale:first_blood
    ["Steve"] = "Stefan", -- A_Classic_Fairytale:dragon, A_Classic_Fairytale:family, A_Classic_Fairytale:queen
    ["Sticky Mine"] = "Mina samoprzylepna", -- Continental_supplies
    ["Sticky Mine Placement Mode"] = "Tryb Stawiania Min Samoprzylepnych", -- Construction_Mode
    ["STICKY MINE PLACEMENT MODE"] = "TRYB STAWIANIA MIN SAMOPRZYLEPNYCH", -- HedgeEditor
    ["Stop, comrades!"] = "Stójcie, towarzysze!", -- A_Classic_Fairytale:queen
    ["Stop right there, puny worms!"] = "Stójcie tam, mizerne robaki!", -- A_Classic_Fairytale:queen
    ["Street Fighters"] = "Uliczni Wojownicy",
    ["Strength: %d (multiplier for ammo)"] = "Siła: %d (wielokrotność amunicji)", -- Battalion
    ["Strong knockback, but no poison."] = "Silny odrzut, ale bez trucizny.", -- Continental_supplies
    ["Stronglings"] = "Silniacy", -- A_Classic_Fairytale:shadow
    ["Structure Placement Mode"] = "Tryb Ustawiania Struktur", -- Construction_Mode
    ["Structure Placer"] = "Stawiacz Struktur", -- Construction_Mode
    ["Stupid, stupid Hogerians!"] = "Głupi, głupi Jeżarianie!", -- A_Space_Adventure:final
    ["Subtract %d"] = "Odejmij %d", -- HedgeEditor
    ["--- Sudden Death ---"] = "--- Nagła Śmierć ---", -- Battalion
    ["Summer Squash"] = "Kabaczek", -- A_Space_Adventure:fruit01
    ["Sundaland"] = "Sundaland", -- Continental_supplies
    ["Sunflame"] = "Słoneczny Płomień", -- Big_Armory
    ["Super weapons: A few crates contain very powerful weapons."] = "Super bronie: Parę skrzyń zawiera bardzo potężne bronie.", -- WxW
    ["Super weapons: %s"] = "Super bronie: %s", -- WxW
    ["Supplies: Each continent gives you unique weapons, specials and health."] = "Posiłki: Każdy kontynent daje ci unikalne bronie, speciały i zdrowie.", -- Continental_supplies
    ["Support Station: Allows placement of crates."] = "Stacja Wsparcia: Pozwala na ustawianie skrzyń.", -- Construction_Mode
    ["Support Station"] = "Stacja Wsparcia", -- Construction_Mode
    ["Sure!"] = "Jasne!", -- A_Classic_Fairytale:epil
    ["Surf Before Crate: %s"] = "Serfuj przed skrzynią: %s", -- WxW
    ["Surf Before Crate: You must bounce off the water once before you can get crates."] = "Serfuj przed skrzynią: Musisz raz się odbić od wody, zanim będziesz mógł zdobyć skrzynie.", -- WxW
    ["Surfer! +15 points!"] = "Surfer! +15 punktów!", -- Space_Invasion
    ["Surfer!"] = "Surfer!", -- WxW
    ["Surprise supplies: Get 1-3 random weapons each turn."] = "Niespodziewane posiłki: Dostań 1-3 losowych broni w każdej turze.", -- Continental_supplies
    ["Survive!"] = "Przeżyj!", -- A_Classic_Fairytale:shadow
    ["%s violated the “All But Last” rule and will be penalized."] = "%s narusza zasadę \"Wszyscy Oprócz Ostatniego\" i będzie ukarany.", -- WxW
    ["%s violated the “Kill The Leader” rule and will be penalized."] = "%s narusza zasadę \"Zabij przywódcę\" i będzie ukarany.", -- WxW
    ["Swap place with a random enemy in the circle."] = "Zamień miejsce z losowym wrogiem w kole.", -- Continental_supplies
    ["%s was a good target."] = "%s był dobrym celem.", -- User_Mission_-_Rope_Knock_Challenge
    ["%s was close to home."] = "%s był blisko domu.", -- ClimbHome
    ["%s was damn close to home."] = "%s był piekielnie blisko domu.", -- ClimbHome
    ["%s was doomed from the beginning."] = "%s był na to skazany od początku.", -- User_Mission_-_Rope_Knock_Challenge
    ["%s was extracted from the scheme"] = "%s został wydobyty ze schematu", -- Continental_supplies
    ["%s was good, but not good enough."] = "%s był dobry, ale nie wystarczająco.", -- ClimbHome
    ["%s was knocked away."] = "%s został zrzucony.", -- User_Mission_-_Rope_Knock_Challenge
    ["%s was really unlucky."] = "%s był prawdziwym pechowcem.", -- User_Mission_-_Rope_Knock_Challenge
    ["%s was shoved away."] = "%s został zmieciony.", -- User_Mission_-_Rope_Knock_Challenge
    ["%s was smashed."] = "%s został zmiażdżony.", -- User_Mission_-_Rope_Knock_Challenge
    ["Sweet"] = "Słodki",
    ["%s went over a quarter of the way towards home."] = "%s przeszedł ćwierć drogi w stronę domu.", -- ClimbHome
    ["%s! Why?!"] = "%s! Czemu?!", -- A_Classic_Fairytale:backstab, A_Classic_Fairytale:united
    ["Swing, Leaks A Lot, on the wings of the wind!"] = "Huśtaj się, Spory Przecieku, na skrzydłach wiatru!", -- A_Classic_Fairytale:first_blood
    ["Swing: [Left]/[Right]"] = "Huśtaj się: [Lewo]/[Prawo]", -- Basic_Training_-_Rope
    ["%s wins, congratulations!"] = "%s wygrywa, gratulacje!", -- A_Space_Adventure:moon01
    ["%s wins!"] = "%s wygrywa!", -- Racer, Space_Invasion, TechRacer, ClimbHome
    ["%s wins with a best time of %.1fs."] = "%s wygrywa z najlepszym czasem %.1fs.", -- Racer, TechRacer
    ["Switch: Drop ball of dirt from parachute (once)"] = "Zmiana: Upuść kulę ziemi ze spadochronu (raz)", -- Continental_supplies
    ["Switched to "] = "Przełączono na ",
    ["Switch Hedgehog (1/3)"] = "Zmiana jeża (1/3)", -- Basic_Training_-_Movement
    ["Switch Hedgehog (2/3)"] = "Zmiana jeża (2/3)", -- Basic_Training_-_Movement
    ["Switch Hedgehog (3/3)"] = "Zmiana jeża (3/3)", -- Basic_Training_-_Movement
    ["Switch Hedgehog (Failed!)"] = "Zmiana jeża (Porażka!)", -- Basic_Training_-_Movement
    ["Switch hedgehog: [Tabulator]"] = "Zmień jeża: [Tab]", -- Basic_Training_-_Movement
    ["Switch Hog"] = "Zmień Jeża", -- Construction_Mode
    ["switch"] = "przełącz", -- Continental_supplies
    ["Switch: Select weapon special"] = "Zmiana: Wybierz speciał broni", -- Continental_supplies
    ["Switch: Toggle crate radar"] = "Zmiana: Przełącz radar skrzyń", -- WxW
    ["%s won!"] = "%s wygrywa!", -- A_Space_Adventure:fruit01
    ["Swords"] = "Miecze", -- Bazooka_Battlefield
    ["Syntax Errol"] = "Błąt Składni", -- A_Classic_Fairytale:dragon
    ["%s, you may choose the rules."] = "%s, możesz wybrać zasady.", -- WxW
    ["szczur"] = "szczur",
    ["Tackleberry"] = "Tackleberry",
    ["Tails"] = "Reszka",
    ["Talk about mixed signals..."] = "Mów o pomieszanych sygnałach...", -- A_Classic_Fairytale:dragon
    ["Tall Potato"] = "Wysoki Ziemniak", -- A_Space_Adventure:fruit01
    ["Tap [Pause] to see the mission texts"] = "Wciśnij [Pauza], by zobaczyć tekst misji", -- Basic_Training_-_Movement
    ["Tap the “rotating arrow” button on the left|until you have selected Cappy, the hedgehog with the cap!"] = "Wciśnij przycisk \"obracającej się strzałki\" po lewej|dopóki nie wybrałeś Cappiego, jeża z czapką!", -- Basic_Training_-_Movement
    ["Target"] = "Cel", -- HedgeEditor
    ["Target Placement Mode"] = "Tryb Stawiania Celów", -- Construction_Mode
    ["TARGET PLACEMENT MODE"] = "TRYB STAWIANIA CELÓW", -- HedgeEditor
    ["Target Practice: Bazooka (easy)"] = "Ćwiczenie celowania: Bazooka (łatwy)", -- Target_Practice_-_Bazooka_easy
    ["Target Practice: Bazooka (hard)"] = "Ćwiczenie celowania: Bazooka (trudny)", -- Target_Practice_-_Bazooka_hard
    ["Target Practice: Grenade (easy)"] = "Ćwiczenie celowania: Granat (łatwy)", -- Target_Practice_-_Grenade_easy
    ["Target Practice: Grenade (hard)"] = "Ćwiczenie celowania: Granat (trudny)", -- Target_Practice_-_Grenade_hard
    ["Target Practice: Homing Bee"] = "Ćwiczenie celowania: Pszczoła", -- Target_Practice_-_Homing_Bee
    ["Target Practice: Shotgun"] = "Ćwiczenie celowania: Strzelba", -- Target_Practice_-_Shotgun
    ["Target Puncher"] = "Uderzacz celów", -- Basic_Training_-_Rope
    ["Targets left: %d"] = "Pozostałe cele: %d", -- TargetPractice
    ["Tatsujin"] = "Tatsudżin",
    ["Tatters"] = "Szmaciarze",
    ["Team %d: "] = "Drużyna %d: ",
    ["Team %d"] = "Drużyna %d", -- SimpleMission
    ["Team highscore: %d"] = "Najlepszy wynik drużyny: %d", -- Utils
    ["Team Identity Mode"] = "Tryb Tożsamości Drużyny", -- HedgeEditor
    ["TEAM IDENTITY MODE"] = "TRYB TOŻSAMOŚCI DRUŻYNY", -- HedgeEditor
    ["Team lowscore: %d"] = "Najniższy wynik drużyny: %d", -- Utils
    ["Teams are tied! Continue playing rounds until we have a winner!"] = "Drużyny remisują! Grajcie rundy, aż nie wyłonimy zwycięzcy!", -- Space_Invasion
    ["Team's best time: %.3fs"] = "Najlepszy czas drużyny: %.3fs", -- Utils
    ["Team Scores:"] = "Wyniki drużyn:", -- Control
    ["Team scores:"] = "Wyniki drużyn:", -- Space_Invasion
    ["Team's longest time: %.3fs"] = "Najdłuższy czas drużyny: %.3fs", -- Utils
    ["Team's top accuracy: %d%"] = "Najlepsza celność drużyny: %d%", -- Utils
    ["Teamwork 2"] = "Praca zespołowa 2", -- User_Mission_-_Teamwork_2
    ["Teamwork"] = "Praca zespołowa", -- User_Mission_-_Teamwork
    ["TechRacer"] = "TechWyścig", -- TechRacer
    ["Teleporation Node"] = "Węzeł Teleportacyjny", -- Construction_Mode
    ["Teleportation Mode"] = "Tryb Teleportacji", -- Construction_Mode
    ["Teleportation Node: Allows teleportation|    between other nodes."] = "Węzeł Teleportacyjny: Pozwala na teleportację|    pomiędzy innymi węzłami.", -- Construction_Mode
    ["Teleportation Node"] = "Węzeł Teleportacyjny", -- Construction_Mode
    ["Teleport hint: just use the mouse to select the destination!"] = "Porada do teleportu: po prostu użyj myszki, by wybrać cel teleportacji!", -- A_Classic_Fairytale:dragon
    ["Teleport hint: Just use the mouse to select the destination!"] = "Wskazówka do teleportu: Uzyj po prostu myszki, by wybrać cel!", -- A_Classic_Fairytale:dragon
    ["Teleport"] = "Teleport", -- Construction_Mode, Frenzy
    ["Teleport to the impact location."] = "Teleportuj się do miejsca uderzenia.", -- Continental_supplies
    ["Teleport to the top of the map, expect fall damage!"] = "Teleportuj się na szczyt mapy, spodziewaj się obrażeń od upadku!", -- Continental_supplies
    ["Teleport unsuccessful. Please teleport within a clan teleporter's sphere of influence."] = "Teleportacja nieudana. Teleportuj się wewnątrz sfery wpływu teleportu klanowego.", -- Construction_Mode
    ["Teleport Unsuccessful. Please teleport within a clan teleporter's sphere of influence."] = "Teleport Nieudany. Proszę teleportować się wewnątrz sfery wpłyu klanowego teleportu.", -- Construction_Mode
    ["Tentacle Terror"] = "Mackowy terror", -- Tentacle_Terror
    ["Textile industry: Will give you a parachute every second turn."] = "Przemysł włókienniczy: Da ci spadochron co drugą turę.", -- Continental_supplies
    ["Thanks, dude! It really means a lot to me."] = "Dzięki, koleś! To naprawdę wiele dla mne znaczy.", -- A_Classic_Fairytale:epil
    ["Thanks!"] = "Dzięki!", -- A_Classic_Fairytale:family
    ["Thanks, man! It really means a lot to me."] = "Dzięki, ziom! To naprawdę wiele dla mne znaczy.", -- A_Classic_Fairytale:epil
    ["Thank you, Dr. Cornelius."] = "Dziękuję, Doktorze Korneliuszu.", -- A_Space_Adventure:cosmos
    ["Thank you for meeting me on such a short notice!"] = "Dziękuję za spotkanie ze mną w tak krótkim czasie!", -- A_Space_Adventure:desert01
    ["Thank you, my hero!"] = "Dziękuję, mój bohaterze!", -- A_Classic_Fairytale:family
    ["Thank you, oh, thank you, Leaks A Lot!"] = "Dziękuję, och, dziękuję, Spory Przecieku!", -- A_Classic_Fairytale:journey
    ["Thank you, oh, thank you, my heroes!"] = "Dziękuję, och, dziękuję, moi bohaterzy!", -- A_Classic_Fairytale:journey
    ["Thanta"] = "Tanta", -- A_Space_Adventure:ice01
    ["That is, indeed, very weird..."] = "To jest, w rzeczy samej, bardzo dziwne...", -- A_Classic_Fairytale:united
    ["That makes it almost invaluable!"] = "To czyni to prawie bezwartościowym!", -- A_Classic_Fairytale:enemy
    ["That ought to show them!"] = "To powinno im pokazać!", -- A_Classic_Fairytale:backstab
    ["That's all, folks!"] = "To wszystko, ludzie!", -- A_Classic_Fairytale:epil
    ["That's for my father!"] = "To za mojego ojca!", -- A_Classic_Fairytale:backstab
    ["That shaman sure knows what he's doing!"] = "Ten szaman z pewnością wie co robi!", -- A_Classic_Fairytale:shadow
    ["That Sinking Feeling"] = "Cisza na morzu",
    ["That's just the way it works, you know."] = "To tak po prostu działa, no wiesz.", -- A_Classic_Fairytale:queen
    ["That's not our problem!"] = "To nie nasz problem!", -- A_Classic_Fairytale:enemy
    ["That's typical of you!"] = "To takie typowe dla ciebie!", -- A_Classic_Fairytale:family
    ["That's why he always wears a hat since then."] = "To dlatego od tego czasu on zawsze nosi kapelusz.", -- A_Space_Adventure:moon02
    ["That traitor won't be killing us anymore!"] = "Ten zdrajca nie będzie nas już więcej zabijał!", -- A_Classic_Fairytale:queen
    ["That was just mean!"] = "To było po prostu wredne!", -- A_Classic_Fairytale:backstab, A_Classic_Fairytale:united
    ["That was pointless. The flag will respawn next round."] = "To było bezcelowe. Flaga odnowi się w następnej rundzie.", -- CTF_Blizzard
    ["The adventure begins!"] = "Przygoda się zaczyna!", -- A_Space_Adventure:cosmos
    ["The air bombs are weaker than usual."] = "Latające miny są słabsze niż zazwyczaj.", -- Battalion
    ["The aliens respect me, even worship me!"] = "Obcy mnie respektują, nawet mnie czczą!", -- A_Classic_Fairytale:queen
    ["The ally units share their ammo."] = "Sprzymierzone jednostki dzielą swoją amunicję.", -- A_Space_Adventure:fruit01
    ["The ammo of %s has been vaporized"] = "Amunicja %s została odparowana", -- Construction_Mode
    ["The answer is ... entertainment. You'll see what I mean."] = "Odpowiedź to... rozrywka. Zobaczysz, co mam na myśli.", -- A_Classic_Fairytale:backstab
    ["The answer is...entertaintment. You'll see what I mean."] = "Odpowiedź to... rozrywka. Zobaczycie co mam na myśli.", -- A_Classic_Fairytale:backstab
    ["The anti-portal surface is all over the floor, and I have nothing to kill him. Dropping something could hurt him enough to kill him."] = "Antyportalowa powierzchnia jest na całej podłodze, a ja nie mam nic, by go zabić. Zrzucenie czegoś mogłoby go skrzywdzić wystarczająco, by go zabić.", -- portal
    ["The big bang"] = "Wielki wybuch", -- A_Space_Adventure:final
    ["The boss has fallen! Retreat!"] = "Szef upadł! Odwrót!", -- A_Space_Adventure:moon01
    ["The Boss"] = "Szef",
    ["The Bull's Eye"] = "Bycze Oko", -- A_Classic_Fairytale:first_blood
    ["The caves are well hidden, they won't find us there!"] = "Jaskinie są dobrze ukryte, nie znajdą nas tutaj!", -- A_Classic_Fairytale:united
    ["The clan of the Red Strawberry wants to take over the dominion and overthrow King Pineapple."] = "Klan Czerwonych Truskawek chce przejąć zwierzchnictwo i obalić Króla Ananasa.", -- A_Space_Adventure:fruit01
    ["The continent of cowards"] = "Kontynent tchórzy", -- Continental_supplies
    ["The continent of dust"] = "Kontynent pyłu", -- Continental_supplies
    ["The continent of firearms"] = "Kontynent broni palnej", -- Continental_supplies
    ["The continent of greed"] = "Kontynent chciwości", -- Continental_supplies
    ["The continent of guerilla tactics"] = "Kontynent taktyk partyzanckich", -- Continental_supplies
    ["The continent of ice and science"] = "Kontynent lodu i nauki", -- Continental_supplies
    ["The continent of medicine"] = "Kontynent lekarstw", -- Continental_supplies
    ["The continent of ninjas"] = "Kontynent ninja", -- Continental_supplies
    ["The continent of sports"] = "Kontynent sportu", -- Continental_supplies
    ["The Crate Frenzy"] = "Szał skrzyń", -- A_Classic_Fairytale:first_blood
    ["The Customer is King"] = "Król jest klientem", -- Challenge_-_Speed_Shoppa_-_ShoppaKing
    ["The device part has been stolen!"] = "Część urządzenia została skradziona!", -- A_Space_Adventure:fruit02
    ["The device part is hidden in one of the crates! Go and get it!"] = "Część urządzenia jest ukryta w jednej ze skrzyń! Idź ją zdobyć!", -- A_Space_Adventure:desert01
    ["The Devs"] = "Deweloperzy",
    ["The Dilemma"] = "Dylemat", -- A_Classic_Fairytale:shadow
    ["The editor weapons and tools have been added!"] = "Edytorowe bronie i narzędzia zostały dodane!", -- HedgeEditor
    ["The editor weapons and tools have been removed!"] = "Edytorowe bronie i narzędzia zostały usunięte!", -- HedgeEditor
    ["The enemies aren't many anyway, it is going to be easy!"] = "Nie ma zbyt wielu wrogów tak w ogóle, to będzie łatwe!", -- A_Space_Adventure:fruit01
    ["The enemy can't move but it might be a good idea to stay out of sight!"] = "Ten wróg nie może się ruszać, ale może być dobrym pomysłem pozostać poza widokiem!", -- A_Classic_Fairytale:dragon
    ["The enemy has taken a crate which we really needed!"] = "Wróg zabrał skrzynię, której naprawdę potrzebowaliśmy!", -- SimpleMission
    ["The enemy hogs play in a random order."] = "Wrogie jeże grają w losowej kolejności.", -- A_Space_Adventure:death02
    ["The enemy is hiding out on yonder ducky!"] = "Na tamtej kaczce ukrywa się nasz przeciwnik!",
    ["The Enemy Of My Enemy"] = "Wróg mojego wroga", -- A_Classic_Fairytale:enemy
    ["The explosion is weaker than usual."] = "Eksplozja jest słabsza niż zazwyczaj.", -- Battalion
    ["The fastest hedgehog was %s from %s with a time of %.3fs."] = "Najszybszym jeżem był %s z drużyny %s z czasem %.3fs.", -- TrophyRace
    ["The fight begins!"] = "Walka się zaczyna!", -- A_Space_Adventure:moon01
    ["The final part"] = "Ostatnia część", -- A_Space_Adventure:death01
    ["The final targets are quite tricky. You need to aim well."] = "Ostatnie cele są dość podchwytliwe. Potrzebujesz dobrze celować.", -- Basic_Training_-_Bazooka
    ["The First Blood"] = "Pierwsza krew", -- A_Classic_Fairytale:first_blood
    ["The First Encounter"] = "Pierwsze starcie", -- A_Classic_Fairytale:shadow
    ["The first hedgehog to kill someone becomes the Mutant."] = "Pierwszy jeż, który kogoś zabije, stanie się Mutantem.", -- Mutant
    ["The first hedgehog which scores %d or more wins the game."] = "Pierwszy jeż, który zdobędzie %d lub więcej punktów, wygrywa mecz.", -- Mutant
    ["The first stop"] = "Pierwszy przystanek", -- A_Space_Adventure:moon01
    ["The first turn will last 25 sec and every other turn 15 sec."] = "Pierwsza tura potrwa 25 sekund, a każda inna 15 sek.", -- A_Space_Adventure:fruit03
    ["The flag will respawn next round."] = "Flaga pojawi się ponownie przy następnej rundzie.",
    ["The flood has stopped! Challenge over."] = "Powódź się zatrzymała! Wyzwanie skończone.", -- User_Mission_-_That_Sinking_Feeling
    ["The food bites back"] = "Jedzenie się odgryza", -- A_Classic_Fairytale:backstab
    ["The forgotten continent"] = "Zapomniany kontynent", -- Continental_supplies
    ["The giant umbrella from the last crate should help break the fall."] = "Ogromna parasolka z ostatniej skrzyni powinna pomóc pokonać upadek.", -- A_Classic_Fairytale:first_blood
    ["The Great Escape"] = "Wielka ucieczka", -- User_Mission_-_The_Great_Escape
    ["- The green target must survive"] = "- Zielony cel musi przeżyć", -- HedgeEditor
    ["- The green targets must survive"] = "- Zielone cele muszą przeżyć", -- HedgeEditor
    ["The guardian"] = "Stróż", -- A_Classic_Fairytale:shadow
    ["The hardships of the war turned %s (%s) into a killing machine: %d invaders destroyed in one round!"] = "Trudy wojny zamieniły %s (%s) w maszynę do zabijania: %d najeźdźców zniszczonych w jednej rundzie!", -- Space_Invasion
    ["The health of your current hedgehog|is shown at the top right corner."] = "Zdrowie twojego aktualnego jeża|jest widoczne w prawym-górnym rogu.", -- Basic_Training_-_Movement
    ["The hedgehog with least points (or most deaths) becomes the Bottom Feeder."] = "Jeż z najmniejszą ilością punktów (lub największą śmierci) stanie się Pasożytem.", -- Mutant
    ["The Hospital"] = "Szpital",
    ["The Individualist"] = "Indywidualista", -- A_Classic_Fairytale:shadow
    ["Their buildings were very primitive back then, even for an uncivilised island."] = "Ich budynki były wtedy bardzo prymitywne, nawet jak na niecywilizowaną wyspę.", -- A_Classic_Fairytale:united
    ["The Iron Curtain"] = "Żelazna Kurtyna",
    ["The Journey Back"] = "Podróż z powrotem", -- A_Classic_Fairytale:journey
    ["The king of %s has died!"] = "Król drużyny %s umarł!", -- Battalion
    ["The last encounter"] = "Ostatnie starcie", -- A_Space_Adventure:death01
    ["The last surviving clan wins."] = "Ostatni ocalały klan wygrywa.", -- TrophyRace
    ["The leader escaped. Defeat the rest of the aliens!"] = "Dowódca uciekł. Pokonaj resztę obcych!", -- A_Classic_Fairytale:queen
    ["The leader seems scared, he will probably flee."] = "Dowódca wydaje się być przestraszony, prawdopodobnie ucieknie.", -- A_Classic_Fairytale:queen
    ["The Leap of Faith"] = "Skok Wiary", -- A_Classic_Fairytale:first_blood
    ["The meteorite has come too close and the anti-gravity device isn't powerful enough to stop it now."] = "Meteoryt zbliżył się za bardzo i antygrawitacyjne urządzenie nie jest wystarczająco potężne, by go teraz zatrzymać.", -- A_Space_Adventure:cosmos
    ["The Moonwalk"] = "Spacer po księżycu", -- A_Classic_Fairytale:journey
    ["The Mutant has super weapons and a lot of health."] = "Mutant ma super bronie i dużo zdrowia.", -- Mutant
    ["The Mutant has super-weapons and a lot of health."] = "Mutant ma super bronie i dużo zdrowia.", -- Mutant
    ["The Mutant loses health quickly, but gains health by killing."] = "Mutant szybko traci zdrowie, ale je odnawia zabijając.", -- Mutant
    ["The Mutant loses health quickly if he doesn't keep scoring kills."] = "Mutant szybko traci zdrowie, jeśli przestanie zaliczać zabójstwa.", -- Mutant
    ["The Navy greets %s for managing to get in a distance of %d away from the mainland!"] = "Marynarka pozdrawia jeża %s, za zdołanie dostać się na odległość %d z dala od lądu!", -- ClimbHome
    ["The next 4 times you play the \"The last encounter\" mission you'll get 20 more hit points and a laser sight."] = "Przez następne 4 razy, w których zagrasz misję \"Ostateczne spotkanie\", dostaniesz 20 więcej punktów zdrowia i celownik laserowy.", -- A_Space_Adventure:death02
    ["The next crate is an utility crate."] = "Następna skrzynia zawiera narzędzia.", -- Basic_Training_-_Movement
    ["The next one is pretty hard! |Tip: You have to do multiple swings!"] = "Następne jest bardzo trudne!|Porada: Masz wielokrotne wymachy!", -- Basic_Training_-_Rope
    ["The next target can only be reached by something called “bouncing bomb”."] = "Następny cel może być sięgnięty jedynie przez coś nazwane \"odbijająca się bomba\".", -- Basic_Training_-_Bazooka
    ["The next target is high in the sky."] = "Następny cel jest wysoko w niebie.", -- Basic_Training_-_Bazooka
    ["Then how do they keep appearing?"] = "Więc jak oni się cały czas pojawiają?", -- A_Classic_Fairytale:shadow
    ["The Ninja-Samurai Alliance"] = "Sojusz Ninja-Samurajski",
    ["Then prepare for battle!"] = "Więc przygotuj się na bitwę!", -- A_Space_Adventure:death01
    ["Then what am I?"] = "Więc czym jestem?", -- A_Classic_Fairytale:epil
    ["The only woman, huh?"] = "Jedyna kobieta, co?", -- A_Classic_Fairytale:epil
    ["The oppression of the elders, of course!"] = "Ucisk starszych, oczywiście!", -- A_Classic_Fairytale:queen
    ["The opression of the elders, of course!"] = "Ucisk starszych, oczywiście!", -- A_Classic_Fairytale:queen
    ["The other hog has died, he should have survived!"] = "Drugi jeż umarł, a powinien przeżyć!", -- A_Space_Adventure:moon02
    ["The other one were all cannibals, spending their time eating the organs of fellow hedgehogs..."] = "To inne składało się z kanibali, spędzających swój czas na jedzeniu organów jeży...", -- A_Classic_Fairytale:first_blood
    ["The Police"] = "Policja",
    ["The power of love! No, wait, the power of the aliens!"] = "Potęga miłości! Nie, czekaj, potęga obcych!", -- A_Classic_Fairytale:queen
    ["The RC plane only carries 2 weak bombs."] = "Radiowy samolocik niesie tylko 2 słabe bomby.", -- Battalion
    ["There are a variety of structures available to aid you."] = "Jest różnorodność dostępnych struktur, by ci pomóc.", -- Construction_Mode
    ["There are no snarky comments this time."] = "Tym razem nie ma  komentarzy.", -- Space_Invasion
    ["There is one below us!"] = "Jedna jest pod nami!", -- A_Space_Adventure:ice01
    ["There must be a spy among us!"] = "Musi być między nami szpieg!", -- A_Classic_Fairytale:backstab
    ["There's more of them? When did they become so hungry?"] = "Jest ich więcej? Kiedy oni stali się tacy głodni?", -- A_Classic_Fairytale:backstab, A_Classic_Fairytale:united
    ["There's nothing more satisfying for me than seeing you share your beauty with the world every morning, my princess!"] = "Nie ma dla mnie nic bardziej satysfakcjonującego, niż widzenie jak dzielisz się swoją pięknością ze światem każdego ranka, moja księżniczko!", -- A_Classic_Fairytale:journey
    ["There's nothing more satisfying to us than seeing you share your beauty..."] = "Nie ma dla mnie nic bardziej satysfakcjonującego, niż widzenie jak dzielisz się swoją pięknością...", -- A_Classic_Fairytale:journey
    ["There's nothing more satisfying to us than seeing you share your beauty with the world every morning, my princess!"] = "Nie ma dla nas nic bardziej satysfakcjonującego, niż widzenie jak dzielisz się swoją pięknością ze światem każdego ranka, moja księżniczko!", -- A_Classic_Fairytale:journey
    ["The respawner respawns %s"] = "Odradzacz odradza %s", -- Construction_Mode
    ["The Rising"] = "Powstanie", -- A_Classic_Fairytale:first_blood
    ["The rope won't get reset."] = "Lina się nie zresetuje.", -- A_Space_Adventure:death02
    ["The Savior"] = "Wybawiciel", -- A_Classic_Fairytale:journey
    ["The score and deaths are shown next to the team bar."] = "Wynik i śmierci są pokazane obok paska drużyny.", -- Mutant
    ["These girders are slippery, like ice."] = "Belki są śliskie jak lód.", -- Basic_Training_-_Movement
    ["These primitive people are so funny!"] = "Prymitywni ludzie są tacy zabawni!", -- A_Classic_Fairytale:backstab
    ["These weapon specials cannot be used close to other hogs."] = "Te speciały broni nie mogą zostać użyte w pobliżu innych jeży.", -- Continental_supplies
    ["The Shadow Falls"] = "Zapada cień", -- A_Classic_Fairytale:shadow
    ["The Showdown"] = "Pojedynek", -- A_Classic_Fairytale:shadow
    ["The Slaughter"] = "Rzeź", -- A_Classic_Fairytale:dragon, A_Classic_Fairytale:first_blood
    ["The Society of Perfectionists greets %s (%s): No misses and %d hits in its best round."] = "Społeczeństwo Perfekcjonistów pozdrawia jeża %s (%s): Żadnego pudła i %d trafień w swojej najlepszej rundzie.", -- Space_Invasion
    ["The Specialists: Each hedgehog starts with its own weapon set"] = "Specjaliści: Każdy jeż zaczyna ze swoim własnym zestawem broni", -- The_Specialists
    ["The spinning arrows above your hedgehog show|which hedgehog is selected right now."] = "Kręcące się strzałki nad twoim jeżem pokazują,|który jeż jest właśnie wybrany.", -- Basic_Training_-_Movement
    ["The spirits of the ancerstors are surely pleased, Leaks A Lot."] = "Dusze przodków są z pewnością zadowolone, Spory Przecieku.", -- A_Classic_Fairytale:first_blood
    ["The spirits of the ancestors are surely pleased, Leaks A Lot."] = "Dusze przodków z pewnością są zadowolone, Spory Przecieku.", -- A_Classic_Fairytale:first_blood
    ["The targets will guide you through the training."] = "Cele poprowadzą cię przez trening.", -- Basic_Training_-_Rope
    ["The team continued their quest of finding the rest of the tribe."] = "Drużyna kontynuowała swoje zadanie odnalezienia reszty plemienia.", -- A_Classic_Fairytale:queen
    ["The teams are tied for the fastest time."] = "Drużyny remisują z najszybszym czasem.", -- Racer, TechRacer
    ["The teams were tied, so an additional round has been played to determine the winner."] = "Drużyny zremisowały, więc dodatkowa runda została zagrana, by wyłonić zwycięzcę.", -- Space_Invasion
    ["The teams were tied, so %d additional rounds have been played to determine the winner."] = "Drużyny remisowały, więc %d dodatkowych rund zostało zagranych, by wyłonić zwycięzcę.", -- Space_Invasion
    ["The time that you have left when you reach the blue hedgehog will be added to the next turn."] = "Czas, który ci pozostanie po dotarciu do niebieskiego jeża, zosanie dodany do następnej tury.", -- A_Space_Adventure:moon02
    ["The Torment"] = "Udręka", -- A_Classic_Fairytale:first_blood
    ["The truth about Professor Hogevil"] = "Prawda o Profesorze Jeżozło", -- A_Space_Adventure:moon02
    ["The tunnel entrance is over there."] = "Wejście do tunelu jest tutaj.", -- A_Space_Adventure:desert01
    ["The tunnel is about to get flooded!"] = "Tunel zaraz zostanie zalany!", -- A_Space_Adventure:desert02
    ["The Tunnel Maker"] = "Wytwórca Tuneli", -- A_Classic_Fairytale:journey
    ["The Ultimate Weapon"] = "Broń Ostateczna", -- A_Classic_Fairytale:first_blood
    ["The Union"] = "Unia", -- A_Classic_Fairytale:enemy
    ["The Union: You can select a hedgehog at the start of your turns."] = "Unia: Możesz wybrać jeża na początku swoich tur.", -- Continental_supplies
    ["The village, unprepared, was destroyed by the cyborgs..."] = "Wioska, nieprzygotowana, została zniszczona przez cyborgów...", -- A_Classic_Fairytale:journey
    ["The walk of Fame"] = "Spacer Chwały", -- A_Classic_Fairytale:shadow
    ["The wasted youth"] = "Zmarnowana młodość", -- A_Classic_Fairytale:first_blood
    ["The way you handled your little internal conflicts …"] = "Sposób, w który radziliście sobie ze swoimi małymi, wewnętrznymi konfliktami...", -- A_Classic_Fairytale:queen
    ["The weapon in that last crate was bestowed upon us by the ancients!"] = "Broń w tej ostatniej skrzyni została nam darowana przez starożytnych!", -- A_Classic_Fairytale:first_blood
    ["The what?!"] = "Że co?", -- A_Classic_Fairytale:dragon
    ["The wind whispers that you are ready to become familiar with tools, now..."] = "Wiatr szepcze, że stałeś się już gotowy, by zapoznać się z narzędziami...", -- A_Classic_Fairytale:first_blood
    ["The wrong hedgehog has taken the crate."] = "Nieodpowiedni jeż zabrał skrzynię.", -- SimpleMission
    ["They are all waiting back in the village, haha."] = "Oni wszyscy już czekają w wiosce, haha.", -- A_Classic_Fairytale:enemy
    ["They are up there! Take this rope and hurry!"] = "One są tam na górze! Weź tę linę i się pośpiesz!", -- A_Space_Adventure:moon01
    ["They Call Me Bullseye! +16 points!"] = "Zwą mnie Oczko! +16 punktów!", -- Space_Invasion
    ["They have weapons we've never seen before!"] = "Mają bronie, których nigdy wcześniej nie widzieliśmy!", -- A_Classic_Fairytale:united
    ["They keep appearing like this. It's weird!"] = "Oni pojawiają się tak po prostu. To dziwne!", -- A_Classic_Fairytale:united
    ["They killed %s! You bastards!"] = "Oni zabili %s! Wy dranie!", -- A_Classic_Fairytale:backstab, A_Classic_Fairytale:united
    ["They must be trying to weaken us!"] = "Muszą próbować nas osłabić!", -- A_Classic_Fairytale:enemy
    ["They never learn"] = "Nigdy się nie nauczą", -- A_Classic_Fairytale:journey
    ["They stumbled upon a pile of weapons, they seemed to be getting closer."] = "Natknęli się na kupkę broni, wydawali się zbliżać.", -- A_Classic_Fairytale:queen
    ["They told us to wear these clothes. They said that this is the newest trend."] = "Powiedzieli nam, by nosić te ubrania. Powiedzieli, że jest to najnowszy trend.", -- A_Classic_Fairytale:enemy
    ["They've been manipulating us all this time!"] = "Manipulowali nami cały ten czas!", -- A_Classic_Fairytale:enemy
    ["They won't hesitate to attack you in order to rob you!"] = "Nie zawahają się oni cię zaatakować, by cię obrabować!", -- A_Space_Adventure:desert01
    ["The Zoo"] = "Zoo",
    ["Thighlicker"] = "Udolizacz", -- A_Classic_Fairytale:united
    ["Things are going to get messy around here."] = "Sprawy tutaj staną się kłopotliwe.", -- A_Space_Adventure:fruit01
    ["This allows to select any hedgehog in your team!"] = "To pozwala wybrać dowolnego jeża w twojej drużynie!", -- Basic_Training_-_Movement
    ["This allows you to create a crate anywhere|within your clan's area of influence,|at the cost of energy."] = "To pozwala ci stworzyć skrzynię gdziekolwiek|wewnątrz strefy wpływu klanu,|kosztem energii.", -- Construction_Mode
    ["This allows you to create and place mines,|sticky mines and barrels anywhere within your|clan's area of influence at the cost of energy."] = "To pozwala ci stworzyć i ustawić miny,|lepkie miny i beczki gdziekolwiek wewnątrz|strefy wpływu twojego klanu, kosztem energii.", -- Construction_Mode
    ["This allows you to create and place mines,|sticky mines and barrels anywhere within your|clan's area of influence at the cost of energy.|Up/down: Choose object type|Left/right: Choose timer (for mines)|Cursor: Place object"] = "To pozwala ci stworzyć i ustawić miny, samoprzylepne|miny i beczki gdziekolwiek wewnątrz strefy wpływu|twojego klanu, kosztem energii.|Góra/dół: Wybierz typ obiektu|Lewo/prawo: Wybierz czas (dla min)|Kursor: Ustaw obiekt", -- Construction_Mode
    ["This almost concludes our tutorial."] = "To prawie kończy nasz samouczek.", -- Basic_Training_-_Flying_Saucer
    ["This also increases the effectiveness of Medicine."] = "To również zwiększa efektywność Lekarstwa.", -- Continental_supplies
    ["This game wasn’t really exciting."] = "Ta gra nie była naprawdę ekscytująca.", -- Space_Invasion
    ["This is a new personal best, congratulations!"] = "To nowy osobisty rekord, gratulacje!", -- A_Space_Adventure:death02, A_Space_Adventure:desert02, A_Space_Adventure:fruit03
    ["This is a new personal best time, congratulations!"] = "To nowy osobisty najlepszy czas, gratulacje!", -- A_Space_Adventure:ice02, A_Space_Adventure:moon02
    ["This is Cappy."] = "To jest Czapeczka", -- Basic_Training_-_Movement
    ["This is it! It's time to make Fell From Heaven fall for me..."] = "To jest to! Już czas sprawić, że Updały z Niebios zakocha się we mnie...", -- A_Classic_Fairytale:first_blood
    ["This island is the only place left on Earth with grass on it!"] = "Ta wyspa to jedyne pozostałe miejsce na Ziemi, w którym jest trawa!", -- A_Classic_Fairytale:enemy
    ["This is seems like a wealthy hedgehog, nice ..."] = "On wygląda jak bogaty jeż, nieźle...", -- A_Space_Adventure:desert01
    ["This is the mission panel."] = "To jest panel misji.", -- Basic_Training_-_Movement
    ["This is the Olympic stadium of saucer flying."] = "To jest Stadion Olimpijski Latania Spodkiem.", -- A_Space_Adventure:ice02
    ["This is the Olympic Stadium of Saucer Flying."] = "To jest Stadion Olimpijski Latania Spodkiem.", -- A_Space_Adventure:ice02
    ["This is typical!"] = "To typowe!", -- A_Classic_Fairytale:dragon
    ["This must be some kind of sorcery!"] = "To musi być jakaś magia!", -- A_Classic_Fairytale:shadow
    ["This must be the caves!"] = "To muszą być jaskinie!", -- A_Classic_Fairytale:backstab
    ["This one's tricky."] = "Ten jest podchwytliwy.",
    ["This planet seems dangerous!"] = "Ta planeta wydaje się niebezpieczna!", -- A_Space_Adventure:cosmos
    ["This rain is really something..."] = "Ten deszcz to naprawdę coś...",
    ["This round’s award for ultimate disappointment goes to: Everyone!"] = "Nagroda tej rundy za ostateczne rozczarowanie idzie do: Każdego!", -- ClimbHome
    ["This seems like a wealthy hedgehog, nice ..."] = "To wygląda jak bogaty jeż, nieźle...", -- A_Space_Adventure:desert01
    ["This %s is so naive! I'm going to shoot this fool so I can keep that device for myself!"] = "Ten %s jest taki naiwny! Mam zamiar zastrzelić tego głupca, żeby zatrzymać urządzenie dla siebie!", -- A_Space_Adventure:fruit02
    ["This was an awesome performance! But this challenge can be finished with even just one RC plane. Can you figure out how?"] = "To było niesamowite przedstawienie! Ale to wyzwanie może być ukończone tyko jednym samolocikiem. Potrafsz wykombinować jak?", -- User_Mission_-_RCPlane_Challenge
    ["This will be fun!"] = "To będzie zabawne!", -- A_Classic_Fairytale:enemy
    ["This will be useful when I need a new platform or if I want to rise."] = "To będzie żyteczne, kiedy będę potrzebował platformy albo chciał się wznieść.", -- portal
    ["This will certainly come in handy."] = "To z pewnością się przyda.", -- User_Mission_-_Teamwork_2
    ["This will certianly come in handy."] = "To zdecydowanie się przyda.", -- User_Mission_-_Teamwork_2
    ["Thompson"] = "Tomson",
    ["Those aliens are destroying the island!"] = "Ci obcy niszczą naszą wyspę!", -- A_Classic_Fairytale:family
    ["Those were scheduled for disposal anyway."] = "Ci i tak mieli iść do śmieci.", -- A_Classic_Fairytale:dragon
    ["Throw a 1 second mine!"] = "Rzuć 1-sekundową minę!", -- Continental_supplies
    ["Throw a baseball at your foes|and send them flying!"] = "Rzuć piłkę baseballową w swoich wrogów|i spraw, że polecą.", -- Knockball
    ["Throw a grenade to destroy the target!"] = "Rzuć granatem, by zniszczyć cel!", -- Basic_Training_-_Grenade
    ["Throw some grenades to destroy the targets!"] = "Rzuć trochę granatów, by zniszczyć cele!", -- Basic_Training_-_Grenade
    ["Thug #%d"] = "Zbir #%d", -- A_Space_Adventure:death01
    ["Tie-breaking round %d"] = "Runda remisowa %d", -- Space_Invasion
    ["Timbers"] = "Drewna",
    ["Time: %.1fs"] = "Czas: %.1fs", -- Racer, TechRacer
    ["Time: %.3fs by %s"] = "Czas: %.3fs (%s)", -- TrophyRace
    ["Time: %.3fs"] = "Czas: %.3fs", -- TrophyRace
    ["Time Box"] = "TARDIS", -- Construction_Mode
    ["Timed Kamikaze! +10 points!"] = "Wymierzony kamikaze! +10 punktów!", -- Space_Invasion
    ["Time extended! +%dsec"] = "Czas przedłużony! +%dsek", -- Space_Invasion
    ["Time extension: %ds"] = "Rozszerzenie czasu: %ds", -- Tumbler
    ["Time for a more interesting stunt, but first just collect the next crate!"] = "Czas na bardziej interesujący wyczyn, ale najpierw zbierz następną skrzynię!", -- Basic_Training_-_Flying_Saucer
    ["Timer"] = "Zapalnik", -- Basic_Training_-_Grenade
    ["Time's up!"] = "Koniec czasu!", -- Basic_Training_-_Sniper_Rifle, SpeedShoppa, Space_Invasion
    ["Time’s up!"] = "Koniec czasu!", -- TargetPractice
    ["Time to run!"] = "Czas uciekać!", -- A_Space_Adventure:fruit01
    ["Tip: Changing your aim while flying is very difficult, so adjust it before you take off."] = "Porada: Celowanie podczas latanie jest bardzo trudne, więc ustaw sobie zanim odlecisz.", -- Basic_Training_-_Flying_Saucer
    ["Tip: Don't remain for too long in the water, or you won't make it."] = "Porada: Nie pozostawaj zbyt długo w wodzie, bo ci się nie uda.", -- Basic_Training_-_Flying_Saucer
    ["Tip: If you get stuck in this training, use \"Skip turn\" to restart the current objective."] = "Porada: Jeśli utkniesz w tym treningu, użyj \"Pominięcia tury\", by zrestartować aktualne zadanie.", -- Basic_Training_-_Flying_Saucer
    ["Tip: See the \"Esc\" key (this menu) if you want to see the currently playing teams continent, or that continents specials."] = "Porada: Wciśnij klawisz \"Esc\" (to menu), jeśli chcesz zobaczyć kontynent aktualnie grającej drużyny oraz jego speciały.", -- Continental_supplies
    ["Tip: See the \"esc\" key (this menu) if you want to see the currently playing teams continent, or that continents specials."] = "Porada: Wciśnij przycisk \"esc\" (to menu), jeśli chcesz zobaczyć kontynent aktualnie grającej drużyny lub specjały tego kontynentu.", -- Continental_supplies
    ["Tip: The rope physics are different than in the real world, |use it to your advantage!"] = "Porada: Fizyka liny jest inna od tej w prawdziwym świecie.|Wykorzystaj to na swoją korzyść!", -- Basic_Training_-_Rope
    ["Tip: You can change your flying saucer|in mid-flight by hitting the [Attack] key twice."] = "Porada: Możesz zmienić swój latający talerz|podczas lotu, wciskając dwukrotnie przycisk [Atak].", -- Basic_Training_-_Flying_Saucer
    ["Tiyuri"] = "Tiyuri",
    ["Toad"] = "Toad",
    ["To begin, walk to the crate to the right."] = "By zacząć, idź do skrzyni po prawej.", -- Basic_Training_-_Movement
    ["To begin with the training, hit the attack key!"] = "By zacząć trening, wciśnij przycisk ataku!", -- Basic_Training_-_Movement
    ["To begin with the training, select the bazooka from the ammo menu!"] = "By zacząć trening, wybierz bazookę z menu amunicji!", -- Basic_Training_-_Bazooka
    ["To begin with the training, select the grenade from the ammo menu!"] = "By zacząć trening, wybierz granat z menu amunicji!", -- Basic_Training_-_Grenade
    ["To begin with the training, tap the attack button!"] = "By zacząć trening, wciśnij przycisk ataku!", -- Basic_Training_-_Movement
    ["To finish hedgehog selection, just do anything|with him, like walking."] = "By skończyć wybierać jeża, zrób z nim cokolwiek,|np. rusz się", -- Basic_Training_-_Movement
    ["To get over the next obstacles, keep some distance from the wall before you back jump."] = "By przekroczyć następną przeszkodę, utrzymaj trochę dystansu od ściany zanim skoczysz w tył.", -- Basic_Training_-_Movement
    ["To get over the water, you have to do multiple|rope shots and swings."] = "By przejść przez wodę, musisz wykonać wielokrotne|strzały i zamachy liną.", -- Basic_Training_-_Rope
    ["Toggle Editing Weapons and Tools: [Precise]+[2]"] = "Przełącz edytujące bronie i narzędzia: [Precyzja]+[2]", -- HedgeEditor
    ["Toggle Help: [Precise]+[1]"] = "Przełącz pomoc: [Precyzja]+[1]", -- HedgeEditor
    ["Toggle Placement/Deletion: [Left], [Right]"] = "Przełącz ustawianie/usuwanie: [Lewo], [Prawo]", -- HedgeEditor
    ["Toggle Shield: [Long jump]"] = "Przełącz osłonę: [Długi skok]", -- Space_Invasion
    ["To help you, of course!"] = "By ci pomóc, oczywiście!", -- A_Classic_Fairytale:journey
    ["To launch a projectile in mid-flight, hold [Precise] and press [Long jump]."] = "By wystrzelić pocisk w locie, przytrzymaj [Precyzja] i naciśnij [Długi skok].", -- Basic_Training_-_Flying_Saucer
    ["Tony"] = "Tony",
    ["Too bad! Then you should really leave!"] = "Jaka szkoda! Więc musisz naprawdę odejść!", -- A_Space_Adventure:fruit01
    ["Too slow! Try again ..."] = "Za wolno! Spróbuj ponownie...", -- A_Space_Adventure:moon02
    ["Top-class elite pilot"] = "Elitarny pilot najwyższej klasy", -- User_Mission_-_RCPlane_Challenge
    ["To reach higher ground, walk to a ledge, look to the left, then do a back jump."] = "By dostać się na wyższy poziom, wejdź na krawędz, spójrz w lewo i wykonaj skok w tył.", -- Basic_Training_-_Movement
    ["Torn Muscle"] = "Rozerwany Mięsień", -- A_Classic_Fairytale:journey
    ["To the caves..."] = "Do jaskiń...", -- A_Classic_Fairytale:united
    ["Touch all waypoints as fast as you can!"] = "Dotknij wszystkich punktów kontrolnych najszybciej jak potrafisz!", -- Racer
    ["- Touch the sparkles near your base to teleport"] = "- Dotknij iskier przy swojej bazie, by się teleportować", -- CTF_Blizzard
    ["To win the game, %s has to get the bottom crates and come back to the surface."] = "By wygrać grę, %s musi się dostać do dolnych skrzyń i wrócić na powierzchnię.", -- A_Space_Adventure:fruit02
    ["To win the game you had to collect the 2 crates with no specific order."] = "By wygrać grę, musisz zebrać 2 skrzynie, bez specjalnej kolejności.", -- A_Space_Adventure:desert01
    ["To win the game you have to eliminate Professor Hogevil."] = "By wygrać grę, musisz wyeliminować Profesora Jeżozło.", -- A_Space_Adventure:death01
    ["To win the game you have to find the right crate."] = "By wygrać grę, musisz znaleźć odpowiednią skrzynię.", -- A_Space_Adventure:desert01
    ["To win the game you have to go next to Thanta."] = "By wygrać grę, musisz pójść obok Tanty.", -- A_Space_Adventure:ice01
    ["To win the game you have to go to the surface."] = "By wygrać grę, musisz pójść na powierzchnię.", -- A_Space_Adventure:desert02
    ["To win the game you have to pass into the rings in time."] = "By wygrać grę, musisz na czas przejść przez pierścienie.", -- A_Space_Adventure:ice02
    ["To win the game you have to stand next to Thanta."] = "By wygrać grę, musisz stanąć obok Tanty.", -- A_Space_Adventure:ice01
    ["Toxic Team"] = "Toksyczny Zespół",
    ["Track completed!"] = "Trasa ukończona!", -- Racer, TechRacer
    ["Training complete!"] = "Trening ukończony!", -- Basic_Training_-_Flying_Saucer
    ["Training"] = "Trening", -- Basic_Training_-_Flying_Saucer, Basic_Training_-_Rope
    ["Traitors don't get to shout around here!"] = "Zdrajcy tutaj nie krzyczą!", -- A_Classic_Fairytale:epil
    ["Traitors"] = "Zdrajcy", -- A_Classic_Fairytale:epil
    ["Trapper"] = "Traper", -- HedgeEditor
    ["Travel carefully as your fuel is limited"] = "Lataj ostrożnie, gdyż twoje paliwo jest ograniczone", -- A_Space_Adventure:cosmos
    ["Travel to all the neighbor planets and collect all the pieces"] = "Poleć na wszystkie sąsiednie planety i zbierz wszystkie części", -- A_Space_Adventure:cosmos
    ["Treasure: Massive weapon bonus in first turn."] = "Skarb: Masywny bonus broni w pierwszej turze.", -- Continental_supplies
    ["Tribe"] = "Plemię", -- A_Classic_Fairytale:backstab
    ["TrophyRace"] = "TrophyRace",
    ["Trunks"] = "Trunks",
    ["Try again!"] = "Spróbuj ponownie!", -- Basic_Training_-_Flying_Saucer
    ["Try it now and dive here to collect the crate on the right girder."] = "Spróbuj teraz i zanurkuj tu, by zebrać skrzynię na prawej belce.", -- Basic_Training_-_Flying_Saucer
    ["Try not to get spotted by the guards!"] = "Spróbuj nie zostać zauważonym przez straże!", -- A_Space_Adventure:cosmos
    ["Try out different bounciness levels to reach difficult targets."] = "Spróbuj różne poziomy odbijalności, by dostać się do trudnych celów.", -- Basic_Training_-_Grenade
    ["Try to be smart and eliminate them quickly. This way you might scare off the rest!"] = "Spróbuj być mądry i wyeliminuj ich szybko. W ten sposób, możesz odstraszyć resztę!", -- A_Space_Adventure:fruit01
    ["Try to keep as many allies alive as possible."] = "Spróbuj utrzymać przy życiu najwięcej sojuszników, ile się da.", -- A_Space_Adventure:fruit01
    ["Try to land softly, as you can still take fall damage!"] = "Spróbuj wylądować miękko, bo nadal możesz otrzymać obrażenia od upadku!", -- Basic_Training_-_Flying_Saucer
    ["Try to protect the chief! You won't lose if he dies, but it is advised that he survives."] = "Spróbuj ochronić wodza! Nie przegramy, gdy on umrze, ale zalecane jest, by przeżył.", -- A_Classic_Fairytale:united
    ["Try to reach and destroy the next target quickly."] = "Spróbuj dostać się i zniszczyć następny cel szybko.", -- Basic_Training_-_Rope
    ["Tumbler"] = "Akrobata", -- Tumbler
    ["Turn around: [Left Shift] + [Left]/[Right]"] = "Obróć się: [Lewy Shift] + [Lewo]/[Prawo]", -- Basic_Training_-_Movement
    ["Turning Around"] = "Obracanie się", -- Basic_Training_-_Movement
    ["Turns: Hogs get %d random weapon(s) from their pool"] = "Tury: Jeż dostaje %d losowych broni ze swojej puli", -- Battalion
    ["Turns: King's health is set to %d%% of the team health"] = "Tury: Zdrowie króla jest ustawione na %d%% zdrowia drużyny", -- Battalion
    ["Turns left: %d"] = "Pozostałe tury: %d", -- A_Classic_Fairytale:journey
    ["Turns: Refill %d weapon and %d helper points|and randomize weapons and helpers based on team points"] = "Tura: Odnów %d broni i %d punktów pomocniczych|i wylosuj bronie i pomocników według punktów drużyny", -- Battalion
    ["Turns until arrival: %d"] = "Tury do przybycia: %d", -- A_Classic_Fairytale:backstab
    ["Turn Time: %dsec"] = "Czas tury: %dsec", -- Space_Invasion
    ["Twenty-Twenty"] = "Dwadzieścia-dzieścia",
    ["Two flowers: All missions complete"] = "Dwa kwiaty: Wszystkie misje ukończone", -- A_Space_Adventure:cosmos
    ["Two little hogs cooperating, getting past obstacles..."] = "Dwa małe jeże współpracują, pokonując przeszkody...", -- A_Classic_Fairytale:journey
    ["Ugly Mug"] = "Brzydki Bandyta",
    ["Uhm...I met one of them and took his weapons."] = "Um... Spotkałem jednego z nich i zabrałem mu bronie.", -- A_Classic_Fairytale:shadow
    ["Uhmm, it's … uhm … my ring!"] = "Umm, to... um... mój pierścień!", -- A_Classic_Fairytale:queen
    ["Uhmm...ok no."] = "Umm... niedobrze.", -- A_Classic_Fairytale:enemy
    ["Ukemi"] = "Ukemi",
    ["Ultra kill!"] = "Ultra zabójstwo!", -- Mutant
    ["unC0Rr"] = "unC0Rr",
    ["Under Construction"] = "W Trakcie Budowy", -- A_Classic_Fairytale:shadow
    ["Under normal circumstances we could easily defeat them but we have kindly sent most of our men to the Kingdom of Sand to help with the annual dusting of the king's palace."] = "W normalnych okolicznościach pokonalibyśmy ich z łatwością, ale życzliwie wysłaliśmy większość naszych ludzi do Królestwa Piasku, by pomóc z corocznym odkurzaniu pałacu króla.", -- A_Space_Adventure:fruit01
    ["Under the meteorite’s shadow ..."] = "Pod cieniem meteorytu...", -- A_Space_Adventure:cosmos
    ["Under the meteorites shadow ..."] = "Pod cieniem meteorytu...", -- A_Space_Adventure:cosmos
    ["Unexpected Igor"] = "Nieoczekiwany Igor", -- A_Classic_Fairytale:dragon
    ["Unique new weapons"] = "Unikalne nowe bronie", -- Continental_supplies
    ["Unit 0x0007"] = "Jednostka 0x0007", -- A_Classic_Fairytale:family
    ["Unit 189"] = "Jednostka 189",
    ["Unit 234"] = "Jednostka 234",
    ["Unit 333"] = "Jednostka 333",
    ["Unit 334a$7%;.*"] = "Jednostka 334a$7%;.*", -- A_Classic_Fairytale:backstab, A_Classic_Fairytale:dragon, A_Classic_Fairytale:enemy, A_Classic_Fairytale:family, A_Classic_Fairytale:queen, A_Classic_Fairytale:united
    ["Unit 3378"] = "Jednostka 3378",
    ["Unit 485"] = "Jednostka 485",
    ["Unit 527"] = "Jednostka 527",
    ["Unit 638"] = "Jednostka 638",
    ["Unit 709"] = "Jednostka 709",
    ["Unit 835"] = "Jednostka 835",
    ["Unit 881"] = "Jednostka 881", -- User_Mission_-_Newton_and_the_Hammock
    ["Unit 883"] = "Jednostka 883",
    ["United We Stand"] = "Zgoda buduje", -- A_Classic_Fairytale:united
    ["Unit"] = "Jednostka",
    ["Unlike bazookas, grenades are not influenced by wind."] = "W przeciwieństwie do bazook, na granaty wiatr nie ma wpływu.", -- Basic_Training_-_Grenade
    ["Unlimited Attacks: Attacks don't end your turn"] = "Nieskończone ataki: Ataki nie kończą twojej tury", -- User_Mission_-_Diver, User_Mission_-_Nobody_Laugh, User_Mission_-_Spooky_Tree
    ["Unlucky Sods"] = "Pechowi Kolesie", -- User_Mission_-_Rope_Knock_Challenge
    ["Unstoppable!"] = "Nie do zatrzymania!",
    ["Unsuspecting Louts"] = "Niepodejrzewające Chamy", -- User_Mission_-_Rope_Knock_Challenge
    ["Up/Down: Adjust dust storm damage"] = "Góra/Dół: Dostosuj obrażenia burzy pyłowej", -- Continental_supplies
    ["Up/Down: Browse through continents"] = "Góra/Dół: Przeglądaj kontynenty", -- Continental_supplies
    ["Up/Down: Change placement mode"] = "Góra/Dół: Zmień tryb stawiania", -- HedgeEditor
    ["Up/down: Choose crate type"] = "Góra/Dół: Zmień typ skrzyni", -- Construction_Mode
    ["Up/down: Choose object type|1-5/Switch/Left/Right: Choose mine timer|Cursor: Place object"] = "Góra/Dół: Wybierz typ obiektu|1-5/Zmiana/Lewo/Prawo: Wybierz czas miny|Kursor: Ustaw obiekt", -- Construction_Mode
    ["Upper-class elite pilot"] = "Elitarny pilot wyższej klasy", -- User_Mission_-_RCPlane_Challenge
    ["Upside-Down World"] = "Świat do góry nogami", -- Continental_supplies
    ["Use it wisely!"] = "Użyj tego mądrze!", -- A_Classic_Fairytale:dragon
    ["Use it with precaution!"] = "Użyj tego z ostrożnośćią!", -- A_Classic_Fairytale:first_blood
    ["User Challenge"] = "Wyzwanie użytkownika",
    ["!"] = "!", -- User_Mission_-_Dangerous_Ducklings
    ["User Mission"] = "Misja użytkownika", -- HedgeEditor
    ["Use the attack key twice to change the flying saucer while being in air."] = "Wciśnij dwukrotnie przycisk ataku, by zmienić latający talerz w powietrzu.", -- A_Space_Adventure:ice02
    ["Use the attack key twice to change the flying saucer while floating in mid-air."] = "Wciśnij dwukrotnie przycisk ataku, by zmienić latający talerz unosząc w powietrzu.", -- A_Space_Adventure:ice02
    ["Use the bazooka and the flying saucer to get the freezer."] = "Użyj bazooki i latającego talerza, by zdobyć zamrażarkę.", -- A_Space_Adventure:ice01
    ["Use the flying saucer from the crate to fly to the moon."] = "Użyj latającego talerza ze skrzyni, by polecieć na księżyc.", -- A_Space_Adventure:cosmos
    ["Use the flying saucer to fly to other planets."] = "Użyj latającego talerza, by polecieć na inne planety.", -- A_Space_Adventure:cosmos
    ["Use the flying saucer to fly to the other planets."] = "Użyj latającego talerza, by polecieć na inne planety.", -- A_Space_Adventure:cosmos
    ["Use the parachute to get the next crate."] = "Użyj spadochronu, by zdobyć następną skrzynię.", -- A_Classic_Fairytale:first_blood
    ["Use the portal gun to get to the next crate, then use the new gun to get to the final destination!|"] = "Użyj wyrzutnika portali, by zdobyć następną skrzynię, potem użyj nowej broni, by dostać sie do ostatniego celu!|", -- A_Classic_Fairytale:dragon
    ["Use the RC plane and destroy the all the targets."] = "Użyj radiowego samolociku i zniszcz wszystkie cele.", -- A_Space_Adventure:desert03
    ["Use the rope in order to catch the blue hedgehog"] = "Użyj liny, żeby złapać niebieskiego jeża", -- A_Space_Adventure:moon02
    ["Use the rope to complete the obstacle course!"] = "Użyj liny, by ukończyć tor przeszkód!", -- Basic_Training_-_Rope
    ["Use the rope to get on the head of the mole, young one!"] = "Użyj liny, by dostać się na głowę jeża, młodzieńcze!", -- A_Classic_Fairytale:first_blood
    ["Use the rope to get to the crate"] = "Użyj liny, by dostać się do skrzyni", -- A_Space_Adventure:cosmos
    ["Use the rope to get to the target!"] = "Użyj liny, by dostać się do celu!", -- Basic_Training_-_Rope
    ["Use the rope to knock your enemies to their doom."] = "Użyj liny, by wybić swoich wrogów na ich zgubę.", -- User_Mission_-_Rope_Knock_Challenge
    ["Use the rope to quickly get to the surface!"] = "Użyj liny, by szybko dostać się na powierzchnię!", -- A_Space_Adventure:desert02
    ["Use the saucer and fly away"] = "Użyj talerza i odleć", -- A_Space_Adventure:cosmos
    ["Use the saucer and fly to the moon"] = "Użyj taerza i poleć na księżyc", -- A_Space_Adventure:cosmos
    ["Use the shield to protect yourself from bazookas."] = "Użyj osłony, by chronić się przed bazookami.", -- Space_Invasion
    ["Use the structure placer to place structures."] = "Użyj stawiacza struktur, by stawiać struktury.", -- Construction_Mode
    ["Use your ammo wisely."] = "Użyj mądrze swojej amunicji.", -- A_Space_Adventure:desert01
    ["Use your available weapons in order to eliminate the enemies."] = "Użyj swoich dostępnych broni, żeby wyeliminować wrogów.", -- A_Space_Adventure:death02, A_Space_Adventure:fruit03
    ["Use your ready time to think."] = "Użyj swojego czasu przygotowania, by pomyśleć.", -- Frenzy
    ["Use your rope to collect all crates as fast as possible."] = "Użyj swojej liny, by jak najszybciej zebrać wszystkie skrzynie.", -- SpeedShoppa
    ["Use your rope to get from start to finish as fast as you can!"] = "Użyj liny, by jak najszybciej dotrzeć od startu do mety",
    ["Use your rope to get to the next target, then destroy it!"] = "Użyj swojej liny, by dostać się do następnego celu, a potem go zniszcz!", -- Basic_Training_-_Rope
    ["Utility Crate Placement Mode"] = "Tryb Stawiania Skrzyń z Narzędziami", -- Construction_Mode
    ["UTILITY CRATE PLACEMENT MODE"] = "TRYB STAWIANIA SKRZYŃ Z NARZĘDZIAMI", -- HedgeEditor
    ["Utility crates extend your time."] = "Skrzynie z narzędziami zwiększają twój czas.", -- Tumbler
    ["Variants: Hogs will be randomized from 12 different variants"] = "Wariant: Jeże będą wylosowane z 12 różnych wariantów.", -- Battalion
    ["Variants: Kings and air generals are disabled"] = "Wariant: Królowie i powietrzni generałowie są wyłączeni", -- Battalion
    ["Variants: The last hog of each team will be a king"] = "Wariant: Ostatni jeż każdej drużyny będzie królem", -- Battalion
    ["Vedgies"] = "Warzywka", -- A_Classic_Fairytale:journey
    ["Vegan Jack"] = "Wegański Jacek", -- A_Classic_Fairytale:enemy
    ["Vega"] = "Vega",
    ["Very valuable, haha!"] = "Bardzo wartościowe, haha!", -- A_Classic_Fairytale:queen
    ["Victory Condition: Collect"] = "Warunek zwycięstwa: Zbierz", -- HedgeEditor
    ["Victory Condition: Destroy"] = "Warunek zwycięstwa: Zniszcz", -- HedgeEditor
    ["Victory for %s!"] = "Zwycięstwo dla %s!", -- Capture_the_Flag
    ["Victory!"] = "Zwycięstwo!", -- Basic_Training_-_Rope
    ["Violence is not the answer to your problems!"] = "Przemoc nie jest odpowiedzią na twoje problemy!", -- A_Classic_Fairytale:first_blood
    ["Visit the planets of Ice, Desert and Fruit before you proceed to the Death Planet"] = "Odwiedź planety Lodu, Pustyni i owoców, zanim udasz się na Planetę Śmierci", -- A_Space_Adventure:cosmos
    ["Vladimir"] = "Władimir",
    ["Void"] = "Pustka", -- Big_Armory
    ["Voldemort"] = "Voldemort", -- portal
    ["Voltorb"] = "Voltorb",
    ["Wait a moment …"] = "Czekaj no chwilę...", -- A_Space_Adventure:final
    ["Walking on Ice"] = "Chodzenie po lodzie", -- Basic_Training_-_Movement
    ["Walk: [Left] and [Right]"] = "Chodzenie: [Lewo] i [Prawo]", -- Basic_Training_-_Movement
    ["Walk: [Left]/[Right]"] = "Chodzenie: [Lewo]/[Prawo]", -- Basic_Training_-_Bazooka
    ["Wall Before Crate: You must touch the marked wall before you can get crates."] = "Ściana przed skrzynią: Musisz dotknąć zaznaczonej ściany, zanim będziesz mógł zebrać skrzynie.", -- WxW
    ["Walls Before Crate: You must touch the %d marked walls before you can get crates."] = "Ściana przed skrzynią: Musisz dotknąć %d zaznaczonych ścian, zanim będziesz mógł zebrać skrzynie.", -- WxW
    ["Wall set: No walls"] = "Ustawienie ścian: Bez ścian", -- WxW
    ["Wall set: %s (%d walls)"] = "Ustawienie ścian: %s (%d ścian)", -- WxW
    ["Wall set: %s"] = "Ustawienie ścian: %s", -- WxW
    ["Walls left: %d"] = "Pozostałe ściany: %d", -- WxW
    ["Wall to wall"] = "Ściana do ściany", -- WxW
    ["Waluigi"] = "Waluigi",
    ["Wario"] = "Wario",
    ["Warming Up"] = "Rozgrzewka", -- Basic_Training_-_Grenade
    ["Warning: Fire cake detected"] = "Ostrzeżenie: Ogniste ciasto wykryte", -- ClimbHome
    ["Warning: Never ever leave the flying saucer while in water!"] = "Ostrzeżenie: Nigdy przenigdy nie opuszczaj latającego talerza pod wodą!", -- Basic_Training_-_Flying_Saucer
    ["WARNING: Sabotage detected!"] = "OSTRZEŻENIE: Wykryto sabotaż!", -- Continental_supplies
    ["Warrior"] = "Wojownik", -- Battalion
    [" was extracted from the scheme|- This continent will be able to use the specials from the other continents!"] = " został wyciągnięty ze schematu|- Ten kontynent będzie mógł użyć specjali z innych kontynentów!", -- Continental_supplies
    ["WatchBot 4000"] = "PaczBot 4000", -- User_Mission_-_Teamwork_2
    ["Watch your steps, young one!"] = "Patrz pod nogi, młodzieńczce!", -- A_Classic_Fairytale:first_blood
    ["Watermelon Heart"] = "Melonowe Serce", -- A_Space_Adventure:fruit02
    ["Water: Rises by 37 per turn"] = "Woda: Podnosi się o 37 co turę", -- Battalion
    ["Waypoint Editing Mode"] = "Tryb Edycji Punktów Kontrolnych", -- HedgeEditor
    ["WAYPOINT EDITING MODE"] = "TRYB EDYCJI PUNKTÓW KONTROLNYCH", -- HedgeEditor
    ["Waypoint placed. Available points remaining: %d"] = "Punkt kontrolny ustawiony. Pozostałe dostępne punkty: %d", -- Racer
    ["Waypoint placement phase"] = "Faza ustawiania punktów kontrolnych", -- Racer
    ["Waypoint removed. Available points: %d"] = "Punkt kontrolny usunięty. Pozostałe punkty: %d", -- Racer
    ["Waypoints remaining: %d"] = "Pozostałe punkty kontrolne: %d", -- Racer, TechRacer
    ["Weaklings"] = "Słabeusze", -- A_Classic_Fairytale:shadow
    ["We all know what happens when you get frightened..."] = "Wszyscy wiemy, co się dzieje, gdy się przestraszysz...", -- A_Classic_Fairytale:first_blood
    ["Weapon Crate Placement Mode"] = "Tryb Ustawiania Skrzyń z Brońmi", -- Construction_Mode
    ["WEAPON CRATE PLACEMENT MODE"] = "TRYB USTAWIANIA SKRZYŃ Z BROŃMI", -- HedgeEditor
    ["Weapon Filter: Dematerializes all ammo|    carried by enemies entering it."] = "Filtr Broni: Dematerializuje całą amunicję|    niesioną przez wroga, który wejdzie.", -- Construction_Mode
    ["Weapon Filter"] = "Filtr Broni", -- Construction_Mode
    ["weaponschemes"] = "schematy broni", -- Continental_supplies
    ["Weapons: Each team starts with %d weapon points"] = "Bronie: ażda drużyna zaczyna z %d punktami broni", -- Battalion
    ["Weapons: Hogs will get 1 out of 3 weapons randomly each turn"] = "Bronie: Jeże dostaną 1 z 3 broni losowo w każdej turze", -- Battalion
    ["Weapons: Nearly every hog variant gets 1 kamikaze"] = "Bronie: Prawie każdy wariant jeża dostaje 1 kamikaze", -- Battalion
    ["Weapon specials: Some weapons have special modes (see weapon description)."] = "Speciały broni: Niektóre bronie mają specjalne tryby (zobacz opis broni).", -- Continental_supplies
    ["Weapons reset: The weapons are reset after each turn."] = "Reset broni: Bronie resetują się po każdej turze.", -- WxW
    ["We are indeed."] = "Jesteśmy zaiste.", -- A_Classic_Fairytale:backstab
    ["We can't defeat them!"] = "Nie możemy ich pokonać!", -- A_Classic_Fairytale:shadow
    ["We can't hold them up much longer!"] = "Nie możemy ich zatrzymać wiele dłużej!", -- A_Classic_Fairytale:united
    ["We can't let them take over our little island!"] = "Nie możemy im pozwolić przejąć naszej małej wyspy!", -- A_Classic_Fairytale:enemy
    ["We come in peace! Just let our friends go!"] = "Przychodzimy w pokoju! Wypuśćcie tylko naszych przyjaciół!", -- A_Classic_Fairytale:queen
    ["We could just have blown up the meteorite from the the beginning!"] = "Mogliśmy od początku po prostu wysadzić ten meteoryt!", -- A_Space_Adventure:final
    ["We don't have time for that now!"] = "Nie mamy teraz na to czasu!", -- A_Classic_Fairytale:queen
    ["We have lost an object which was critical to this mission."] = "Straciliśmy obiekt, który był krytyczny dla tej misji.", -- SimpleMission
    ["We have no time to waste..."] = "Nie mamy czasu do stracenia...", -- A_Classic_Fairytale:journey
    ["We have nowhere else to live!"] = "Nie mamy gdzie indziej mieszkać!", -- A_Classic_Fairytale:enemy
    ["We have spotted the enemy! We'll attack when the enemies start gathering!"] = "Zauważyliśmy wrogra! Zaatakujemy, kiedy wrogowie zaczną się zbierać!", -- A_Space_Adventure:fruit02
    ["We have to find our folk!"] = "Musimy odnaleźć naszych ludzi!", -- A_Classic_Fairytale:queen
    ["We have to hurry! Are you armed?"] = "Musimy się pośpieszyć! Jesteś uzbrojony?", -- A_Space_Adventure:moon01
    ["We have to protect the village!"] = "Musimy bronić wioski!", -- A_Classic_Fairytale:united
    ["We have to unite and defeat those cylergs!"] = "Musimy się zjednoczyć i pokonać cylergów!", -- A_Classic_Fairytale:enemy
    ["Welcome home! Please take a seat"] = "Witaj w domu! Usiądź proszę", -- ClimbHome
    ["Welcome, Leaks A Lot!"] = "Witaj, Spory Przecieku!", -- A_Classic_Fairytale:journey
    ["Welcome, %s, surprised to see me?"] = "Witaj, %s, zaskoczony widząc mnie?", -- A_Space_Adventure:death01
    ["Welcome to the Death Planet!"] = "Witaj na Planecie Śmierci!", -- A_Space_Adventure:cosmos
    ["Welcome to the Desert Planet!"] = "Witaj na Planecie Pustyni!", -- A_Space_Adventure:cosmos
    ["Welcome to the Fruit Planet!"] = "Witaj na Planecie Owoców!", -- A_Space_Adventure:cosmos
    ["Welcome to the meteorite!"] = "Witaj na meteorycie!", -- A_Space_Adventure:cosmos
    ["Welcome to the moon!"] = "Witaj na księżycu!", -- A_Space_Adventure:cosmos
    ["Welcome to the Planet of Ice!"] = "Witaj na Planecie Lodu!", -- A_Space_Adventure:cosmos
    ["Well done."] = "Dobra robota",
    ["Well done! Let's destroy the next target!"] = "Dobra robota! Zniszcz następny cel!", -- Basic_Training_-_Rope
    ["Well done! The next target awaits."] = "Dobra robota! Następny cel czeka.", -- Basic_Training_-_Rope
    ["We'll give you a problem then!"] = "Więc sprawimy ci problem!", -- A_Classic_Fairytale:enemy
    ["We'll play a game first."] = "Najpierw zagramy w grę.", -- A_Space_Adventure:moon02
    ["We'll spare your life for now!"] = "Na razie oszczędzimy twoje życie!", -- A_Classic_Fairytale:backstab
    ["Well, that escalated quickly!"] = "Cóż, to eskalowało szybko.", -- ClimbHome
    ["Well that was an unnecessary act of violence."] = "Cóż, to był niepotrzebny akt przemocy.", -- A_Classic_Fairytale:epil
    ["Well, that was an unnecessary act of violence."] = "Cóż, to był niepotrzebny akt przemocy.", -- A_Classic_Fairytale:epil
    ["Well, that was a waste of time."] = "Cóż, to była strata czasu.", -- A_Classic_Fairytale:dragon
    ["We'll use our communicators to contact you."] = "Użyjemy naszych komunikatorów, by się z tobą skontaktować.", -- A_Space_Adventure:cosmos
    ["Well, well! Isn't that the cutest thing you've ever seen?"] = "No no! Czy to nie najsłodsza rzecz, jaką kiedykolwiek widziałeś?", -- A_Classic_Fairytale:journey
    ["Well, yes. This was a cyborg television show."] = "Cóż, tak. To był program telewizyjny cyborgów.", -- A_Classic_Fairytale:enemy
    ["Well, you're about to wake up!"] = "Cóż, zaraz się obudzisz!", -- A_Classic_Fairytale:queen
    ["We made sure noone followed us!"] = "Upewniliśmy się, że nikt nas nie śledził!", -- A_Classic_Fairytale:backstab
    ["We need it to get split into at least two parts."] = "Musimy go podzielić co najmniej na dwie części.", -- A_Space_Adventure:cosmos
    ["We need to go back!"] = "Musimy wrócic!", -- A_Classic_Fairytale:queen
    ["We need to hurry!"] = "Musimy się śpieszyć!", -- A_Classic_Fairytale:queen
    ["We need to move!"] = "Musimy ruszać!", -- A_Classic_Fairytale:united
    ["We need to prevent their arrival!"] = "Musimy zapobiec ich przybyciu!", -- A_Classic_Fairytale:backstab
    ["We need to warn the village."] = "Musimy ostrzec wioskę.", -- A_Classic_Fairytale:shadow
    ["We need you to go there and detonate them yourself! Good luck!"] = "Musimy tam pójść i zdetonować je sami! Powodzenia!", -- A_Space_Adventure:cosmos
    ["We oppressed her, the only woman in the tribe!"] = "Uciskaliśmy ją, jedyną kobietę w plemieniu!", -- A_Classic_Fairytale:epil
    ["We're terribly sorry!"] = "Bardzo nam przykro!", -- A_Classic_Fairytale:epil
    ["We risk our lives going through challenges."] = "Ryzykujemy nasze życia, przechodząc przez wyzwania.", -- A_Classic_Fairytale:queen
    ["We should better report this and continue our watch!"] = "Lepiej powinniśmy to zgłosić i kontynuować naszą wartę!", -- A_Space_Adventure:cosmos
    ["We should head back to the village now."] = "Powinniśmy udać się teraz z powrotem do wioski.", -- A_Classic_Fairytale:shadow
    ["We, the youth, have to constantly prove our value."] = "My, młodzi, musimy stale dowodzić naszej wartości.", -- A_Classic_Fairytale:queen
    ["We trusted you, you fool!"] = "Ufaliśmy ci, głupcze!", -- A_Classic_Fairytale:queen
    ["We were trying to save her and we got lost."] = "Próbowaliśmy ją uratować i się zgubiliśmy.", -- A_Classic_Fairytale:family
    ["We were your home! Your family!"] = "My byliśmy twoim domem! Twoją rodziną!", -- A_Classic_Fairytale:queen
    ["We won't accept you destroying our village!"] = "Nie zaakceptujemy twojego niszczenia naszej wioski!", -- A_Classic_Fairytale:queen
    ["We won't let you hurt any more of us!"] = "Nie pozwolimy ci skrzywdzić więcej z nas!", -- A_Classic_Fairytale:queen
    ["We won't let you hurt her!"] = "Nie pozwolimy ci jej skrzywdzić!", -- A_Classic_Fairytale:journey
    ["We work and work until we sweat blood."] = "Pracujemy i pracujemy, aż pocimy się krwią.", -- A_Classic_Fairytale:queen
    ["What?! A cannibal? Here? There is no time to waste! Come, you are prepared."] = "Co? Kanibal? Tutaj? Nie ma czasu do stracenia! Chodź, jesteś przygotowany.", -- A_Classic_Fairytale:first_blood
    ["What a douche!"] = "Co za dupek!", -- A_Classic_Fairytale:enemy
    ["What am I gonna...eat, yo?"] = "Co ja zamierzam... zjeść, joł?", -- A_Classic_Fairytale:family
    ["What are you doing at a distance so great, young one?"] = "Co robisz w odległości tak wielkiej, młodzieńcze?", -- A_Classic_Fairytale:first_blood
    ["What are you doing? Let her go!"] = "Co ty robisz? Wypuść ją!", -- A_Classic_Fairytale:journey
    ["What a ride!"] = "Co za jazda!", -- A_Classic_Fairytale:shadow
    ["What a strange cave!"] = "Co za dziwna jaskinia!", -- A_Classic_Fairytale:dragon
    ["What a strange feeling!"] = "Co za dziwne uczucie!", -- A_Classic_Fairytale:backstab
    ["What?!"] = "Co?!", -- A_Classic_Fairytale:queen
    ["What could you possibly forget in that cage?"] = "Czego mógłbyś zapomnieć w tej klatce?", -- A_Classic_Fairytale:queen
    ["What does it look like?"] = "Jak to wygląda?", -- A_Classic_Fairytale:queen
    ["What do my faulty eyes observe? A spy!"] = "Co widzą moje wadliwe oczy? Szpiega!", -- A_Classic_Fairytale:first_blood
    ["What do you say? Are you in?"] = "Co powiesz? Wchodzisz w to?", -- A_Space_Adventure:fruit02
    ["What do you say? Will you fight for us?"] = "Co powiesz? Będziesz dla nas walczył?", -- A_Space_Adventure:fruit01
    ["What do you want to do?"] = "Co chcesz zrobić?", -- A_Space_Adventure:fruit01
    ["Whatever floats your boat..."] = "Cokolwiek ci pasuje...", -- A_Classic_Fairytale:shadow
    ["What?! For all this struggle I just win some ... time? Oh dear!"] = "Co?! Za cały ten wysiłek wygrałem tylko trochę... czasu? O rany!", -- portal
    ["What has %s ever done to you?"] = "Co %s ci kiedykolwiek zrobił?", -- A_Classic_Fairytale:backstab
    ["What? Here? How did they find us?!"] = "Co? Tutaj? Jak nas znaleźli?!", -- A_Classic_Fairytale:backstab
    ["What? Is it over already?"] = "Co? To już koniec?", -- ClimbHome
    ["What is it that you forgot?"] = "O czym takim zapomniałeś?", -- A_Classic_Fairytale:queen
    ["What is this place?"] = "Co to za miejsce?", -- A_Classic_Fairytale:dragon, A_Classic_Fairytale:enemy
    ["What oppression? You were the most unoppressed member of the tribe!"] = "Jaki ucisk? Byłaś najbardziej nieuciskanym członkiem naszego plemienia!", -- A_Classic_Fairytale:queen
    ["What shall we do with the traitor?"] = "Co powinniśmy zrobić ze zdrajcą?", -- A_Classic_Fairytale:backstab
    ["What's in the box, you ask? Let's find out!"] = "Co jest w pudle pytasz? Dowiedzmy się!", -- Basic_Training_-_Movement
    ["What the?"] = "Co do?", -- A_Classic_Fairytale:queen
    ["WHAT?! You're the ones attacking us!"] = "CO? To wy nas atakowaliście!", -- A_Classic_Fairytale:enemy
    ["When I find it..."] = "Kiedy to znajdę...", -- A_Classic_Fairytale:dragon
    ["When?"] = "Kiedy?", -- A_Classic_Fairytale:enemy
    ["When you're in mid-air, you can continue to aim|and fire another rope if you're not attached."] = "Kiedy jesteś w powietrzu, możesz nadal celować|i wystrzelić kolejną linę, jeśli nie jesteś zaczepiony.", -- Basic_Training_-_Rope
    ["Where are all these crates coming from?!"] = "Skąd przychodzą te wszystkie skrzynie?!", -- A_Classic_Fairytale:shadow
    ["Where are they?!"] = "Gdzie oni są?!", -- A_Classic_Fairytale:backstab
    ["Where did that alien run?"] = "Gdzie ten obcy pobiegł?", -- A_Classic_Fairytale:dragon
    ["Where did you get the exploding apples and the magic bow that shoots many arrows?"] = "Gdzie zdobyłeś eksplodujące jabłka i magiczny łuk, który strzela wieloma strzałami?", -- A_Classic_Fairytale:shadow
    ["Where did you get the exploding apples?"] = "Gdzie ty zdobyłeś wybuchowe jabłka?", -- A_Classic_Fairytale:shadow
    ["Where did you get the magic bow that shoots many arrows?"] = "Gdzie zdobyłeś magiczny łuk, który strzela wieloma strzałami?", -- A_Classic_Fairytale:shadow
    ["Where did you get the weapons in the forest, Dense Cloud?"] = "Gdzie zdobyłeś bronie w lesie, Gęsta Chmuro?", -- A_Classic_Fairytale:backstab
    ["Where do you get that?!"] = "Gdzie takie się zdobywa?", -- A_Classic_Fairytale:enemy
    ["Where have you been?!"] = "Gdzie ty byłeś?!", -- A_Classic_Fairytale:backstab
    ["Where have you been?"] = "Gdzie ty byłeś?", -- A_Classic_Fairytale:united
    ["While in modification mode, you can change|the LandFlag by clicking on an object."] = "W trybie modyfikacji, możesz zmienić|Flagę Terenu, klikając na obiekt.", -- HedgeEditor
    ["White Tee"] = "Biały Cel", -- A_Space_Adventure:ice01
    ["Who's there?! I'll get you!"] = "Ko tam jest?! Dopadnę cię!", -- A_Space_Adventure:desert01
    ["Why are you doing this?"] = "Czemu to robisz?", -- A_Classic_Fairytale:journey
    ["Why are you helping us, uhm...?"] = "Czemu nam pomagasz, um...?", -- A_Classic_Fairytale:family
    ["Why can't he just let her go?!"] = "Czemu on nie może jej po prostu puścić?", -- A_Classic_Fairytale:family
    ["Why?"] = "Czemu?", -- A_Classic_Fairytale:queen
    ["… why did I risk my life to collect all the parts of the anti-gravity device?"] = "...czemu ryzykowałem moje życie, by zebrać wszystkie części urządzenia antygrawitacyjnego?", -- A_Space_Adventure:final
    ["Why did you do this?"] = "Czemu to zrobiłaś?", -- A_Classic_Fairytale:queen
    ["Why did you kill your father?"] = "Czemu zabiłaś swojego ojca?", -- A_Classic_Fairytale:queen
    ["Why do men keep hurting me?"] = "Czemu mężczyźni cały czas mnie krzywdzą?", -- A_Classic_Fairytale:first_blood
    ["Why do you always have to call me names?"] = "Czemu zawsze musisz mówić mi po imieniu?", -- A_Classic_Fairytale:queen
    ["Why do you keep betraying us?"] = "Czemu wciąż nas zdradzasz?", -- A_Classic_Fairytale:queen
    ["Why do you not like me?"] = "Czemu mnie nie lubisz?", -- A_Classic_Fairytale:shadow
    ["Why do you want to take over our island?"] = "Czemu chcecie przejąć naszą wyspę?", -- A_Classic_Fairytale:enemy
    ["Why me?!"] = "Czemu ja?!", -- A_Classic_Fairytale:backstab
    ["Why %s? Why?"] = "Czemu %s? Czemu?", -- A_Classic_Fairytale:backstab
    ["Why, why, why, why!"] = "Czemu, czemu, czemu, czemu!", -- A_Classic_Fairytale:queen
    ["Why would they do this?"] = "Czemu mieliby to zrobić?", -- A_Classic_Fairytale:backstab
    ["- Will get 1-3 random weapons"] = "- Dostanie 1-3 losowe bronie", -- Continental_supplies
    ["- Will Get 1-3 random weapons"] = "- Dostaniesz 1-3 losowych broni", -- Continental_supplies
    ["- Will give you a parachute every second turn."] = "- Da ci spadochron co drugą turę.", -- Continental_supplies
    ["Will this ever end?"] = "Czy to się kiedyś skończy?",
    ["Will you give me the other parts?"] = "Dasz mi resztę części?", -- A_Space_Adventure:death01
    ["Wind"] = "Wiatr", -- Basic_Training_-_Bazooka
    ["Winner: %s"] = "Zwycięzca: %s", -- Mutant
    ["Winning time: %s"] = "Wygrywający czas: %s", -- Racer, TechRacer
    ["Win"] = "Wygrana", -- A_Space_Adventure:ice01
    ["Wise Oak"] = "Mądry Dąb", -- A_Classic_Fairytale:backstab, A_Classic_Fairytale:dragon, A_Classic_Fairytale:enemy, A_Classic_Fairytale:epil, A_Classic_Fairytale:family, A_Classic_Fairytale:queen
    ["With Dense Cloud on the land of shadows, I'm the village's only hope..."] = "Z Gęstą Chmurą w krainie cieni, jestem jedyną nadzieją wioski...", -- A_Classic_Fairytale:journey
    ["With low bounciness, it barely bounces at all, but it is much more predictable."] = "Z niską odbijalnością, ledwo będzie się odbijać, ale jest dużo bardziej przewidywalny.", -- Basic_Training_-_Grenade
    ["With the rest of the tribe gone, it was up to %s to save the village."] = "Z resztą naszego plemienia nieobecną, jedynie %s mógł uratować wioskę.", -- A_Classic_Fairytale:dragon
    ["Worry not, for it is a peaceful animal! There is no reason to be afraid..."] = "Nie martw się, gdyż jest to pokojowe zwierzę! Nie ma powodu się obawiać...", -- A_Classic_Fairytale:first_blood
    ["Wow, what a dream!"] = "Wow, co za sen!", -- A_Classic_Fairytale:backstab
    ["Xeli"] = "Xeli",
    ["Xerxes"] = "Xerxes",
    ["Y3K1337"] = "Y3K1337", -- A_Classic_Fairytale:journey, A_Classic_Fairytale:shadow
    ["Yay, we won!"] = "Jej, wygraliśmy!", -- A_Classic_Fairytale:enemy
    ["Y Chwiliad"] = "Y Człiliad", -- A_Classic_Fairytale:dragon
    ["Yeah...I think it's a 'he', lol."] = "Racja... zdaje się, że to \"on\", lol.", -- A_Classic_Fairytale:shadow
    ["Yeah, sure! I died. Hilarious!"] = "Tak, racja! Umarłem. Przezabawne!", -- A_Classic_Fairytale:backstab
    ["Yeah, sure! I died. Hillarious!"] = "Racja, pewnie! Umarłem. Komiczne.", -- A_Classic_Fairytale:backstab
    ["Yeah, take that!"] = "Tak, przyjmij to!", -- A_Classic_Fairytale:dragon
    ["Yeah? Watcha gonna do? Cry?"] = "Tak? Co zamierzasz zrobić? Płakać?", -- A_Classic_Fairytale:journey
    ["Yeah, well, for some dude to be happy, some other dude has to suffer."] = "Racja, cóż, żeby jakiś koleś był szczęśliwy, inny koleś musi cierpieć.", -- A_Classic_Fairytale:queen
    ["Yellow Pepper"] = "Żółta Papryka", -- A_Space_Adventure:fruit01
    ["Yellow Watermelons"] = "Żółte Melony", -- A_Space_Adventure:fruit01
    ["Yellow"] = "Żółty",
    ["Yes, but you're … different!"] = "Tak, ale ty jesteś... inny!", -- A_Classic_Fairytale:queen
    ["Yes!"] = "Tak!", -- A_Classic_Fairytale:enemy
    ["Yes, yeees! You are now ready to enter the real world!"] = "Tak, taaak! Jesteś teraz gotowy, by wejść w prawdziwy świat!", -- A_Classic_Fairytale:first_blood
    ["Yeti"] = "Yeti",
    ["Yikes!"] = "Jejciu!", -- A_Space_Adventure:desert01
    ["Yo, dude! Get away from our weapons!"] = "Hej, ziom! Odejdź od naszych broni!", -- A_Classic_Fairytale:queen
    ["Yo, dude, we're here, too!"] = "Hej, ziom, my też tu jesteśmy!", -- A_Classic_Fairytale:family
    ["Yo, escort my buttocks!"] = "Ej ty, eskortuj moje pośladki!", -- A_Classic_Fairytale:queen
    ["Yoshi"] = "Yoshi",
    ["Yo, the aliens gave me plants. Medicinal plants. Lots of it."] = "Hej, obcy dali mi rośliny. Medyczne rośliny. Dużo roślin.", -- A_Classic_Fairytale:queen
    ["You are far from home, and the water is rising, climb up as high as you can!|Your score will be based on your height."] = "Jesteś daleko od domu, a woda się podwyższa. Wspinaj się najwyżej jak potrafisz!|Twój wynik będzie zależeć od twojej wysokości.", -- ClimbHome
    ["You are given the chance to turn your life around..."] = "Masz szansę odwrócić swoje życie...", -- A_Classic_Fairytale:shadow
    ["You are in control of all the active ally units."] = "Sprawujesz kontrolę nad wszystkimi aktywnymi sprzymierzonymi jednostkami.", -- A_Space_Adventure:fruit01
    ["You are indeed the best PAotH pilot."] = "Jesteś doprawdy najlepszym pilotem PSJ.", -- A_Space_Adventure:desert03
    ["You are out of danger, time to go to the moon!"] = "Jesteś poza niebezpieczeństwem, czas lecieć na księżyc!", -- A_Space_Adventure:cosmos
    ["You are playing with our lives here!"] = "Bawisz się tu naszymi życiami!", -- A_Classic_Fairytale:enemy
    ["You are sabotaged, RUN!"] = "Jesteś sabotowany, UCIEKAJ!", -- Continental_supplies
    ["You are the one who fled! So, you are alive."] = "Ty jesteś tym, który uciekł! Więc, przeżyłeś.", -- A_Space_Adventure:fruit02
    ["You bear impressive skills, %s!"] = "Posiadasz imponujące zdolności, %s!", -- A_Classic_Fairytale:dragon
    ["You can also hold down the key for “Precise Aim” to prevent slipping."] = "Możesz także przytrzymać przycisk “Precyzyjnego celowania”, by zapobiec źlizganiu się.", -- Basic_Training_-_Movement
    ["You can always trust me! I love you!"] = "Zawsze możesz mi ufać! Kocham cię!", -- A_Classic_Fairytale:epil
    ["You can always trust me!"] = "Zawsze możesz mi ufać!", -- A_Classic_Fairytale:epil
    ["You can avoid some battles."] = "Możesz uniknąć niektórych bitw.", -- A_Space_Adventure:desert01
    ["You can change the detonation timer of grenades."] = "Możesz zmienić zapalnik detonacyjny granatów.", -- Basic_Training_-_Grenade
    ["You can choose another planet by replaying this mission."] = "Możesz wyrać inną planetę, grając w tę misję jeszcze raz.", -- A_Space_Adventure:cosmos
    ["You can dive with your flying saucer!"] = "Możesz nurkować swoim spodkiem!", -- Basic_Training_-_Flying_Saucer
    ["You can even change your aiming direction in mid-flight if you first hold [Precice] and then press [Up] or [Down]."] = "Możesz zmieniać kierunek celowania w locie, jeśli najpierw przytrzymasz [Precyzja], a potem wciśniesz [Góra] lub [Dół].", -- Basic_Training_-_Flying_Saucer
    ["You can even change your aiming direction in mid-flight if you first hold [Precise] and then press [Up] or [Down]."] = "Możesz nawet zmienić swój kierunek celowania w czasie lotu, jeśli wpierw przytrzymasz [Precyzję] i wciśniesz [Góra] lub [Dół]", -- Basic_Training_-_Flying_Saucer
    ["You can further customize the race by changing the scheme script paramater."] = "Możesz jeszcze dostosować swój wyścig, zmieniając parametr skryptu w schemacie.", -- TechRacer
    ["You can further customize the race by changing the scheme script parameter."] = "Możesz bardziej dostosować wyścig, zmienając parametr skryptu w schemacie.", -- TechRacer
    ["You can only use the sniper rifle or the watermelon bomb."] = "Możesz użyć tylko karabinu snajperskiego lub bomby arbuzowej.", -- A_Space_Adventure:fruit03
    ["You can practice moving around and using utilities in this mission.|However, it will never end!"] = "W tej misji możesz poćwiczyć chodzenie i używanie narzędzi.|Jednakże, ona nigdy się nie skończy!", -- A_Classic_Fairytale:epil
    ["You can set the bounciness of grenades (and grenade-like weapons)."] = "Możesz ustawić odbijalnoś granatów (i granato-podobnych broni).", -- Basic_Training_-_Grenade
    ["- You can switch between hogs at the start of your turns. (Not first one)"] = "- Możesz zmieniać między jeżami na początku swoich tur. (Nie tej pierwszej)", -- Continental_supplies
    ["You can’t open a portal on the blue surface."] = "Nie możesz otworzyć portalu na niebieskiej powierzchni.", -- portal
    ["You can use the other 2 hogs to assist you."] = "Możesz użyć 2 innych jeży, by ci pomogli.", -- A_Space_Adventure:fruit02
    ["You can use the rope to reach new places."] = "Możesz użyć liny, by dostać się do nowych miejsc.", -- Basic_Training_-_Rope
    ["You choose well, %s!"] = "Wybierz mądrze, %s!", -- A_Space_Adventure:fruit01
    ["You completed the mission in %.3f seconds."] = "Ukończyłeś tę misję w %.3f sekund.", -- A_Space_Adventure:ice02
    ["You completed the mission in %d rounds."] = "Ukończyłeś tę misję w %d rund.", -- A_Space_Adventure:death02, A_Space_Adventure:fruit03
    ["You couldn't have come to a worse time, %s!"] = "Nie mogłeś przyjść o gorszej porze, %s!", -- A_Space_Adventure:fruit01
    ["You couldn't possibly believe that after refusing my offer I'd just let you go!"] = "Nie mógłbyś chyba wierzyć, że po odrzuceniu mojej oferty po prostu puściłbym cię wolno!", -- A_Classic_Fairytale:journey
    ["You'd almost swear the water was rising!"] = "Prawie byś przysiągł, że woda zdaje się podnosić!",
    ["You'd better watch your steps..."] = "Lepiej patrz po nogi...", -- A_Classic_Fairytale:journey
    ["You defended yourself against Captain Lime."] = "Obroniłeś się przed Kapitanem Limonką.", -- A_Space_Adventure:fruit02
    ["You defended yourself against %s."] = "Broń się przeciwko %s.", -- A_Space_Adventure:fruit02
    ["You did great, %s! However, we aren't out of danger yet!"] = "Dobrze sobie poradziłeś, %s! Jednakże, wciąż jesteśmy zagrożeni!", -- A_Space_Adventure:cosmos
    ["You did not make it in time, try again!"] = "Nie udało ci się na czas, spróbuj ponownie!", -- Basic_Training_-_Rope
    ["You don't deserve my sacrifice!"] = "Nie zasługujesz na moje poświęcenie!", -- A_Classic_Fairytale:queen
    ["You drove Professor Hogevil away."] = "Odpędziłeś Profesora Jeżozło.", -- A_Space_Adventure:moon01
    ["You drove the minions away."] = "Odpędziłeś sługusów.", -- A_Space_Adventure:moon01
    ["You earned the \"Rope Master\" achievement for finishing in under 50 seconds."] = "Zdobyłeś osiągnięcie \"Mistrz Liny\" za ukończenie poniżej 50 sekund.", -- Basic_Training_-_Rope
    ["You endangered your whole tribe, you bastard!"] = "Naraziłeś całe swoje plemię, ty draniu!", -- A_Classic_Fairytale:queen
    ["You failed!"] = "Ponosisz porażkę!", -- Basic_Training_-_Rope
    ["You failed to kill all enemies in a single turn."] = "Nie udało ci się zabić wszystkich wrogów w jednej turze.", -- Big_Armory
    ["You failed to kill all enemies in this turn."] = "Nie udało ci się zabić wszystkich wrogów w tej turze.", -- Big_Armory
    ["You fought bravely and you helped us win this battle!"] = "Walczyłeś dzielnie i pomogłeś nam wygrać tę bitwę!", -- A_Space_Adventure:fruit02
    ["You give me no choice!"] = "Nie pozostawiasz mi wyboru!", -- A_Classic_Fairytale:queen
    ["You got a killer mask there, amigo!"] = "Masz tu zabójczą maskę, amigo!", -- A_Classic_Fairytale:epil
    ["You got me!"] = "Masz mnie!", -- A_Space_Adventure:moon02
    ["You had %.1fs remaining on the clock (+%d points)."] = "Pozostało ci na zegarze %.1fs sekund (+%d punktów).", -- TargetPractice
    ["You had %.2fs remaining on the clock (+%d points)."] = "Pozostało ci na zegarze %.2fs sekund (+%d punktów).", -- Basic_Training_-_Sniper_Rifle
    ["You have 7 turns until the next wave arrives.|Make sure the arriving cannibals are greeted appropriately!|If the hog dies, the cause is lost.|Hint: You might want to use some mines ..."] = "Masz 7 tur, zanim następna fala nadejrzie.|Upewnij się, że nadchodzące kanibale są odpowiednio powitane!|Jeśli umrze jeż, sprawa jest stracona.|Wskazówka: Możesz chcieć użyć paru min...", -- A_Classic_Fairytale:backstab
    ["You have 7 turns until the next wave arrives.|Make sure the arriving cannibals are greeted appropriately!|If the hog dies, the cause is lost.|Hint: you might want to use some mines..."] = "Masz 7 tur, zanim przybędzie następna fala.|Upewnij się, że przybywający kanibale zostaną odpowiednio powitani!|Jeśli jeż umrze, sprawa będzie przegrana.|Porada: możesz chcieć użyć paru min...", -- A_Classic_Fairytale:backstab
    ["You have acquired the last device part."] = "Zdobyłeś ostatnią część urządzenia.", -- A_Space_Adventure:death01
    ["You have activated Switch Hedgehog!"] = "Aktywowałeś Zmianę Jeża!", -- Basic_Training_-_Movement
    ["You have beaten the challenge!"] = "Pokonałeś wyzwanie!", -- ClimbHome
    ["You have beaten the team record, congratulations!"] = "Pobiłeś rekord drużyny, gratulacje!", -- Utils
    ["You have been giving us out to the enemy, haven't you!"] = "Wydawałeś nas wrogom, co nie!", -- A_Classic_Fairytale:backstab
    ["You have chosen the perfect moment to leave."] = "Wybrałeś idealny moment, by odejść.", -- A_Classic_Fairytale:united
    ["You have chosen to fight!"] = "Wybrałeś bitwę!", -- A_Space_Adventure:fruit01
    ["You have chosen to flee."] = "Wybrałeś ucieczkę.", -- A_Space_Adventure:fruit01
    ["You have collected %d out of %d crate(s)."] = "Zebrałeś %d z %d skrzyń.", -- SpeedShoppa
    ["You have collected the “Switch Hedgehog” utility!"] = "Zebrałeś narzędzie “Zmiany Jeża”!", -- Basic_Training_-_Movement
    ["You have completed the Basic Bazooka Training!"] = "Ukończyłeś trening podstaw korzystania z bazooki!", -- Basic_Training_-_Bazooka
    ["You have completed the Basic Grenade Training!"] = "Ukończyłeś trening podstaw korzystania z granatu!", -- Basic_Training_-_Grenade
    ["You have completed the Basic Movement Training!"] = "Ukończyłeś trening podstaw poruszania się!", -- Basic_Training_-_Movement
    ["You have completed this challenge in %.2f s (+%d points)."] = "Ukończyłeś to wyzwanie w %.2fs (+%d punktów).", -- User_Mission_-_Rope_Knock_Challenge
    ["You have destroyed all targets!"] = "Zniszczyłeś wszystkie cele!", -- TargetPractice
    ["You have destroyed all the targets."] = "Zniszczyłeś wszystkie cele.", -- A_Space_Adventure:desert03
    ["You have destroyed %d of %d targets (+%d points)."] = "Zniszczyłeś %d z %d celów (+%d punktów)", -- Basic_Training_-_Sniper_Rifle, TargetPractice
    ["You have destroyed %d of %d targets."] = "Zniszczyłeś %d z %d celów.", -- Basic_Training_-_Bazooka
    ["You have dropped %d missiles."] = "Upuściłeś %d rakiet.", -- User_Mission_-_RCPlane_Challenge
    ["You have eliminated all visible enemy hedgehogs!"] = "Wyeliminowałeś wszystkie widoczne wrogie jeże!", -- A_Space_Adventure:fruit01
    ["You have eliminated Professor Hogevil."] = "Wyeliminowałeś Profesora Jeżozło.", -- A_Space_Adventure:moon01
    ["You have eliminated the evil minions."] = "Wyeliminowałeś złych sługusów.", -- A_Space_Adventure:moon01
    ["You have escaped successfully."] = "Pomyślnie uciekłeś.", -- A_Space_Adventure:desert02
    ["You have failed to complete your task, young one!"] = "Nie udało ci się ukończyć twojego zadania, młodzieńcze!", -- A_Classic_Fairytale:journey
    ["You have failed to save the tribe!"] = "Nie udało ci się uratować plemienia!", -- A_Classic_Fairytale:backstab
    ["You have finally figured it out!"] = "Nareszcie to wykombinowałeś!", -- A_Classic_Fairytale:enemy
    ["You have finished the Basic Rope Training!"] = "Ukończyłeś podstawowy trening linowy!", -- Basic_Training_-_Rope
    ["You have finished the bazooka training!"] = "Ukończyłeś trening bazookowy!", -- Basic_Training_-_Bazooka
    ["You have finished the challenge in %.3f s."] = "Ukończyłeś wyzwanie w %.3fs.", -- SpeedShoppa
    ["You have finished the challenge!"] = "Ukończyłęś wyzwanie!", -- User_Mission_-_RCPlane_Challenge
    ["You have finished the Flying Saucer Training!"] = "Ukończyłeś trening latającego spodka!", -- Basic_Training_-_Flying_Saucer
    ["You have finished the target practice!"] = "Ukończyłeś ćwiczenie celności!", -- TargetPractice
    ["You have kidnapped our whole tribe!"] = "Porwałeś całe nasze plemię!", -- A_Classic_Fairytale:enemy
    ["You have killed all enemies."] = "Zabiłeś wszystkich wrogów.", -- Big_Armory
    ["You have killed an innocent hedgehog!"] = "Zabiłeś niewinnego jeża!", -- A_Classic_Fairytale:backstab
    ["You have killed %d of 16 hedgehogs (+%d points)."] = "Zabiłeś %d z 16 jeży (+%d punktów)", -- User_Mission_-_Rope_Knock_Challenge
    ["You have launched %d bazookas."] = "Wystrzeliłeś %d bazook.", -- Target_Practice_-_Bazooka_easy, Target_Practice_-_Bazooka_hard, Basic_Training_-_Bazooka
    ["You have launched %d homing bees."] = "Wystrzeliłeś %d pszczół.", -- Target_Practice_-_Homing_Bee
    ["You have made %d shots."] = "Wykonałeś %d strzałów.", -- Basic_Training_-_Sniper_Rifle
    ["You have managed to catch the blue hedgehog in %.3f seconds."] = "Udało ci się złapać niebieskiego jeża w %.3f sekund.", -- A_Space_Adventure:moon02
    ["You have never worked a bit in your life!"] = "Nigdy nie pracowałaś ani trochę w swoim życiu!", -- A_Classic_Fairytale:queen
    ["You have nothing to be afraid of now."] = "Teraz nie masz się czego bać.", -- A_Classic_Fairytale:epil
    ["You haven't rescued anyone."] = "Nikogo nie uratowałeś.", -- User_Mission_-_That_Sinking_Feeling
    ["You have obtained an achievement: Lively Lifeguard"] = "Zdobyłeś osiągnięcie: Żwawy Ratownik", -- User_Mission_-_That_Sinking_Feeling
    ["You have perfectly beaten the challenge!"] = "Perfekcyjnie pokonałeś wyzwanie!", -- User_Mission_-_RCPlane_Challenge
    ["You have proven yourself worthy to see our most ancient secret!"] = "Udowodniłeś, że jesteś godny, by zobaczyć nasz najbardziej starożytny sekret!", -- A_Classic_Fairytale:first_blood
    ["You have proven yourselves worthy!"] = "Udowodniliście, że jesteście godni!", -- A_Classic_Fairytale:enemy
    ["You have reached the take-off area successfully!"] = "Dotarłeś pomyślnie do strefy odlotu!", -- A_Space_Adventure:fruit01
    ["You have rescued H and Dr. Cornelius."] = "Uratowałeś J i Dr Korneliusza.", -- A_Space_Adventure:death01
    ["You have SCORED!!"] = "Zdobyłeś PUNKT!!",
    ["You have shot %d times."] = "Strzeliłeś %d razy.", -- TargetPractice
    ["You have successfully eliminated Professor Hogevil."] = "Pomyślnie wyeliminowałeś Profesora Jeżozło.", -- A_Space_Adventure:death01
    ["You have successfully finished the campaign!"] = "Pomyślnie ukończyłeś kampanię!", -- A_Classic_Fairytale:epil
    ["You have successfully finished the sniper rifle training!"] = "Pomyślnie ukończyłeś trening karabinem snajperskim!", -- Basic_Training_-_Sniper_Rifle
    ["You have thrown %d cluster bombs."] = "Wyrzuciłeś %d granatów odłamkowych.", -- Target_Practice_-_Cluster_Bomb
    ["You have thrown %d grenades."] = "Wyrzuciłeś %d granatów.", -- Target_Practice_-_Grenade_easy, Target_Practice_-_Grenade_hard
    ["You have to be careful and must not die!"] = "Musisz być ostrożny i nie możesz umrzeć!", -- A_Space_Adventure:cosmos
    ["You have to catch the other hog 3 times."] = "Musisz złapać tamtego jeża 3 razy.", -- A_Space_Adventure:moon02
    ["You have to complete the main mission on moon in order to travel to other planets."] = "Musisz ukończyć główną misję na księżycu, by podróżować na inne planety.", -- A_Space_Adventure:cosmos
    ["You have to continue alone from now on."] = "Od teraz musisz kontynuować samemu.", -- A_Space_Adventure:cosmos
    ["You have to destroy all the explosives without dying!"] = "Musisz zniszczyć wszystkie ładunki wybuchowe bez umierania!", -- A_Space_Adventure:final
    ["You have to destroy all the targets."] = "Musisz zniszczyć wszystkie cele.", -- A_Space_Adventure:desert03
    ["You have to destroy the target above by dropping a grenade on it from your flying saucer."] = "Musisz zniszczyć cel powyżej, upuszczając na niego granat z twojego latającego talerza.", -- Basic_Training_-_Flying_Saucer
    ["You have to destroy two targets, but the previous technique would be very difficult or dangerous to use."] = "Musisz zniszczyć dwa cele, ale poprzednia technika byłaby bardzo trudna lub niebezpieczna do użycia.", -- Basic_Training_-_Flying_Saucer
    ["You have to drop the grenade from rope!"] = "Musisz upuścić granat z liny!", -- Basic_Training_-_Rope
    ["You have to eliminate all the enemies."] = "Musisz wyeliminować wszystkich wrogów.", -- A_Space_Adventure:death02, A_Space_Adventure:fruit03
    ["You have to eliminate all the visible enemies."] = "Musisz wyeliminować wszystkich widocznych wrogów.", -- A_Space_Adventure:fruit01
    ["You have to get the weapons and rescue the PAotH researchers."] = "Musisz zdobyć bronie i uratować naukowców PSJ.", -- A_Space_Adventure:moon01
    ["You have to get to the left-most land and remove any enemy hog from there."] = "Musisz dostać się do lądu po lewej i usunać stamtąd każdego wroga.", -- A_Space_Adventure:fruit01
    ["You have to go back to the moon!"] = "Musisz wrócić na księżyc!", -- A_Space_Adventure:cosmos
    ["You have to move upwards, not downwards, %s!"] = "Musisz poruszać się w górę, nie w dół, %s!", -- ClimbHome
    ["You have to reach the left-most place on the map."] = "Musisz dostać się na lewą część mapy.", -- A_Space_Adventure:fruit01
    ["You have to stand very close to him"] = "Musisz stanąć bardzo blisko niego", -- A_Space_Adventure:moon02
    ["You have to travel again"] = "Musisz podróżować ponownie", -- A_Space_Adventure:cosmos
    ["You have to try again!"] = "Musisz spróbować ponownie!", -- A_Space_Adventure:cosmos
    ["You have triggered the secret Do-Not-Rope-to-the-Moon Defense System."] = "Uruchomiłeś sekretny Nie-Używaj-Liny-Na-Księżyc System Obronny.", -- A_Space_Adventure:cosmos
    ["You have unlocked the target radar!"] = "Odblokowałeś radar celów!", -- TargetPractice
    ["You have used %d flying saucers."] = "Użyłeś %d latających talerzy.", -- A_Space_Adventure:ice02
    ["You have used %d RC planes."] = "Użyłeś %d radiowych samolocików.", -- User_Mission_-_RCPlane_Challenge
    ["You have used only 1 RC plane. Outstanding!"] = "Użyłeś tylko 1 radiowy samolocik. Wybitnie!", -- User_Mission_-_RCPlane_Challenge
    ["You have violated PAotH regulations!"] = "Naruszyłeś regulacje PSJ!", -- A_Space_Adventure:cosmos
    ["You have won the game by proving true cooperative skills!"] = "Wygrałeś grę, udowadniając prawdziwe zdolności współpracy!", -- A_Classic_Fairytale:enemy
    ["You just appeared out of thin air!"] = "Właśnie pojawiłeś się znikąd!", -- A_Classic_Fairytale:backstab
    ["You just can't let it go, can you!"] = "Po prostu nie możesz odpuścić, co!", -- A_Classic_Fairytale:queen
    ["You just committed suicide..."] = "Właśnie popełniłeś samobójstwo...", -- A_Classic_Fairytale:shadow
    ["You just got yourself some extra health.|The more health your hedgehogs have, the better!"] = "Właśnie sobie zdobyłeś trochę dodatkowego zdrowia.|Im więcej zdrowia ma twój jeż, tym lepiej!", -- Basic_Training_-_Movement
    ["You killed my father, you monster!"] = "Zabiłeś mojego ojca, ty potworze!", -- A_Classic_Fairytale:backstab
    ["You know...taking a stroll."] = "No wiesz... udanie się na przechadzkę.", -- A_Classic_Fairytale:backstab
    ["You know what? I don't even regret anything!"] = "Wiesz co? Nawet nic nie żałuję!", -- A_Classic_Fairytale:backstab
    ["You'll get an extra sniper rifle every time you kill an enemy hog with a limit of max 4 rifles."] = "Dostaniesz dodatkowy karabin snajperski za każdym razem, gdy zabijesz wrogiego jeża, z limitem maksymalnie 4 karabinów.", -- A_Space_Adventure:fruit03
    ["You'll get an extra teleport every time you kill an enemy hog with a limit of max 2 teleports."] = "Dostaniesz dodatkowy teleport za każdym razem, gdy zabijesz wrogiego jeża, z limitem maksymalnie 4 teleportów.", -- A_Space_Adventure:fruit03
    ["You'll get extra time in case you need it when you pass a ring."] = "Dostaniesz dodatkowy czas, na wypadek gdybyś go potrzebował, gdy przejdziesz pierścień.", -- A_Space_Adventure:ice02
    ["You'll have only 2 watermelon bombs during the game."] = "Będziesz miał tylko 2 arbuzowe bomby podczas tej gry.", -- A_Space_Adventure:fruit03
    ["You'll have only one RC plane at the start of the mission."] = "Będziesz miał tylko jeden radiowy samolocik na początku misji.", -- A_Space_Adventure:desert03
    ["You'll have to eliminate Captain Lime at the end."] = "Będziesz musiał na koniec wyeliminować Kapitana Limonkę.", -- A_Space_Adventure:fruit02
    ["You'll have to eliminate %s at the end."] = "Będziesz musiał na końcu weliminować %s.", -- A_Space_Adventure:fruit02
    ["You'll lose if you die or if your time is up."] = "Przegrasz, jeśli zginiesz lub skończy ci się czas.", -- A_Space_Adventure:moon02
    ["You'll see what I mean!"] = "Zobaczysz, co mam na myśli!", -- A_Classic_Fairytale:enemy
    ["You lost your target, try again!"] = "Straciłeś swój cel, spróbuj ponownie!", -- TargetPractice
    ["You may find it handy."] = "Może ci się przydać.", -- A_Space_Adventure:cosmos
    ["You may only attack from a rope!"] = "Możesz atakować tylko z liny!", -- WxW
    ["You may only place 1 Extra Time crate per turn."] = "Możesz postawić tylko 1 skrzynię z Dodatkowym Czasem na turę.", -- Construction_Mode
    ["You may only place %d crates per round."] = "Możesz postawić tylko %d skrzyń na turę.", -- Construction_Mode
    ["- You may only score when your flag is in your base"] = "- Możesz zapunktować tylko, gdy twoja flaga jest w twojej bazie.", -- Capture_the_Flag
    ["You meatbags are pretty slow, you know!"] = "Wy mięsne worki jesteście całkiem wolni, wiecie!", -- A_Classic_Fairytale:enemy
    ["You might want to find a way to instantly kill arriving cannibals!"] = "Możesz chcieć znaleźć sposób, by natychmiastowo zabić przybywających kanibali!", -- A_Classic_Fairytale:backstab
    ["You must attack from a rope, after you collected a crate!"] = "Musisz zaatakować z liny, po tym jak zbierzesz skrzynię!", -- WxW
    ["You must first collect a crate before you attack!"] = "Musisz najpierw zdobyć skrzynię, zanim zaatakujesz!", -- WxW
    ["You must survive the flood in order to score."] = "Musisz przeżyć powódź, żeby zapunktować.", -- User_Mission_-_That_Sinking_Feeling
    ["You never give me plants!"] = "Nigdy nie dajesz mi roślin!", -- A_Classic_Fairytale:queen
    ["Young one, you are telling us that they can instantly change location without a shaman?"] = "Młodzieńcze, mówisz nam, że mogą oni natychmiastowo zmienić lokację, bez szamana?", -- A_Classic_Fairytale:united
    ["You now have infinite fuel, grenades and bazookas for fun."] = "Masz teraz nieskończona paliwo, granaty i bazooki, dla zabawy.", -- Basic_Training_-_Flying_Saucer
    ["You only get 1 rope this time, don't waste it!"] = "Dostajesz tym razem tylko jedną linę, nie zmarnuj jej!", -- Basic_Training_-_Rope
    ["You only have 2 flying saucers this time."] = "Masz tym razem tylko 2 latające talerze.", -- Basic_Training_-_Flying_Saucer
    ["You only have one flying saucer this time."] = "Masz tym razem tylko 1 latający talerz.", -- Basic_Training_-_Flying_Saucer
    ["You probably know what to do next..."] = "Prawdopodobnie wiesz, co dalej robić...", -- A_Classic_Fairytale:first_blood
    ["Your accuracy was %.1f%% (+%d points)."] = "Twoja celność wynosiła %.1f%% (+%d punktów).", -- TargetPractice
    ["Your accuracy was %.1f%%."] = "Twoja celność wynosiła %.1f%%.", -- Basic_Training_-_Bazooka, TargetPractice
    ["Your ammo is limited this time."] = "Twoja amunicja jest teraz ograniczona.", -- Basic_Training_-_Bazooka
    ["Your deaths will be avenged, %s!"] = "Wasze śmierci zostaną pomszczone, %s!", -- A_Classic_Fairytale:enemy
    ["Your death will not be in vain, Dense Cloud!"] = "Twoja śmierć nie pójdzie na marne, Gęsta Chmuro!", -- A_Classic_Fairytale:shadow
    ["You're a coward!"] = "Jesteś tchórzem!", -- A_Classic_Fairytale:queen
    ["You're...alive!? But we saw you die!"] = "Ty... żyjesz?! Ale widzieliśmy jak umierasz!", -- A_Classic_Fairytale:backstab
    ["You're a pathetic liar!"] = "Jesteś żałosnym kłamcą!", -- A_Classic_Fairytale:backstab
    ["You're funny!"] = "Jesteś zabawny!", -- A_Classic_Fairytale:journey
    ["You're getting pretty good! |Tip: When you shorten you rope, you move faster!|And when you lengthen it, you move slower."] = "Stajesz się całkiem dobry!|Porada: Kiedy skrócisz swoją linę, poruszasz się szybciej!|A kiedy ją wydłużysz, poruszasz się wolniej.", -- Basic_Training_-_Rope
    ["You're on your way to freeing your tribe!"] = "Jesteś na swojej drodze, by uwolnić twoje plemię!", -- A_Classic_Fairytale:queen
    ["You're pathetic! You are not worthy of my attention..."] = "Jesteś żałosny! Nie jesteś godzien mojej uwagi...", -- A_Classic_Fairytale:shadow
    ["You're probably wondering why I bought you back..."] = "Pewnie zastanawiacie się, dlaczego sprowadziłem was z powrotem...", -- A_Classic_Fairytale:backstab
    ["You're probably wondering why I brought you back ..."] = "Prawdopodobnie się zastanawiasz dlaczego cię sprowadziłem z powrotem...", -- A_Classic_Fairytale:backstab
    ["Your escape took you %d turns."] = "Twoja ucieczka zajęła ci %d tur.", -- A_Space_Adventure:desert02
    ["You're so brave! I feel safe with you."] = "Jesteś taki odważny! Czuję się przy tobie bezpieczna.", -- A_Classic_Fairytale:epil
    ["You're some piece of hypocrite junkie!"] = "Jesteś kawałkiem hipokryckiego ćpuna!", -- A_Classic_Fairytale:queen
    ["You're terrorizing the forest...We won't catch anything like this!"] = "Terroryzujesz las... Nie złapiemy nic takiego!", -- A_Classic_Fairytale:shadow
    ["You retrieved the lost part."] = "Odzyskałeś straconą część.", -- A_Space_Adventure:fruit02
    ["Your fastest escape so far: %d turns"] = "Twoja najszybsza ucieczka jak dotąd: %d tur", -- A_Space_Adventure:desert02
    ["Your fastest victory so far: %d rounds"] = "Twoja najszybsza wygrana jak dotąd: %d rund", -- A_Space_Adventure:death02, A_Space_Adventure:fruit03
    ["Your first destination is the moon in order to get more fuel."] = "Twój pierwszy cel to księżyc, żeby zdobyć więcej paliwa.", -- A_Space_Adventure:cosmos
    ["Your hedgehog died!"] = "Twój jeż umarł!", -- User_Mission_-_That_Sinking_Feeling
    ["Your hedgehog has been revived!"] = "Twój jeż został ożywiony!", -- Basic_Training_-_Bazooka, Basic_Training_-_Grenade, Basic_Training_-_Movement, Basic_Training_-_Rope
    ["Your hedgehog was panicly afraid of the water and decided to go in a safe distance of %d from it."] = "Twój jeż panicznie obawiał się wody i zdecydował udać się na bezpieczną odległość %d od niej.", -- ClimbHome
    ["Your height over time"] = "Twoja wysokość ponad czasem", -- ClimbHome
    ["Your hogs must survive!"] = "Twoje jeże muszą przeżyć!", -- A_Classic_Fairytale:journey
    ["Your movement skills will be evaluated now."] = "Twoje zdolności ruchowe zostaną teraz ocenione.", -- A_Classic_Fairytale:first_blood
    ["Your next task is to collect some crates by using the rope!"] = "Twoje następne zadanie to zebrać trochę skrzyń, używając liny!", -- A_Classic_Fairytale:first_blood
    ["Your personal best time so far: %.3f seconds"] = "Twój osobisty najlepszy czas jak dotąd: %.3f sekund", -- A_Space_Adventure:ice02, A_Space_Adventure:moon02
    ["Your rank: %s"] = "Twoja ranga: %s", -- User_Mission_-_RCPlane_Challenge
    ["Your rope is gone! Try again!"] = "Twoja lina się skończyła! Spróbuj ponownie!", -- Basic_Training_-_Rope
    ["You saved %d of 8 hegehogs."] = "Uratowałeś %d z 8 jeży.", -- User_Mission_-_That_Sinking_Feeling
    ["You see, hedgehog spikes are very, very valuable."] = "Widzisz, kolce jeża są bardzo, bardzo cenne.", -- A_Classic_Fairytale:queen
    ["You see the wind strength at the bottom right corner."] = "Możesz zobaczyć siłę wiatru w prawym dolnym rogu.", -- Basic_Training_-_Bazooka
    ["You see the wind strength at the top."] = "Na górze widzisz siłę wiatru.", -- Basic_Training_-_Bazooka
    ["You should have known that we don't rely on meatbags!"] = "Powinienieś wiedzieć, że nie polegamy na mięsnych workach!", -- A_Classic_Fairytale:queen
    ["You should know this more than anyone, Leaks!"] = "Powinieneś to wiedzieć lepiej niż wszyscy, Przecieku!", -- A_Classic_Fairytale:queen
    ["You speak great truth, Hannibal. Here, take a sip!"] = "Mówisz wielką prawdę, Hannibalu. Masz, weź łyka!", -- A_Classic_Fairytale:epil
    ["You've been assaulting us, we have been just defending ourselves!"] = "Najeżdżaliście nas, my tylko się broniliśmy!", -- A_Classic_Fairytale:enemy
    ["You've reached the goal!| |Time: "] = "Dotarłeś do celu!| |Czas: ",
    ["You will be avenged!"] = "Zostaniesz pomszczony!", -- A_Classic_Fairytale:shadow
    ["You will fail if you run out of ammo and there are still targets available."] = "Oblejesz, jeśli skończy ci się amunicja i zostaną dostępne cele.", -- A_Space_Adventure:desert03
    ["You will gain some extra ammo from the crates the next time you play the \"Getting to the device\" mission."] = "Dostaniesz trochę dodatkowej amunicji ze skrzyń następnym razem, gdy zagrasz w misję \"Dojście do urządzenia\".", -- A_Space_Adventure:fruit03
    ["You will play every 3 turns."] = "Będziesz grał co każde 3 tury.", -- A_Space_Adventure:fruit01
    ["- You will recieve 2-4 weapons on each kill! (Even on own hogs)"] = "- Otrzymasz 2-3 bronie za każde zabójstwo! (Nawet swoich jeży)", -- Continental_supplies
    ["You won't believe what happened to me!"] = "Nie uwierzysz, co mi się przytrafiło!", -- A_Classic_Fairytale:backstab
    ["Yuck! I bet they'll keep worshipping her even after I save the village!"] = "Fuj! Założę się, że dalej będą ją czcić, nawet jak uratuję wioskę.", -- A_Classic_Fairytale:family
    ["Yumme Gunpowder"] = "Pyszny Proch Strzelniczy",
    ["Zealandia"] = "Zelandia", -- Continental_supplies
    ["Zombie"] = "Zombie",
    ["Zombi"] = "Zombi", -- portal
    ["'Zooka Team"] = "Bazookinierzy",
    ["Zoom: [Pinch] with 2 fingers"] = "Przybliż: [Uszczypnij] dwoma palcami", -- Basic_Training_-_Movement
    ["Zoom: [Rotate mouse wheel]"] = "Przyybliż: [Obróć kółkiem myszy]", -- Basic_Training_-_Movement
    ["Zork"] = "Zork", -- A_Classic_Fairytale:dragon, A_Classic_Fairytale:family, A_Classic_Fairytale:queen
}
